@@ -18,13 +18,5 @@
  */
 
 int ivtv_audio_set_io(struct ivtv *itv);
-void ivtv_audio_set_volume(struct ivtv *itv, int volume);
-void ivtv_audio_set_mute(struct ivtv *itv, int mute);
-int ivtv_audio_get_volume(struct ivtv *itv);
-int ivtv_audio_get_mute(struct ivtv *itv);
-void ivtv_audio_freq_changed(struct ivtv *itv);
-int ivtv_audio_set_matrix(struct ivtv *itv, struct ivtv_msp_matrix *matrix);
-void ivtv_audio_set_std(struct ivtv *itv);
-void ivtv_audio_set_audmode(struct ivtv *itv, u32 audmode);
-u32 ivtv_audio_get_rxsubchans(struct ivtv *itv, struct v4l2_tuner *vt);
+void ivtv_audio_set_route(struct ivtv *itv, struct v4l2_routing *route);
 void ivtv_audio_set_audio_clock_freq(struct ivtv *itv, u8 freq);
