@@ -34,9 +34,6 @@
  * AVerMedia M179 GPIO info by Chris Pinkham <cpinkham@bc2va.org>
  *                using information provided by Jiun-Kuei Jung @ AVerMedia.
  */
-#ifndef MODULE
-#define MODULE
-#endif /* MODULE */
 
 #include "ivtv-compat.h"
 
@@ -62,15 +59,18 @@
 #include <asm/semaphore.h>
 #include <asm/system.h>
 
+#include <media/v4l2-common.h>
+#include <media/tuner.h>
+#include "cx2341x.h"
+
 #define IVTV_INTERNAL
 #include "ivtv.h"
 
-#include "tuner.h"
-#ifdef LINUX26
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 #include <linux/workqueue.h>
 #else
 #include <linux/tqueue.h>
-#endif /* LINUX26 */
+#endif
 
 #ifdef CONFIG_LIRC_I2C
 #  error "This driver is not compatible with the LIRC I2C kernel configuration option."
@@ -99,28 +99,43 @@
 #define IVTV_YUV_BUFFER_OFFSET_3  0x00370000	/* Forth YUV Buffer */
 #define IVTV_YUV_BUFFER_UV_OFFSET 0x65400	/* Offset to UV Buffer */
 
-extern u32 yuv_offset[4];
+extern const u32 yuv_offset[4];
 
+// Maximum ivtv driver instances.
 // Based on 6 PVR500s each with two PVR15s...
+// TODO: make this dynamic. I believe it is only a global in order to support
+// ivtv-fb. There must be a better way to do that.
 #define IVTV_MAX_CARDS 12
 
-#define IVTV_CARD_PVR_250 0	/* wintv pvr 250 */
-#define IVTV_CARD_PVR_350 1	/* encoder, decoder, tv-out */
-#define IVTV_CARD_M179    2	/* AVerMedia M179 (encoder only) */
-#define IVTV_CARD_MPG600  3	/* Kuroutoshikou ITVC16-STVLP/YUAN MPG600, encoder only */
-#define IVTV_CARD_MPG160  4	/* Kuroutoshikou ITVC15-STVLP/YUAN MPG160
-				   cx23415 based, but does not have tv-out */
-#define IVTV_CARD_PVR_150 5	/* wintv pvr 150 */
-#define IVTV_CARD_PG600 6	/* YUAN PG600/DIAMONDMM PVR-550 based on the CX Falcon 2 */
-#define IVTV_CARD_AVC2410 7	/* Adaptec AVC-2410 */
-#define IVTV_CARD_AVC2010 8	/* Adaptec AVD-2010 (No Tuner) */
-
-#define NO_AUDIO    0		/* the card can't handle audio */
-#define USE_MSP34XX 1		/* switch audio by MSP34XX */
-#define USE_GPIO    2		/* switch audio by GPIO */
-#define USE_CX25840 3		/* switch audio by CX25840 */
-#define USE_PVR150  4		/* switch audio with WM8775 and CX25840 */
-#define USE_CS53L32A 5		/* switch audio with CS53L32A */
+// Supported cards
+#define IVTV_CARD_PVR_250 	 0	/* WinTV PVR 250 */
+#define IVTV_CARD_PVR_350 	 1	/* encoder, decoder, tv-out */
+#define IVTV_CARD_PVR_150 	 2	/* WinTV PVR 150 and PVR 500 (really just two
+					   PVR150s on one PCI board) */
+#define IVTV_CARD_M179    	 3	/* AVerMedia M179 (encoder only) */
+#define IVTV_CARD_MPG600  	 4	/* Kuroutoshikou ITVC16-STVLP/YUAN MPG600, encoder only */
+#define IVTV_CARD_MPG160  	 5	/* Kuroutoshikou ITVC15-STVLP/YUAN MPG160
+					   cx23415 based, but does not have tv-out */
+#define IVTV_CARD_PG600 	 6	/* YUAN PG600/DIAMONDMM PVR-550 based on the CX Falcon 2 */
+#define IVTV_CARD_AVC2410 	 7	/* Adaptec AVC-2410 */
+#define IVTV_CARD_AVC2010 	 8	/* Adaptec AVD-2010 (No Tuner) */
+#define IVTV_CARD_TG5000TV   	 9  	/* NAGASE TRANSGEAR 5000TV, encoder only */
+#define IVTV_CARD_VA2000MAX_SNT6 10  	/* VA2000MAX-STN6 */
+#define IVTV_CARD_CX23416GYC 	 11  	/* Kuroutoshikou CX23416GYC-STVLP (Yuan MPG600GR OEM) */
+#define IVTV_CARD_GV_MVPRX   	 12  	/* I/O Data GV-MVP/RX, RX2, RX2W */
+#define IVTV_CARD_GV_MVPRX2E 	 13  	/* I/O Data GV-MVP/RX2E */
+#define IVTV_CARD_GOTVIEW_PCI_DVD    14	/* GotView PCI DVD */
+#define IVTV_CARD_GOTVIEW_PCI_DVD2   15	/* GotView PCI DVD2 */
+#define IVTV_CARD_YUAN_MPC622    16	/* Yuan MPC622 miniPCI */
+#define IVTV_CARD_DCTMTVP1 	 17     /* DIGITAL COWBOY DCT-MTVP1 */
+/* Variants of existing cards but with the same PCI IDs. The driver
+   detects these based on other device information.
+   These cards must always come last.
+   New cards must be inserted above, and the indices of the cards below
+   must be adjusted accordingly. */
+#define IVTV_CARD_PVR_350_V1 	     18	/* encoder, decoder, tv-out, V1 (uses saa7114) */
+#define IVTV_CARD_CX23416GYC_NOGR    19	/* Kuroutoshikou CX23416GYC-STVLP (Yuan MPG600GR OEM) */
+#define IVTV_CARD_CX23416GYC_NOGRYCS 20	/* Kuroutoshikou CX23416GYC-STVLP (Yuan MPG600GR OEM) */
 
 #define IVTV_AUDIO_32000 0x2	/* 32 kHz audio */
 #define IVTV_AUDIO_44100 0x0	/* 44.1 kHz audio */
@@ -130,24 +145,32 @@ extern u32 yuv_offset[4];
 #define IVTV_V4L2_ENC_PCM_OFFSET 24	/* offset from 0 to register pcm v4l2 minors on */
 #define IVTV_V4L2_ENC_YUV_OFFSET 32	/* offset from 0 to register yuv v4l2 minors on */
 #define IVTV_V4L2_DEC_YUV_OFFSET 48	/* offset from 0 to register decoder yuv v4l2 minors on */
-#define IVTV_V4L2_DEC_VBI_OFFSET  4	/* offset from 0 to register decoder vbi input v4l2 minors on */
-#define IVTV_V4L2_DEC_VOUT_OFFSET 8	/* offset from 0 to register vbi output v4l2 minors on */
+#define IVTV_V4L2_DEC_VBI_OFFSET   8	/* offset from 0 to register decoder vbi input v4l2 minors on */
+#define IVTV_V4L2_DEC_VOUT_OFFSET 16	/* offset from 0 to register vbi output v4l2 minors on */
 
 #define IVTV_ENC_MEM_START 0x00000000
 #define IVTV_DEC_MEM_START 0x01000000
 
+/* system vendor and device IDs */
 #define PCI_VENDOR_ID_ICOMP  0x4444
 #define PCI_DEVICE_ID_IVTV15 0x0803
 #define PCI_DEVICE_ID_IVTV16 0x0016
-#define IVTV_PCI_ID_HAUPPAUGE 0x0070	/* subsystem vendor id */
-#define IVTV_PCI_ID_ADAPTEC 0x9005	/* subsystem vendor id */
-#define IVTV_PCI_ID_AVERMEDIA 0x1461
-#define IVTV_PCI_ID_YUAN1 0x12ab
-#define IVTV_PCI_ID_YUAN2 0xff01
-#define IVTV_PCI_ID_YUAN3 0xffab
-#define IVTV_PCI_ID_DIAMONDMM 0xff92
-#define IVTV_PCI_ID_XTREME550 0x0070
-#define IVTV_PCI_ID_IODATA 0x10fc
+
+/* subsystem vendor ID */
+#define IVTV_PCI_ID_HAUPPAUGE 		0x0070
+#define IVTV_PCI_ID_HAUPPAUGE_ALT1 	0x0270
+#define IVTV_PCI_ID_HAUPPAUGE_ALT2 	0x4070
+#define IVTV_PCI_ID_ADAPTEC 		0x9005
+#define IVTV_PCI_ID_AVERMEDIA 		0x1461
+#define IVTV_PCI_ID_YUAN1		0x12ab
+#define IVTV_PCI_ID_YUAN2 		0xff01
+#define IVTV_PCI_ID_YUAN3 		0xffab
+#define IVTV_PCI_ID_YUAN4 		0xfbab
+#define IVTV_PCI_ID_DIAMONDMM 		0xff92
+#define IVTV_PCI_ID_XTREME550 		0x0070
+#define IVTV_PCI_ID_IODATA 		0x10fc
+#define IVTV_PCI_ID_MELCO 		0x1154
+#define IVTV_PCI_ID_GOTVIEW 		0xffac
 
 #define IVTV_MBOX_MAX_BOXES 20
 #define IVTV_MBOX_API_BOXES 6
@@ -323,9 +346,6 @@ struct ivtv_dec_dma {
 #define IVTV_DMA_WRITE_ERR (IVTV_DMA_ERR_LIST | IVTV_DMA_ERR_WRITE)
 #define IVTV_DMA_ERR (IVTV_DMA_ERR_LIST | IVTV_DMA_ERR_WRITE | IVTV_DMA_ERR_READ)
 
-#define ITVC_GET_REG		0x1
-#define ITVC_SET_REG		0x2
-
 /* video related */
 #define IVTV_MAX_INPUTS 9
 
@@ -447,16 +467,7 @@ struct ivtv_dec_dma {
 #define IVTV_SDRAM_SLEEPTIME (60 * HZ / 100)	/* 600ms */
 
 /*Used for locating the firmware mailboxes*/
-#if defined(CONFIG_FW_LOADER) || defined(CONFIG_FW_LOADER_MODULE)
-#include <linux/firmware.h>
-#define IVTV_FIRM_ENC_FILENAME "v4l-cx2341x-enc.fw"
-#define IVTV_FIRM_DEC_FILENAME "v4l-cx2341x-dec.fw"
 #define IVTV_DECODE_INIT_MPEG_FILENAME "v4l-cx2341x-init.mpg"
-#else
-#define IVTV_FIRM_ENC_FILENAME "/lib/modules/v4l-cx2341x-enc.fw"
-#define IVTV_FIRM_DEC_FILENAME "/lib/modules/v4l-cx2341x-dec.fw"
-#define IVTV_DECODE_INIT_MPEG_FILENAME "/lib/modules/v4l-cx2341x-init.mpg"
-#endif
 
 #define IVTV_FIRM_IMAGE_SIZE 256*1024
 #define IVTV_FIRM_SEARCH_ENCODER_START 0x00000000
@@ -479,107 +490,18 @@ struct ivtv_dec_dma {
 #define IVTV_CMD_SOFT_RESET  1	/* quick + find mboxes */
 #define IVTV_CMD_FULL_RESET  2	/* full stop/upload/start/find mboxes */
 
-/* Firmware API commands */
-
-/* MPEG decoder API */
-#define IVTV_API_DEC_PING_FW 			0x00000000
-#define IVTV_API_DEC_START_PLAYBACK 		0x00000001
-#define IVTV_API_DEC_STOP_PLAYBACK 		0x00000002
-#define IVTV_API_DEC_PLAYBACK_SPEED 		0x00000003
-#define IVTV_API_DEC_STEP_VIDEO 		0x00000005
-#define IVTV_API_DEC_DMA_BLOCKSIZE 		0x00000008
-#define IVTV_API_DEC_XFER_INFO 			0x00000009
-#define IVTV_API_DEC_DMA_STATUS 		0x0000000a
-#define IVTV_API_DEC_DMA_FROM_HOST 		0x0000000b
-#define IVTV_API_DEC_PAUSE_PLAYBACK 		0x0000000d
-#define IVTV_API_DEC_HALT_FW 			0x0000000e
-#define IVTV_API_DEC_DISP_STANDARD 		0x00000010
-#define IVTV_API_DEC_GETVER 			0x00000011
-#define IVTV_API_DEC_STREAM_INPUT 		0x00000014
-#define IVTV_API_DEC_TIMING_INFO 		0x00000015
-#define IVTV_API_DEC_SELECT_AUDIO 		0x00000016
-#define IVTV_API_DEC_EVENT_NOTIFICATION 	0x00000017
-#define IVTV_API_DEC_DISPLAY_BUFFERS 		0x00000018
-#define IVTV_API_DEC_EXTRACT_VBI 		0x00000019
-#define IVTV_API_DEC_DECODE_SOURCE 		0x0000001a
-#define IVTV_API_DEC_AUDIO_OUTPUT 		0x0000001b
-#define IVTV_API_DEC_SET_AV_DELAY 		0x0000001c
-#define IVTV_API_DEC_BUFFER 			0x0000001e
-
-/* MPEG encoder API */
-#define IVTV_API_ENC_PING_FW 			0x00000080
-#define IVTV_API_BEGIN_CAPTURE 			0x00000081
-#define IVTV_API_END_CAPTURE 			0x00000082
-#define IVTV_API_ASSIGN_AUDIO_ID 		0x00000089
-#define IVTV_API_ASSIGN_VIDEO_ID 		0x0000008b
-#define IVTV_API_ASSIGN_PCR_ID 			0x0000008d
-#define IVTV_API_ASSIGN_FRAMERATE 		0x0000008f
-#define IVTV_API_ASSIGN_FRAME_SIZE 		0x00000091
-#define IVTV_API_ASSIGN_BITRATES 		0x00000095
-#define IVTV_API_ASSIGN_GOP_PROPERTIES 		0x00000097
-#define IVTV_API_ASSIGN_ASPECT_RATIO 		0x00000099
-#define IVTV_API_ASSIGN_DNR_FILTER_MODE 	0x0000009b
-#define IVTV_API_ASSIGN_DNR_FILTER_PROPS 	0x0000009d
-#define IVTV_API_ASSIGN_CORING_LEVELS 		0x0000009f
-#define IVTV_API_ASSIGN_SPATIAL_FILTER_TYPE 	0x000000a1
-#define IVTV_API_ASSIGN_3_2_PULLDOWN 		0x000000b1
-#define IVTV_API_SELECT_VBI_LINE 		0x000000b7
-#define IVTV_API_ASSIGN_STREAM_TYPE 		0x000000b9
-#define IVTV_API_ASSIGN_OUTPUT_PORT 		0x000000bb
-#define IVTV_API_ASSIGN_AUDIO_PROPERTIES 	0x000000bd
-#define IVTV_API_ENC_HALT_FW 			0x000000c3
-#define IVTV_API_ENC_GETVER 			0x000000c4
-#define IVTV_API_ASSIGN_GOP_CLOSURE 		0x000000c5
-#define IVTV_API_ENC_GET_SEQ_END 		0x000000c6
-#define IVTV_API_ASSIGN_PGM_INDEX_INFO 		0x000000c7
-#define IVTV_API_CONFIG_VBI 			0x000000c8
-#define IVTV_API_ASSIGN_DMA_BLOCKLEN 		0x000000c9
-#define IVTV_API_PREV_DMA_INFO_MB_10		0x000000ca
-#define IVTV_API_PREV_DMA_INFO_MB_9		0x000000cb
-#define IVTV_API_SCHED_DMA_TO_HOST 		0x000000cc
-#define IVTV_API_INITIALIZE_INPUT 		0x000000cd
-#define IVTV_API_ASSIGN_FRAME_DROP_RATE 	0x000000d0
-#define IVTV_API_PAUSE_ENCODER 			0x000000d2
-#define IVTV_API_REFRESH_INPUT 			0x000000d3
-#define IVTV_API_ASSIGN_COPYRIGHT		0x000000d4
-#define IVTV_API_EVENT_NOTIFICATION 		0x000000d5
-#define IVTV_API_ASSIGN_NUM_VSYNC_LINES 	0x000000d6
-#define IVTV_API_ASSIGN_PLACEHOLDER 		0x000000d7
-#define IVTV_API_MUTE_VIDEO 			0x000000d9
-#define IVTV_API_MUTE_AUDIO 			0x000000da
-#define IVTV_API_ENC_UNKNOWN			0x000000db
-#define IVTV_API_ENC_MISC 			0x000000dc
-
-/* OSD API */
-#define IVTV_API_FB_GET_FRAMEBUFFER 		0x00000041
-#define IVTV_API_FB_GET_PIXEL_FORMAT 		0x00000042
-#define IVTV_API_FB_SET_PIXEL_FORMAT 		0x00000043
-#define IVTV_API_FB_GET_STATE 			0x00000044
-#define IVTV_API_FB_SET_STATE 			0x00000045
-#define IVTV_API_FB_GET_OSD_COORDS 		0x00000046
-#define IVTV_API_FB_SET_OSD_COORDS 		0x00000047
-#define IVTV_API_FB_GET_SCREEN_COORDS 		0x00000048
-#define IVTV_API_FB_SET_SCREEN_COORDS 		0x00000049
-#define IVTV_API_FB_GET_GLOBAL_ALPHA 		0x0000004a
-#define IVTV_API_FB_SET_GLOBAL_ALPHA 		0x0000004b
-#define IVTV_API_FB_SET_BLEND_COORDS 		0x0000004c
-#define IVTV_API_FB_GET_FLICKER_STATE 		0x0000004f
-#define IVTV_API_FB_SET_FLICKER_STATE 		0x00000050
-#define IVTV_API_FB_BLT_COPY 			0x00000052
-#define IVTV_API_FB_BLT_FILL 			0x00000053
-#define IVTV_API_FB_BLT_TEXT 			0x00000054
-#define IVTV_API_FB_SET_FRAMEBUFFER_WINDOW 	0x00000056
-#define IVTV_API_FB_SET_CHROMA_KEY 		0x00000060
-#define IVTV_API_FB_GET_ALPHA_CONTENT_INDEX 	0x00000061
-#define IVTV_API_FB_SET_ALPHA_CONTENT_INDEX 	0x00000062
-
 /* i2c stuff */
 #define I2C_CLIENTS_MAX 16
-#define I2C_TIMING (0x7<<4)
-#define IVTV_REG_I2C_SETSCL_OFFSET (0x7000 /*| IVTV_REG_OFFSET*/)
-#define IVTV_REG_I2C_SETSDA_OFFSET (0x7004 /*| IVTV_REG_OFFSET*/)
-#define IVTV_REG_I2C_GETSCL_OFFSET (0x7008 /*| IVTV_REG_OFFSET*/)
-#define IVTV_REG_I2C_GETSDA_OFFSET (0x700c /*| IVTV_REG_OFFSET*/)
+#define I2C_TIMING (0x7 << 4)
+#define IVTV_REG_I2C_SETSCL_OFFSET 0x7000
+#define IVTV_REG_I2C_SETSDA_OFFSET 0x7004
+#define IVTV_REG_I2C_GETSCL_OFFSET 0x7008
+#define IVTV_REG_I2C_GETSDA_OFFSET 0x700c
+
+/* GPIO */
+#define IVTV_REG_GPIO_IN_OFFSET    0x9008
+#define IVTV_REG_GPIO_OUT_OFFSET   0x900c
+#define IVTV_REG_GPIO_DIR_OFFSET   0x9020
 
 /* debugging */
 /* NOTE: extra space before comma in 'itv->num , ## args' is required for
@@ -650,7 +572,6 @@ struct ivtv_options {
 	int cardtype;		/* force card type on load */
 	int tuner;		/* set tuner on load */
 	int radio;		/* enable/disable radio */
-        int tda9887;
 	int dynbuf;		/* Enable/Disable dynamic buffers */
 	int newi2c;		/* New I2C algorithm */
 };
@@ -682,7 +603,6 @@ struct ivtv_mailbox {
 
 /* per buffer bit flags */
 #define IVTV_F_B_NEED_BUF_SWAP  0	/* bit 0 for swap needed */
-#define IVTV_F_B_NEED_BUF_VBI   1	/* bit 0 for vbi processing needed */
 
 /* per-stream, s_flags */
 #define IVTV_F_S_DMAP		0
@@ -704,19 +624,14 @@ struct ivtv_mailbox {
 #define IVTV_F_I_EOS		1
 #define IVTV_F_I_RADIO_USER	2
 #define IVTV_F_I_PASSTHROUGH	3
-#define IVTV_F_I_DIG_PAUSE	4
 #define IVTV_F_I_DIG_RST	5
-#define IVTV_F_I_MUTE_PAUSE	6
 #define IVTV_F_I_DEC_YUV	7
 
 /* dma-tasklet, dma-thread, t_flags */
 /* tasklets */
-#define IVTV_F_T_DEC_VSYNC	0
 #define IVTV_F_T_DEC_VBI_INSERT	1
 #define IVTV_F_T_ENC_DMA_NEEDED 2
-#define IVTV_F_T_ENC_DMA_DONE 	3
 #define IVTV_F_T_ENC_VBI_DMA 	4
-#define IVTV_F_T_DEC_SCHED	5
 #define IVTV_F_T_DEC_DONE	6
 #define IVTV_F_T_DEC_STARTED	7 /* Decoder has been started */
 #define IVTV_F_T_DEC_RUNNING	8 /*Decoder thread is active in it's main loop*/
@@ -732,8 +647,6 @@ struct ivtv_mailbox {
 /* firmware-reset, r_flags */
 #define FW_RESET_NEEDED        	0
 #define OSD_RESET_NEEDED       	1
-#define DEC_DMA_ERR		3
-#define FW_SOFT_RESET_NEEDED  	4
 #define FW_RESET_LOCK	  	5
 #define FW_RESET_SHUTDOWN  	6
 
@@ -796,15 +709,12 @@ struct ivtv_stream {
 	unsigned long trans_id;
 	int subtype;
 	int type;
-	int dmatype;
 	u64 pts;		/* last seen Program Time Stamp (PTS) */
 	wait_queue_head_t waitq;
 	struct video_device *v4l2dev;
 
 	/* Buffer Stats */
 	atomic_t allocated_buffers;
-	u32 buf_backup;
-	u32 last_offset;
 	int buffers;
 	u32 buf_min;
 	u32 buf_max;
@@ -960,6 +870,7 @@ struct yuv_playback_info
 
 /* forward declaration of struct defined in ivtv-cards.h */
 struct ivtv_card;
+struct ivtv;
 
 #define IVTV_VBI_FRAMES 50
 
@@ -967,19 +878,18 @@ struct ivtv_card;
 struct ivtv {
 	const struct ivtv_card *card;	/* card information */
         const char *card_name;
-	int has_cx23415;		/* 1 if it is a cx23415 based card, 0 for cx23416 */
-        u8 has_cx25840; 
-        u8 has_saa7114; 
-        u8 has_saa7115; 
-        u8 has_tda9887; 
+	u8 has_cx23415;		/* 1 if it is a cx23415 based card, 0 for cx23416 */
         u8 is_50hz; 
         u8 is_60hz;
-	u32 v4l2_cap;		/* V4L2 capabilities of card */
-        u32 i2c_tv_tuner_addr;
-        u32 i2c_radio_tuner_addr;
-        u32 hw_flags;
 	u8 pvr150_workaround;
 	u8 set_fm_high_sensitivity; /* the tuner requires PORT1=1 for radio */
+	u8 nof_inputs;
+	u8 nof_audio_inputs;
+	u32 v4l2_cap;		/* V4L2 capabilities of card */
+        u32 hw_flags;
+
+	// controlling Video decoder function
+	int (*video_dec_func)(struct ivtv *, unsigned int, void *); 
 
 	struct pci_dev *dev;
 	struct ivtv_options options;
@@ -1041,7 +951,7 @@ struct ivtv {
 	int vbi_insert_mpeg;
 
 	/* Buffer for the maximum of 2 * 18 * packet_size sliced VBI lines.
-	   One for /dev/vbi0 and one for /dev/vbi4 */
+	   One for /dev/vbi0 and one for /dev/vbi8 */
         struct v4l2_sliced_vbi_data vbi_sliced_data[36];
         struct v4l2_sliced_vbi_data vbi_sliced_dec_data[36];
 	/* Buffer for VBI data inserted into MPEG stream.
@@ -1068,7 +978,10 @@ struct ivtv {
 	struct semaphore enc_msem;
 	struct semaphore dec_msem;
 
-	unsigned char card_rev, *enc_mem, *dec_mem, *reg_mem;
+	u8 card_rev, *enc_mem, *dec_mem, *reg_mem;
+
+	/* GPIO direction, in and out addresses */
+	u8 *gpio_dir, *gpio_in, *gpio_out;
 
 	u32 idx_sdf_offset;
 	u32 idx_sdf_num;
@@ -1095,14 +1008,7 @@ struct ivtv {
 
 	/* codec settings */
 	struct ivtv_ioctl_codec codec;
-	u32 audmode_tv;
-	u32 audmode_radio;
-	unsigned long freq_tv;
-	unsigned long freq_radio;
-	u32 audio_input_tv;
-	u32 audio_input_radio;
-	u32 msp34xx_tuner_input;
-	u32 msp34xx_audio_output;
+	u32 audio_input;
 	u32 active_input;
 	u32 active_output;
 	v4l2_std_id std;
@@ -1132,15 +1038,12 @@ struct ivtv {
 
 	atomic_t streams_setup;
 
-#ifdef LINUX26
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	struct workqueue_struct *vbi_work_queues;
-	struct workqueue_struct *fw_work_queues;
 	struct work_struct vbi_work_queue;
-	struct work_struct fw_work_queue;
 #else
 	struct tq_struct vbi_work_queue;
-	struct tq_struct fw_work_queue;
-#endif /* LINUX26 */
+#endif
 
 	struct ivtv_dec_dma dec_dma_stat;
 	struct ivtv_dma_settings dma_cfg;
@@ -1199,14 +1102,10 @@ void dec_schedule_work(struct ivtv *itv);
 void enc_schedule_work(struct ivtv *itv);
 void enc_vbi_schedule_work(struct ivtv *itv);
 void dec_vbi_schedule_work(struct ivtv *itv);
-void fw_schedule_work(struct ivtv *itv);
 
-#ifdef LINUX26
-int ivtv_schedule_work(struct work_struct *work);
-#else
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 int ivtv_schedule_work(struct tq_struct *work);
-#endif /* LINUX26 */
-void ivtv_flush_scheduled_work(struct ivtv *itv);
+#endif
 
 /* Hardware/IRQ */
 void ivtv_set_irq_mask(struct ivtv *itv, unsigned long mask);
@@ -1215,6 +1114,10 @@ void ivtv_clear_irq_mask(struct ivtv *itv, unsigned long mask);
 /* Return non-zero if a signal is pending */
 int ivtv_sleep_timeout(int timeout, int intr);
 
+/* Read Hauppauge eeprom */
+struct tveeprom; /* forward reference */
+void ivtv_read_eeprom(struct ivtv *itv, struct tveeprom *tv);
+
 /* debug stuff, to get the locking right */
 #ifndef WARN_ON
 #define WARN_ON(condition) do { \
