@@ -31,9 +31,8 @@
 #include "ivtv-video.h"
 #include "ivtv-streams.h"
 #include "ivtv-cards.h"
-#include "audiochip.h"
-#include "cx25840.h"
 #include "ivtv-ioctl.h"
+#include "ivtv-controls.h"
 
 typedef unsigned long uintptr_t;
 
@@ -181,7 +180,7 @@ static int ivtv_check_digitizer_sync(struct ivtv *itv)
 
 	IVTV_DEBUG_INFO("Checking digitizer\n");
 
-	itv->card->video_dec_func(itv, VIDIOC_G_TUNER, &vt);
+	itv->video_dec_func(itv, VIDIOC_G_TUNER, &vt);
 
 	if (vt.signal) {
 		IVTV_DEBUG_INFO("Digitizer sync GOOD\n");
@@ -201,7 +200,7 @@ static int ivtv_reset_digitizer(struct ivtv *itv)
 		/* x is just a placeholder. It's unused */
 		/* this just resets the scaler.. 
 			shouldn't need to re-do any settings */
-		ret = itv->card->video_dec_func(itv, VIDIOC_INT_RESET, &x);
+		ret = itv->video_dec_func(itv, VIDIOC_INT_RESET, &x);
 	}
 
 	/* Re-Init Digitizer */
@@ -506,11 +505,8 @@ static int ivtv_read(struct file *filp, char *ubuf, size_t count, loff_t * pos)
 			}
 
 			/* Possible digitizer sync issue? */
-			IVTV_DEBUG_WARN(
-				   "Check digitizer Sync\n");
-        		if ((itv->card->type == IVTV_CARD_PVR_150) ||
-            			(itv->card->type == IVTV_CARD_PG600))
-			{
+			IVTV_DEBUG_WARN("Check digitizer Sync\n");
+        		if (itv->hw_flags & IVTV_HW_CX25840) {
 				x = 1;
 			} else {
 				x = ivtv_check_digitizer_sync(itv);
@@ -1289,29 +1285,18 @@ int ivtv_v4l2_close(struct inode *inode, struct file *filp)
 
 	/* Stop radio */
 	if (id->type == IVTV_ENC_STREAM_TYPE_RAD) {
-		struct v4l2_frequency vf;
+                int cfg = TDA9887_PORT1_ACTIVE;
 
 		// Closing radio device, return to TV mode
 		ivtv_mute(itv);
-		/* Switch tuner to TV */
-		ivtv_tv_tuner(itv, VIDIOC_S_STD, &itv->std);
 		/* Mark that the radio is no longer in use */
 		clear_bit(IVTV_F_I_RADIO_USER, &itv->i_flags);
-		/* Select TV frequency */
-		vf.frequency = itv->freq_tv;
-		vf.type = V4L2_TUNER_ANALOG_TV;
-		ivtv_tv_tuner(itv, VIDIOC_S_FREQUENCY, &vf);
-		/* Make sure IF demodulator is updated properly, or we'll get static */
-		if (itv->options.tda9887 >= 0) {
-                	int cfg = TDA9887_PORT1_ACTIVE;
-			ivtv_tda9887(itv, VIDIOC_S_STD, &itv->std);
-			if (itv->set_fm_high_sensitivity)
-                		ivtv_tda9887(itv, TDA9887_SET_CONFIG, &cfg);
-		}
+		/* Switch tuner to TV */
+		ivtv_call_i2c_clients(itv, VIDIOC_S_STD, &itv->std);
+		if (itv->set_fm_high_sensitivity)
+                	ivtv_call_i2c_clients(itv, TDA9887_SET_CONFIG, &cfg);
 		/* Select correct audio input (i.e. TV tuner or Line in) */
 		ivtv_audio_set_io(itv);
-		/* Start automatic sound detection (if supported) */
-		ivtv_audio_freq_changed(itv);
 		/* Done! Unmute and continue. */
 		ivtv_unmute(itv);
 		ivtv_release_stream(itv, id->type);
@@ -1428,9 +1413,10 @@ int ivtv_v4l2_close(struct inode *inode, struct file *filp)
 		writel(0, itv->reg_mem + 0x0282c);
 		writel(0, itv->reg_mem + 0x02904);
 		writel(0, itv->reg_mem + 0x02910);
-		
+
 		// Set the display to black
 		ivtv_write_reg(0x01008080, itv->reg_mem + 0x2898);
+
 		clear_bit(IVTV_F_S_DECODING_YUV, &stream->s_flags);
         }
 
@@ -1496,7 +1482,7 @@ int ivtv_v4l2_open(struct inode *inode, struct file *filp)
 	filp->private_data = item;
 
 	if (item->type == IVTV_ENC_STREAM_TYPE_RAD) {
-		struct v4l2_frequency vf;
+                int cfg = TDA9887_PORT1_INACTIVE;
 
 		/* Try to claim this stream */
 		if (ivtv_claim_stream(item, item->type)) {
@@ -1508,24 +1494,13 @@ int ivtv_v4l2_open(struct inode *inode, struct file *filp)
 		/* We have the radio */
 		ivtv_mute(itv);
 		/* Switch tuner to radio */
-		ivtv_radio_tuner(itv, AUDC_SET_RADIO, 0);
-		/* Switch IF demodulator to radio */
-		if (itv->options.tda9887 >= 0) {
-                	int cfg = TDA9887_PORT1_INACTIVE;
-			ivtv_tda9887(itv, AUDC_SET_RADIO, 0);
-			if (itv->set_fm_high_sensitivity)
-                		ivtv_tda9887(itv, TDA9887_SET_CONFIG, &cfg);
-		}
+		ivtv_call_i2c_clients(itv, AUDC_SET_RADIO, 0);
+		if (itv->set_fm_high_sensitivity)
+                	ivtv_call_i2c_clients(itv, TDA9887_SET_CONFIG, &cfg);
 		/* Mark that the radio is being used. */
 		set_bit(IVTV_F_I_RADIO_USER, &itv->i_flags);
-		/* Select radio frequency */
-		vf.type = V4L2_TUNER_RADIO;
-		vf.frequency = itv->freq_radio;
-		ivtv_radio_tuner(itv, VIDIOC_S_FREQUENCY, &vf);
 		/* Select the correct audio input (i.e. radio tuner) */
 		ivtv_audio_set_io(itv);
-		/* Start automatic sound detection (if supported) */
-		ivtv_audio_freq_changed(itv);
 		/* Done! Unmute and continue. */
 		ivtv_unmute(itv);
 	}
@@ -1557,27 +1532,32 @@ int ivtv_v4l2_open(struct inode *inode, struct file *filp)
 
 void ivtv_mute(struct ivtv *itv)
 {
-	IVTV_DEBUG_INFO("Mute\n");
+	struct v4l2_control ctrl = { V4L2_CID_AUDIO_MUTE, 1 };
+
 	/* Mute sound to avoid pop */
-	ivtv_audio_set_mute(itv, 1);
+	ivtv_control_ioctls(itv, VIDIOC_S_CTRL, &ctrl);
 
 	if (atomic_read(&itv->capturing))
-        	ivtv_vapi(itv, IVTV_API_MUTE_AUDIO, 1);
+        	ivtv_vapi(itv, CX2341X_ENC_MUTE_AUDIO, 1);
+
+	IVTV_DEBUG_INFO("Mute\n");
 }
 
 void ivtv_unmute(struct ivtv *itv)
 {
+	struct v4l2_control ctrl = { V4L2_CID_AUDIO_MUTE, 0 };
+
         /* initialize or refresh input */
         if (atomic_read(&itv->capturing) == 0)
-                ivtv_vapi(itv, IVTV_API_INITIALIZE_INPUT, 0);
+                ivtv_vapi(itv, CX2341X_ENC_INITIALIZE_INPUT, 0);
 
 	if (atomic_read(&itv->capturing)) {
 		ivtv_sleep_timeout(HZ / 10, 0);
-		ivtv_vapi(itv, IVTV_API_ENC_MISC, 12);
-		ivtv_vapi(itv, IVTV_API_MUTE_AUDIO, 0);
+		ivtv_vapi(itv, CX2341X_ENC_MISC, 12);
+		ivtv_vapi(itv, CX2341X_ENC_MUTE_AUDIO, 0);
 	}
 
 	/* Unmute */
-	ivtv_audio_set_mute(itv, 0);
+	ivtv_control_ioctls(itv, VIDIOC_S_CTRL, &ctrl);
 	IVTV_DEBUG_INFO("Unmute\n");
 }
