@@ -27,18 +27,20 @@
 #include "ivtv-firmware.h"
 
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
-#define FWDEV(x) &((x)->dev)
+#define FWDEV(x) &((x)->dev->dev)
 #else
-#define FWDEV(x) itv->name
+#define FWDEV(x) (x)->name
 #endif
 
+#include <linux/firmware.h>
+
 static u32 ivtv_firm_search_id[] =
     { 0x12345678, 0x34567812, 0x56781234, 0x78123456 };
 
 static int ivtv_check_enc_firmware(struct ivtv *itv)
 {
 	/* check enc firmware */
-	if (ivtv_vapi(itv, IVTV_API_ENC_PING_FW, 0)) {
+	if (ivtv_vapi(itv, CX2341X_ENC_PING_FW, 0)) {
 		IVTV_DEBUG_WARN("Encoder firmware dead!\n");
 		/* WARNING this creates a race (if another process is
 		   in the midst of an api command).. */
@@ -52,7 +54,7 @@ static int ivtv_check_enc_firmware(struct ivtv *itv)
 static int ivtv_check_dec_firmware(struct ivtv *itv)
 {
 	/* check dec firmware */
-	if (ivtv_vapi(itv, IVTV_API_DEC_PING_FW, 0)) {
+	if (ivtv_vapi(itv, CX2341X_DEC_PING_FW, 0)) {
 		IVTV_DEBUG_WARN("Decoder firmware dead!\n");
 		/* WARNING this creates a race (if another process is
 		   in the midst of an api command).. */
@@ -67,7 +69,6 @@ int ivtv_check_firmware(struct ivtv *itv)
 {
 	int enc_error, dec_error = 0;
 
-	IVTV_DEBUG_INFO("Checking firmware\n");
 	/* fixme need to grab the semaphore to ensure that
 	   there are no programs in the mailbox right now */
 
@@ -80,15 +81,12 @@ int ivtv_check_firmware(struct ivtv *itv)
 	return enc_error | dec_error;
 }
 
-#if defined(CONFIG_FW_LOADER) || defined(CONFIG_FW_LOADER_MODULE)
-/* Use hotplug support */
 static int load_fw_direct(const char *fn, char *mem, struct ivtv *itv, long size)
 {
 	const struct firmware *fw = NULL;
-	struct pci_dev *pdev = itv->dev;
 	int retval = -ENOMEM;
 
-	if (request_firmware(&fw, fn, FWDEV(pdev)) == 0) {
+	if (request_firmware(&fw, fn, FWDEV(itv)) == 0) {
 		int i;
 		u32 *dst = (u32 *)mem;
 		const u32 *src = (const u32 *)fw->data;
@@ -112,38 +110,12 @@ static int load_fw_direct(const char *fn, char *mem, struct ivtv *itv, long size
 
 	return retval;
 }
-#else
-/* do it ourselves (for older 2.4 kernels) */
-static int load_fw_direct(const char *fn, char *mem, struct ivtv *itv, long size) 
-{ 
-	kernel_filep filep; 
-	loff_t file_offset = 0; 
-	int retval = -EINVAL; 
-	mm_segment_t fs = get_fs(); 
-
-	set_fs(get_ds()); 
-
-	filep = kernel_file_open(fn, 0, 0); 
-
-	if (kernel_file_is_err(filep)) { 
-		IVTV_INFO("unable to open firmware\n"); 
-	} else { 
-		retval = kernel_file_read(filep, mem, size, &file_offset); 
-		kernel_file_close(filep); 
-		IVTV_INFO("loaded %s firmware (%d bytes)\n", fn, retval);
-	} 
-
-	set_fs(fs); 
-
-	return retval; 
-} 
-#endif
 
 static int ivtv_enc_firmware_copy(struct ivtv *itv)
 {
 	IVTV_DEBUG_INFO("Loading encoder image\n");
 
-	if (load_fw_direct(ivtv_efw?ivtv_efw:IVTV_FIRM_ENC_FILENAME,
+	if (load_fw_direct(ivtv_efw ? ivtv_efw : CX2341X_FIRM_ENC_FILENAME,
 			   (char *)(itv->enc_mem), itv, IVTV_FIRM_IMAGE_SIZE) !=
 	    IVTV_FIRM_IMAGE_SIZE) {
 		IVTV_DEBUG_WARN("failed loading encoder firmware\n");
@@ -155,7 +127,7 @@ static int ivtv_enc_firmware_copy(struct ivtv *itv)
 static int ivtv_dec_firmware_copy(struct ivtv *itv)
 {
 	IVTV_DEBUG_INFO("Loading decoder image\n");
-	if (load_fw_direct(ivtv_dfw?ivtv_dfw:IVTV_FIRM_DEC_FILENAME,
+	if (load_fw_direct(ivtv_dfw ? ivtv_dfw : CX2341X_FIRM_DEC_FILENAME,
 			   (char *)(itv->dec_mem), itv,IVTV_FIRM_IMAGE_SIZE) !=
 	    IVTV_FIRM_IMAGE_SIZE) {
 		IVTV_DEBUG_WARN("failed loading decoder firmware\n");
@@ -176,7 +148,7 @@ static int ivtv_stop_firmware_dec(struct ivtv *itv)
 		/*halt dec firmware */
 		if (itv->has_cx23415) {
 			x = ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-				     IVTV_API_DEC_HALT_FW, &result, 0,
+				     CX2341X_DEC_HALT_FW, &result, 0,
 				     &data[0]);
 			if (x)
 				IVTV_DEBUG_WARN(
@@ -200,7 +172,7 @@ static int ivtv_stop_firmware_enc(struct ivtv *itv)
 	if (NULL != itv->enc_mbox) {
 		/*halt enc firmware */
 		x = ivtv_api(itv, itv->enc_mbox, &itv->enc_msem,
-			     IVTV_API_ENC_HALT_FW, &result, 0, &data[0]);
+			     CX2341X_ENC_HALT_FW, &result, 0, &data[0]);
 		if (x)
 			IVTV_DEBUG_WARN("stop_fw error 3. Code %d\n",
 				   x);
@@ -287,7 +259,7 @@ void ivtv_start_firmware(struct ivtv *itv)
 	writel((readl(itv->reg_mem + IVTV_REG_SPU) & IVTV_MASK_SPU_ENABLE),
 	       (IVTV_REG_SPU + itv->reg_mem));
 
-	IVTV_DEBUG_WARN("Sleeping for 100 ms\n");
+	IVTV_DEBUG_INFO("Sleeping for 100 ms\n");
 	ivtv_sleep_timeout(HZ / 10, 0);
 
 	if (!itv->has_cx23415) {
@@ -298,7 +270,7 @@ void ivtv_start_firmware(struct ivtv *itv)
 			IVTV_MASK_VPU_ENABLE15), (IVTV_REG_VPU + itv->reg_mem));
 	}
 
-	IVTV_DEBUG_WARN("Sleeping for 100 ms\n");
+	IVTV_DEBUG_INFO("Sleeping for 100 ms\n");
 	ivtv_sleep_timeout(HZ / 10, 0);
 }
 
@@ -406,7 +378,7 @@ int ivtv_firmware_versions(struct ivtv *itv)
 
 	/* Encoder */
 	IVTV_DEBUG_INFO("Getting encoder firmware rev.\n");
-	x = ivtv_api(itv, itv->enc_mbox, &itv->enc_msem, IVTV_API_ENC_GETVER,
+	x = ivtv_api(itv, itv->enc_mbox, &itv->enc_msem, CX2341X_ENC_GET_VERSION,
 		     &result, 0, &data[0]);
 	if (x) {
 		IVTV_ERR("error getting Encoder firmware version\n");
@@ -422,7 +394,7 @@ int ivtv_firmware_versions(struct ivtv *itv)
 
 	/* Decoder */
 	IVTV_DEBUG_INFO("Getting decoder firmware rev.\n");
-	x = ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, IVTV_API_DEC_GETVER,
+	x = ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_DEC_GET_VERSION,
 		     &result, 0, &data[0]);
 	if (x) {
 		IVTV_ERR("error getting Decoder firmware version\n");
@@ -449,53 +421,6 @@ int ivtv_firmware_copy(struct ivtv *itv)
 	return 0;
 }
 
-int ivtv_stop_firmware(struct ivtv *itv)
-{
-	u32 data[IVTV_MBOX_MAX_DATA], result;
-	int x = 0;
-
-	IVTV_DEBUG_INFO("Stopping firmware\n");
-
-	if (atomic_read(&itv->capturing)) {
-		x = ivtv_stop_all_captures(itv);
-		if (x)
-			IVTV_DEBUG_WARN("stop_fw error 1. Code %d\n",
-				   x);
-	}
-
-	/*Stop decoder_playback */
-	data[0] = 1;		/* 0: render last frame, 1: stop NOW! :) */
-	data[1] = 0;		/* "low 4 bytes of stop index" */
-	data[2] = 0;		/* 0: stop immedeately */
-	x = ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		     IVTV_API_DEC_STOP_PLAYBACK, &result, 3, &data[0]);
-	if (x)
-		IVTV_DEBUG_WARN("stop_fw error 2. Code %d\n", x);
-
-	/*halt enc firmware */
-	x = ivtv_api(itv, itv->enc_mbox, &itv->enc_msem, IVTV_API_ENC_HALT_FW,
-		     &result, 0, &data[0]);
-	if (x)
-		IVTV_DEBUG_WARN("stop_fw error 3. Code %d\n", x);
-
-	IVTV_DEBUG_INFO("Sleeping for 10ms\n");
-	ivtv_sleep_timeout(HZ / 100, 0);
-
-	/*halt dec firmware */
-	if (itv->has_cx23415) {
-		x = ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-			     IVTV_API_DEC_HALT_FW, &result, 0, &data[0]);
-		if (x)
-			IVTV_DEBUG_WARN("stop_fw error 4. Code %d\n",
-				   x);
-
-		IVTV_DEBUG_INFO("Sleeping for 10ms\n");
-		ivtv_sleep_timeout(HZ / 100, 0);
-	}
-
-	return 0;
-}
-
 int ivtv_firmware_init(struct ivtv *itv)
 {
 	int x;
@@ -555,30 +480,7 @@ int ivtv_firmware_init(struct ivtv *itv)
 		IVTV_DEBUG_WARN("Error loading firmware %d!\n", x);
 		return x;
 	}
-
-	/*I guess this is read-modify-write :) */
-	writel((readl(itv->reg_mem + IVTV_REG_SPU) & IVTV_MASK_SPU_ENABLE),
-	       (IVTV_REG_SPU + itv->reg_mem));
-
-	IVTV_DEBUG_INFO("Sleeping for 100 ms\n");
-	ivtv_sleep_timeout(HZ / 10, 0);
-
-	/*I guess this is read-modify-write :) */
-	if (!itv->has_cx23415) {
-		writel((readl(itv->reg_mem + IVTV_REG_VPU) &
-			IVTV_MASK_VPU_ENABLE16), (IVTV_REG_VPU + itv->reg_mem));
-	} else {
-		writel((readl(itv->reg_mem + IVTV_REG_VPU) &
-			IVTV_MASK_VPU_ENABLE15), (IVTV_REG_VPU + itv->reg_mem));
-	}
-
-	IVTV_DEBUG_INFO("Sleeping for 100 ms\n");
-	ivtv_sleep_timeout(HZ / 10, 0);
-
-	/* GPIO init */
-	IVTV_DEBUG_INFO("GPIO INIT\n");
-	ivtv_set_gpio_audio(itv, IVTV_GPIO_AUDIO_INIT);
-
+	ivtv_start_firmware(itv);
 	return 0;
 }
 
@@ -665,13 +567,13 @@ ivtv_init_mpeg_decoder(struct ivtv *itv)
 							   bitmap. see docs. */
 	if ( ivtv_api(itv, itv->dec_mbox,
                       &itv->dec_msem,
-                      IVTV_API_DEC_DECODE_SOURCE, &result, 4, &data[0]) != 0)
+                      CX2341X_DEC_SET_DECODER_SOURCE, &result, 4, &data[0]) != 0)
 	{
 		IVTV_ERR("ivtv_init_mpeg_decoder failed to set decoder source\n");
 		return;
 	}
 
-	if (ivtv_vapi(itv, IVTV_API_DEC_START_PLAYBACK, 2,
+	if (ivtv_vapi(itv, CX2341X_DEC_START_PLAYBACK, 2,
                       itv->dec_options.gop_offset,
                       itv->dec_options.mute_frames) != 0)
 	{
@@ -686,11 +588,11 @@ ivtv_init_mpeg_decoder(struct ivtv *itv)
 		IVTV_INFO("failed to read mpeg decoder initialisation file %s\n",IVTV_DECODE_INIT_MPEG_FILENAME);
 	} else
 	{
-		ivtv_vapi(itv, IVTV_API_DEC_DMA_FROM_HOST, 3,
+		ivtv_vapi(itv, CX2341X_DEC_SCHED_DMA_FROM_HOST, 3,
                           0, readbytes, 0);
 		ivtv_sleep_timeout(HZ/10,0);
 	}
-	ivtv_vapi(itv, IVTV_API_DEC_STOP_PLAYBACK, 4,
+	ivtv_vapi(itv, CX2341X_DEC_STOP_PLAYBACK, 4,
                   0,
                   itv->dec_options.pts_low,
                   itv->dec_options.pts_hi, 1);
