@@ -17,17 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* MPG600/MPG160/M179 audio stuff */
-#define IVTV_GPIO_AUDIO_INIT    0
-#define IVTV_GPIO_AUDIO_TUNER   1
-#define IVTV_GPIO_AUDIO_LINE    2
-#define IVTV_GPIO_AUDIO_MUTE    3
-#define IVTV_GPIO_AUDIO_UNMUTE  4
-
 /* GPIO stuff */
-void ivtv_set_gpio_audio(struct ivtv *itv, int command);
-int ivtv_get_gpio_audio(struct ivtv *itv, int command);
-void ivtv_set_gpio_audiomode(struct ivtv *itv, int audio_mode);
-int ivtv_get_gpio_audiomode(struct ivtv *itv);
-void ivtv_set_gpio_audio_clock_freq(struct ivtv *itv, u8 freq);
+void ivtv_gpio_init(struct ivtv *itv);
 void ivtv_reset_ir_gpio(struct ivtv *itv);
+int ivtv_gpio(struct ivtv *itv, unsigned int command, void *arg);
