@@ -17,8 +17,55 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/*
+    This file includes an i2c implementation that was reverse engineered
+    from the Hauppauge windows driver.  Older ivtv versions used i2c-algo-bit,
+    which whilst fine under most circumstances, had trouble with the Zilog
+    CPU on the PVR-150 which handles IR functions (occasional inability to
+    communicate with the chip until it was reset) and also with the i2c
+    bus being completely unreachable when multiple PVR cards were present.
+
+    The implementation is very similar to i2c-algo-bit, but there are enough
+    subtle differences that the two are hard to merge.  The general strategy
+    employed by i2c-algo-bit is to use udelay() to implement the timing
+    when putting out bits on the scl/sda lines.  The general strategy taken 
+    here is to poll the lines for state changes (see ivtv_waitscl and
+    ivtv_waitsda).  In addition there are small delays at various locations
+    which poll the SCL line 5 times (ivtv_scldelay).  I would guess that
+    since this is memory mapped I/O that the length of those delays is tied
+    to the PCI bus clock.  There is some extra code to do with recovery
+    and retries.  Since it is not known what causes the actual i2c problems
+    in the first place, the only goal if one was to attempt to use 
+    i2c-algo-bit would be to try to make it follow the same code path.
+    This would be a lot of work, and I'm also not convinced that it would 
+    provide a generic benefit to i2c-algo-bit.  Therefore consider this
+    an engineering solution -- not pretty, but it works.
+
+    Some more general comments about what we are doing:
+
+    The i2c bus is a 2 wire serial bus, with clock (SCL) and data (SDA)
+    lines.  To communicate on the bus (as a master, we don't act as a slave),
+    we first initiate a start condition (ivtv_start).  We then write the 
+    address of the device that we want to communicate with, along with a flag 
+    that indicates whether this is a read or a write.  The slave then issues
+    an ACK signal (ivtv_ack), which tells us that it is ready for reading / 
+    writing.  We then proceed with reading or writing (ivtv_read/ivtv_write),
+    and finally issue a stop condition (ivtv_stop) to make the bus available
+    to other masters.
+
+    There is an additional form of transaction where a write may be
+    immediately followed by a read.  In this case, there is no intervening
+    stop condition.  (Only the msp3400 chip uses this method of data transfer).
+ */
+
 #include "ivtv-driver.h"
 #include "ivtv-cards.h"
+#include "ivtv-gpio.h"
+
+#include <media/ir-kbd-i2c.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
+#include <media/i2c-compat.h>
+#endif
 
 /* i2c implementation for cx23415/6 chip, ivtv project.
  * Author: Kevin Thayer (nufan_wfk at yahoo.com)
@@ -32,14 +79,36 @@
 #define IVTV_CX25840_I2C_ADDR 		0x44
 #define IVTV_SAA7115_I2C_ADDR 		0x21
 #define IVTV_SAA7127_I2C_ADDR 		0x44
-#define IVTV_TDA9887_I2C_ADDR 		0x43	/* SECAM */
+#define IVTV_SAA717x_I2C_ADDR 		0x21
 #define IVTV_MSP3400_I2C_ADDR 		0x40
 #define IVTV_HAUPPAUGE_I2C_ADDR 	0x50
-#define IVTV_WM8775_I2C_ADDR		0x1B
+#define IVTV_WM8739_I2C_ADDR 		0x1a
+#define IVTV_WM8775_I2C_ADDR		0x1b
 #define IVTV_TEA5767_I2C_ADDR		0x60
+#define IVTV_UPD64031A_I2C_ADDR 	0x12
+#define IVTV_UPD64083_I2C_ADDR 		0x5c
+#define IVTV_TDA985X_I2C_ADDR      	0x5b
 
-#ifndef NEW_I2C
-/* pre i2c-2.8.0 */
+/* This array should match the IVTV_HW_ defines */
+static const u8 hw_driverids[] = {
+	I2C_DRIVERID_CX25840,
+	I2C_DRIVERID_SAA711X,
+	I2C_DRIVERID_SAA7127,
+	I2C_DRIVERID_MSP3400,
+	I2C_DRIVERID_TUNER,
+	I2C_DRIVERID_WM8775,
+	I2C_DRIVERID_CS53L32A,
+	I2C_DRIVERID_TVEEPROM,
+	I2C_DRIVERID_SAA711X,
+	I2C_DRIVERID_TVAUDIO,
+	I2C_DRIVERID_UPD64031A,
+	I2C_DRIVERID_UPD64083,
+	I2C_DRIVERID_SAA717X,
+	I2C_DRIVERID_WM8739,
+	0 		/* IVTV_HW_GPIO dummy driver ID */
+};
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 static void ivtv_i2c_inc(struct i2c_adapter *adapter)
 {
 	MOD_INC_USE_COUNT;
@@ -49,12 +118,11 @@ static void ivtv_i2c_dec(struct i2c_adapter *adapter)
 {
 	MOD_DEC_USE_COUNT;
 }
-#endif /* NEW_I2C */
+#endif
 
 static int attach_inform(struct i2c_client *client)
 {
 	struct ivtv *itv = (struct ivtv *)i2c_get_adapdata(client->adapter);
-        u32 addr = client->addr;
 	int i;
 
 	IVTV_DEBUG_I2C("i2c client attach\n");
@@ -64,27 +132,24 @@ static int attach_inform(struct i2c_client *client)
 			break;
 		}
 	}
-        if (addr == IVTV_TDA9887_I2C_ADDR)
-                itv->has_tda9887 = 1;
-        if (client->driver && client->driver->id == I2C_DRIVERID_TUNER ) {
-                if (itv->i2c_tv_tuner_addr) {
-                        // We found a second tuner. One of them is the
-                        // radio tuner TEA5767, the other is the TV tuner.
-                        if (addr == IVTV_TEA5767_I2C_ADDR) {
-                                itv->i2c_radio_tuner_addr = addr;
-                        }
-                        else {
-                                itv->i2c_radio_tuner_addr = itv->i2c_tv_tuner_addr;
-                                itv->i2c_tv_tuner_addr = addr;
-                        }
-                }
-                else {
-                        itv->i2c_tv_tuner_addr = addr;
-                }
-        }
-	IVTV_INFO("i2c attach to card #%d %s [client=%s, addr=%02x]\n",
-		   itv->num, (i < I2C_CLIENTS_MAX) ? "ok" : "failed", client->name, addr);
-
+	if (i == I2C_CLIENTS_MAX) {
+		IVTV_ERR("insufficient room for new I2C client!\n");
+	}
+/* Needs fixing later given sufficient research into the kernels handling
+   of remotes */
+#if 0
+	if (client->addr == 0x7a) {
+		struct IR_i2c *ir = i2c_get_clientdata(client);
+		IVTV_INFO("%s i2c IR detected (%s).\n",
+#if LINUX_VERSION_CODE <=  KERNEL_VERSION(2,6,15)
+				client->driver->name,ir->phys);
+#else
+		client->driver->driver.name, ir->phys);
+#endif
+		ir->get_key   = get_key_pinnacle;
+		ir->ir_codes  = ir_codes_pinnacle;
+	}
+#endif
 	return 0;
 }
 
@@ -106,6 +171,7 @@ static int detach_inform(struct i2c_client *client)
 	return 0;
 }
 
+/* Set the serial clock line to the desired state */
 static void ivtv_setscl(struct ivtv *itv, int state)
 {
 	/* write them out */
@@ -113,6 +179,7 @@ static void ivtv_setscl(struct ivtv *itv, int state)
 	writel(~state, itv->reg_mem + IVTV_REG_I2C_SETSCL_OFFSET);
 }
 
+/* Set the serial data line to the desired state */
 static void ivtv_setsda(struct ivtv *itv, int state)
 {
 	/* write them out */
@@ -120,23 +187,28 @@ static void ivtv_setsda(struct ivtv *itv, int state)
 	writel(~state & 1, itv->reg_mem + IVTV_REG_I2C_SETSDA_OFFSET);
 }
 
+/* Read the serial clock line */
 static int ivtv_getscl(struct ivtv *itv)
 {
 	return readl(itv->reg_mem + IVTV_REG_I2C_GETSCL_OFFSET) & 1;
 }
 
+/* Read the serial data line */
 static int ivtv_getsda(struct ivtv *itv)
 {
 	return readl(itv->reg_mem + IVTV_REG_I2C_GETSDA_OFFSET) & 1;
 }
 
+/* Implement a short delay by polling the serial clock line */
 static void ivtv_scldelay(struct ivtv *itv)
 {
 	int i;
+
 	for (i = 0; i < 5; ++i)
-		(void) ivtv_getscl(itv);
+		ivtv_getscl(itv);
 }
 
+/* Wait for the serial clock line to become set to a specific value */
 static int ivtv_waitscl(struct ivtv *itv, int val)
 {
 	int i;
@@ -149,6 +221,7 @@ static int ivtv_waitscl(struct ivtv *itv, int val)
 	return 0;
 }
 
+/* Wait for the serial data line to become set to a specific value */
 static int ivtv_waitsda(struct ivtv *itv, int val)
 {
 	int i;
@@ -161,6 +234,7 @@ static int ivtv_waitsda(struct ivtv *itv, int val)
 	return 0;
 }
 
+/* Wait for the slave to issue an ACK */
 static int ivtv_ack(struct ivtv *itv)
 {
 	int ret = 0;
@@ -188,6 +262,7 @@ static int ivtv_ack(struct ivtv *itv)
 	return ret;
 }
 
+/* Write a single byte to the i2c bus and wait for the slave to ACK */
 static int ivtv_sendbyte(struct ivtv *itv, unsigned char byte)
 {
 	int i, bit;
@@ -219,6 +294,8 @@ static int ivtv_sendbyte(struct ivtv *itv, unsigned char byte)
 	return ivtv_ack(itv);
 }
 
+/* Read a byte from the i2c bus and send a NACK if applicable (i.e. for the
+   final byte) */
 static int ivtv_readbyte(struct ivtv *itv, unsigned char *byte, int nack)
 {
 	int i;
@@ -249,6 +326,8 @@ static int ivtv_readbyte(struct ivtv *itv, unsigned char *byte, int nack)
 	return 0;
 }
 
+/* Issue a start condition on the i2c bus to alert slaves to prepare for
+   an address write */
 static int ivtv_start(struct ivtv *itv)
 {
 	int sda; 
@@ -274,7 +353,7 @@ static int ivtv_start(struct ivtv *itv)
 	return 0;
 }
 
-
+/* Issue a stop condition on the i2c bus to release it */
 static int ivtv_stop(struct ivtv *itv)
 {
 	int i;
@@ -310,6 +389,8 @@ static int ivtv_stop(struct ivtv *itv)
 	return 0;
 }
 
+/* Write a message to the given i2c slave.  do_stop may be 0 to prevent
+   issuing the i2c stop condition (when following with a read) */
 static int ivtv_write(struct ivtv *itv, unsigned char addr, unsigned char *data, u32 len, int do_stop)
 {
 	int retry, ret = -EREMOTEIO;
@@ -324,17 +405,15 @@ static int ivtv_write(struct ivtv *itv, unsigned char addr, unsigned char *data,
 				ret = ivtv_sendbyte(itv, data[i]);
 		}
 		if (ret != 0 || do_stop) {
-			(void) ivtv_stop(itv);
+			ivtv_stop(itv);
 		}
 	}
-	if (ret == 0) {
-		IVTV_DEBUG_I2C("i2c write to %x succeeded\n", addr);
-	} else {
+	if (ret)
 		IVTV_DEBUG_I2C("i2c write to %x failed\n", addr);
-	}
 	return ret;
 }
 
+/* Read data from the given i2c slave.  A stop condition is always issued. */
 static int ivtv_read(struct ivtv *itv, unsigned char addr, unsigned char *data, u32 len)
 {
 	int retry, ret = -EREMOTEIO;
@@ -347,210 +426,213 @@ static int ivtv_read(struct ivtv *itv, unsigned char addr, unsigned char *data,
 		for (i = 0; ret == 0 && i < len; ++i) {
 			ret = ivtv_readbyte(itv, &data[i], i == len - 1);
 		}
-		(void) ivtv_stop(itv);
+		ivtv_stop(itv);
 	}
-	if (ret == 0) {
-		IVTV_DEBUG_I2C("i2c read from %x succeeded\n", addr);
-	} else {
+	if (ret)
 		IVTV_DEBUG_I2C("i2c read from %x failed\n", addr);
-	}
 	return ret;
 }
 
+/* Kernel i2c transfer implementation.  Takes a number of messages to be read
+   or written.  If a read follows a write, this will occur without an 
+   intervening stop condition */
 static int ivtv_xfer(struct i2c_adapter *i2c_adap, struct i2c_msg *msgs, int num)
 {
 	struct ivtv *itv = i2c_get_adapdata(i2c_adap);
-	int retval = 0;
+	int retval;
 	int i;
 
 	down(&itv->i2c_bus_lock);
-	for (i = 0 ; i < num; i++) {
-		if (msgs[i].flags & I2C_M_RD) {
-			/* read */
-			IVTV_DEBUG_I2C("xfer: read %d/%d\n", i, num);
+	for (i = retval = 0; retval == 0 && i < num; i++) {
+		if (msgs[i].flags & I2C_M_RD)
 			retval = ivtv_read(itv, msgs[i].addr, msgs[i].buf, msgs[i].len);
-			if (retval < 0)
-				goto err;
-		} else {
-			/* if followed by a read, don't stop */
-			int stop;
-			stop = !(i + 1 < num && msgs[i + 1].flags == I2C_M_RD);
+		else {
+ 			/* if followed by a read, don't stop */
+ 			int stop = !(i + 1 < num && msgs[i + 1].flags == I2C_M_RD);
 
-			/* write */
-			IVTV_DEBUG_I2C("xfer: write %d/%d, stop %d\n", i, num, stop);
 			retval = ivtv_write(itv, msgs[i].addr, msgs[i].buf, msgs[i].len, stop);
-			if (retval < 0)
-				goto err;
 		}
 	}
 	up(&itv->i2c_bus_lock);
-	return num;
-
-err:
- 	up(&itv->i2c_bus_lock);
-	return retval;
+	return retval ? retval : num;
 }
 
-static u32 functionality(struct i2c_adapter *adap)
+/* Kernel i2c capabilities */
+static u32 ivtv_functionality(struct i2c_adapter *adap)
 {
 	return I2C_FUNC_I2C | I2C_FUNC_SMBUS_EMUL;
 }
 
 static struct i2c_algorithm ivtv_algo = {
 	.master_xfer   = ivtv_xfer,
-	.functionality = functionality,
+	.functionality = ivtv_functionality,
 };
 
 /* template for our-bit banger */
 static struct i2c_adapter ivtv_i2c_adap_hw_template = {
 	.name = "ivtv i2c driver",
-	.id = I2C_ALGO_BIT|I2C_HW_B_BT848,	/* algo-bit is OR'd with this */
+	.id = I2C_HW_B_CX2341X,
 	.algo = &ivtv_algo,
 	.algo_data = NULL,			/* filled from template */
 	.client_register = attach_inform,
 	.client_unregister = detach_inform,
-#ifndef NEW_I2C
-/* pre i2c-2.8.0 */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	.inc_use = ivtv_i2c_inc,	/* inc usage */
 	.dec_use = ivtv_i2c_dec,	/* dec usage */
 #else
-/* i2c-2.8.0 and later */
 	.owner = THIS_MODULE,
-#endif /* NEW_I2C */
-#ifdef LINUX26
+#ifdef I2C_ADAP_CLASS_TV_ANALOG
 	.class = I2C_ADAP_CLASS_TV_ANALOG,
-#endif /* LINUX26 */
+#endif
+#endif
 };
 
-
 static void ivtv_setscl_old(void *data, int state)
 {
-	struct ivtv *itv = (struct ivtv *)data;
+        struct ivtv *itv = (struct ivtv *)data;
 
-	if (state)
-		itv->i2c_state |= 0x01;
-	else
-		itv->i2c_state &= ~0x01;
+        if (state)
+                itv->i2c_state |= 0x01;
+        else
+                itv->i2c_state &= ~0x01;
 
-	/* write them out */
-	/* write bits are inverted */
-	writel(~itv->i2c_state, itv->reg_mem + IVTV_REG_I2C_SETSCL_OFFSET);
+        /* write them out */
+        /* write bits are inverted */
+        writel(~itv->i2c_state, itv->reg_mem + IVTV_REG_I2C_SETSCL_OFFSET);
 }
 
 static void ivtv_setsda_old(void *data, int state)
 {
-	struct ivtv *itv = (struct ivtv *)data;
+        struct ivtv *itv = (struct ivtv *)data;
 
-	if (state)
-		itv->i2c_state |= 0x01;
-	else
-		itv->i2c_state &= ~0x01;
+        if (state)
+                itv->i2c_state |= 0x01;
+        else
+                itv->i2c_state &= ~0x01;
 
-	/* write them out */
-	/* write bits are inverted */
-	writel(~itv->i2c_state, itv->reg_mem + IVTV_REG_I2C_SETSDA_OFFSET);
+        /* write them out */
+        /* write bits are inverted */
+        writel(~itv->i2c_state, itv->reg_mem + IVTV_REG_I2C_SETSDA_OFFSET);
 }
 
 static int ivtv_getscl_old(void *data)
 {
-	struct ivtv *itv = (struct ivtv *)data;
+        struct ivtv *itv = (struct ivtv *)data;
 
-	return readb(itv->reg_mem + IVTV_REG_I2C_GETSCL_OFFSET);
+        return readb(itv->reg_mem + IVTV_REG_I2C_GETSCL_OFFSET);
 }
 
 static int ivtv_getsda_old(void *data)
 {
-	struct ivtv *itv = (struct ivtv *)data;
+        struct ivtv *itv = (struct ivtv *)data;
 
-	return readb(itv->reg_mem + IVTV_REG_I2C_GETSDA_OFFSET);
+        return readb(itv->reg_mem + IVTV_REG_I2C_GETSDA_OFFSET);
 }
 
 /* template for i2c-bit-algo */
 static struct i2c_adapter ivtv_i2c_adap_template = {
-	.name = "ivtv i2c driver",
-	.id = I2C_ALGO_BIT|I2C_HW_B_BT848,	/* algo-bit is OR'd with this */
-	.algo = NULL,				/* set by i2c-algo-bit */
-	.algo_data = NULL,			/* filled from template */
-	.client_register = attach_inform,
-	.client_unregister = detach_inform,
-#ifndef NEW_I2C
-/* pre i2c-2.8.0 */
+        .name = "ivtv i2c driver",
+        .id = I2C_HW_B_BT848,      /* algo-bit is OR'd with this */
+	.algo = NULL,                           /* set by i2c-algo-bit */
+        .algo_data = NULL,                      /* filled from template */
+        .client_register = attach_inform,
+        .client_unregister = detach_inform,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	.inc_use = ivtv_i2c_inc,	/* inc usage */
 	.dec_use = ivtv_i2c_dec,	/* dec usage */
 #else
-/* i2c-2.8.0 and later */
 	.owner = THIS_MODULE,
-#endif
-#ifdef LINUX26
+#ifdef I2C_ADAP_CLASS_TV_ANALOG
 	.class = I2C_ADAP_CLASS_TV_ANALOG,
 #endif
+#endif
 };
 
 static struct i2c_algo_bit_data ivtv_i2c_algo_template = {
-	NULL,			/* ?? */
-	ivtv_setsda_old,	/* setsda function */
-	ivtv_setscl_old,	/* " */
-	ivtv_getsda_old,	/* " */
-	ivtv_getscl_old,	/* " */
-	10,			/* udelay or mdelay */
-	10,			/* whatever above isn't */
-	200			/* timeout */
+        NULL,                   /* ?? */
+        ivtv_setsda_old,        /* setsda function */
+        ivtv_setscl_old,        /* " */
+        ivtv_getsda_old,        /* " */
+        ivtv_getscl_old,        /* " */
+        10,                     /* udelay or mdelay */
+        10,                     /* whatever above isn't */
+        200                     /* timeout */
 };
 
 static struct i2c_client ivtv_i2c_client_template = {
 	.name = "ivtv internal use only",
 };
 
-
-
 static int ivtv_call_i2c_client(struct ivtv *itv, int addr, unsigned int cmd,
 				void *arg)
 {
 	struct i2c_client *client;
-	int i;
 	int retval;
+	int i;
 
 	down(&itv->i2c_lock);
-
-	IVTV_DEBUG_I2C("call_i2c_client\n");
+	IVTV_DEBUG_I2C("call_i2c_client addr=%02x\n", addr);
 	for (i = 0; i < I2C_CLIENTS_MAX; i++) {
 		client = itv->i2c_clients[i];
 		if (client == NULL) {
-			IVTV_DEBUG_I2C("no client %d found\n", i);
 			continue;
 		}
 		if (client->driver->command == NULL) {
-			IVTV_DEBUG_I2C(
-				   "i2c client with no command interface: "
-				   "0x%02x\n", (u8) client->addr);
 			continue;
 		}
 		if (addr == client->addr) {
-			IVTV_DEBUG_I2C(
-				   "using client %d, addr 0x%02x\n", i,
-				   (u8) addr);
 			retval = client->driver->command(client, cmd, arg);
 			up(&itv->i2c_lock);
 			return retval;
 		}
-		IVTV_DEBUG_I2C(
-			   "skipping client %d, addr 0x%02x\n", i,
-			   (u8) client->addr);
 	}
-	IVTV_DEBUG_WARN("i2c client addr: 0x%02x not found for command 0x%x!\n",
-		   (u8) addr, cmd);
+	IVTV_ERR("i2c addr 0x%02x not found for command 0x%x!\n", addr, cmd);
 	up(&itv->i2c_lock);
 	return -ENODEV;
 }
 
-int ivtv_wm8775(struct ivtv *itv, unsigned int cmd, void *arg)
+/* Find the i2c device matching the IVTV_HW_ flag and return
+   its i2c address or -ENODEV if no matching device was found. */
+int ivtv_i2c_hw_addr(struct ivtv *itv, u32 hw)
 {
-	return ivtv_call_i2c_client(itv, IVTV_WM8775_I2C_ADDR, cmd, arg);
+	struct i2c_client *client;
+	int i = long_log2(hw);
+	int id = 0;
+	int retval = -ENODEV;
+
+	if (i < ARRAY_SIZE(hw_driverids))
+		id = hw_driverids[i];
+	down(&itv->i2c_lock);
+	for (i = 0; i < I2C_CLIENTS_MAX; i++) {
+		client = itv->i2c_clients[i];
+		if (client == NULL)
+			continue;
+		if (id == client->driver->id) {
+			retval = client->addr;
+			break;
+		}
+	}
+	up(&itv->i2c_lock);
+	return retval;
 }
 
-int ivtv_cs53l32a(struct ivtv *itv, unsigned int cmd, void *arg)
+/* Calls i2c device based on IVTV_HW_ flag. If hw == 0, then do nothing.
+   If hw == IVTV_HW_GPIO then call the gpio handler. */
+int ivtv_i2c_hw(struct ivtv *itv, u32 hw, unsigned int cmd, void *arg)
 {
-	return ivtv_call_i2c_client(itv, IVTV_CS53L32A_I2C_ADDR, cmd, arg);
+	int addr;
+
+	if (hw == IVTV_HW_GPIO)
+		return ivtv_gpio(itv, cmd, arg);
+	if (hw == 0)
+		return 0;
+
+	addr = ivtv_i2c_hw_addr(itv, hw);
+	if (addr < 0) {
+		IVTV_ERR("i2c hardware 0x%08x not found for command 0x%x!\n", hw, cmd);
+		return addr;
+	}
+	return ivtv_call_i2c_client(itv, addr, cmd, arg);
 }
 
 int ivtv_cx25840(struct ivtv *itv, unsigned int cmd, void *arg)
@@ -568,49 +650,53 @@ int ivtv_saa7127(struct ivtv *itv, unsigned int cmd, void *arg)
 	return ivtv_call_i2c_client(itv, IVTV_SAA7127_I2C_ADDR, cmd, arg);
 }
 
+int ivtv_saa717x(struct ivtv *itv, unsigned int cmd, void *arg)
+{
+	return ivtv_call_i2c_client(itv, IVTV_SAA717x_I2C_ADDR, cmd, arg);
+}
+
 int ivtv_msp34xx(struct ivtv *itv, unsigned int cmd, void *arg)
 {
 	return ivtv_call_i2c_client(itv, IVTV_MSP3400_I2C_ADDR, cmd, arg);
 }
 
-int ivtv_radio_tuner(struct ivtv *itv, unsigned int cmd, void *arg)
+int ivtv_upd64031a(struct ivtv *itv, unsigned int cmd, void *arg)
 {
-        u32 addr = itv->i2c_radio_tuner_addr ? itv->i2c_radio_tuner_addr :
-                                               itv->i2c_tv_tuner_addr;
-
-	return ivtv_call_i2c_client(itv, addr, cmd, arg);
+	return ivtv_call_i2c_client(itv, IVTV_UPD64031A_I2C_ADDR, cmd, arg);
 }
 
-int ivtv_tv_tuner(struct ivtv *itv, unsigned int cmd, void *arg)
+int ivtv_upd64083(struct ivtv *itv, unsigned int cmd, void *arg)
 {
-	return ivtv_call_i2c_client(itv, itv->i2c_tv_tuner_addr, cmd, arg);
+	return ivtv_call_i2c_client(itv, IVTV_UPD64083_I2C_ADDR, cmd, arg);
 }
 
-int ivtv_tda9887(struct ivtv *itv, unsigned int cmd, void *arg)
+/* broadcast cmd for all I2C clients and for the gpio subsystem */
+void ivtv_call_i2c_clients(struct ivtv *itv, unsigned int cmd, void *arg)
 {
-	return ivtv_call_i2c_client(itv, IVTV_TDA9887_I2C_ADDR, cmd, arg);
-}
-
-int ivtv_hauppauge(struct ivtv *itv, unsigned int cmd, void *arg)
-{
-	return ivtv_call_i2c_client(itv, IVTV_HAUPPAUGE_I2C_ADDR, cmd, arg);
+	if (itv->i2c_adap.algo == NULL) {
+		IVTV_ERR("adapter is not set");
+		return;
+	}
+	i2c_clients_command(&itv->i2c_adap, cmd, arg);
+	if (itv->hw_flags & IVTV_HW_GPIO)
+		ivtv_gpio(itv, cmd, arg);
 }
 
 /* init + register i2c algo-bit adapter */
 int __devinit init_ivtv_i2c(struct ivtv *itv)
 {
 	IVTV_DEBUG_I2C("i2c init\n");
- 
- 	if (itv->options.newi2c) {
- 		memcpy(&itv->i2c_adap, &ivtv_i2c_adap_hw_template,
- 		       sizeof(struct i2c_adapter));
- 	} else {
- 		memcpy(&itv->i2c_adap, &ivtv_i2c_adap_template,
- 		       sizeof(struct i2c_adapter));
- 		memcpy(&itv->i2c_algo, &ivtv_i2c_algo_template,
- 		       sizeof(struct i2c_algo_bit_data));
- 		itv->i2c_algo.data = itv;
-		itv->i2c_adap.algo_data = &itv->i2c_algo;
+
+	if (itv->options.newi2c) {
+		memcpy(&itv->i2c_adap, &ivtv_i2c_adap_hw_template,
+	 	       sizeof(struct i2c_adapter));
+	} else {
+                memcpy(&itv->i2c_adap, &ivtv_i2c_adap_template,
+                       sizeof(struct i2c_adapter));
+                memcpy(&itv->i2c_algo, &ivtv_i2c_algo_template,
+                       sizeof(struct i2c_algo_bit_data));
+                itv->i2c_algo.data = itv;
+                itv->i2c_adap.algo_data = &itv->i2c_algo;
 	}
 
 	sprintf(itv->i2c_adap.name + strlen(itv->i2c_adap.name), " #%d",
@@ -622,22 +708,22 @@ int __devinit init_ivtv_i2c(struct ivtv *itv)
 	itv->i2c_client.adapter = &itv->i2c_adap;
   
 	IVTV_DEBUG_I2C("setting scl and sda to 1\n");
-	ivtv_setscl_old(itv, 1);
-	ivtv_setsda_old(itv, 1);
+	ivtv_setscl(itv, 1);
+	ivtv_setsda(itv, 1);
 
-	if (itv->options.newi2c)
-		return i2c_add_adapter(&itv->i2c_adap);
-	else
-		return i2c_bit_add_bus(&itv->i2c_adap);
+        if (itv->options.newi2c)
+                return i2c_add_adapter(&itv->i2c_adap);
+        else
+                return i2c_bit_add_bus(&itv->i2c_adap);
 }
 
 void __devexit exit_ivtv_i2c(struct ivtv *itv)
 {
 	IVTV_DEBUG_I2C("i2c exit\n");
 
-	if (itv->options.newi2c) {
-		i2c_del_adapter(&itv->i2c_adap);
-	} else {
-		i2c_bit_del_bus(&itv->i2c_adap);
-	}
+        if (itv->options.newi2c) {
+                i2c_del_adapter(&itv->i2c_adap);
+        } else {
+                i2c_bit_del_bus(&itv->i2c_adap);
+        }
 }
