@@ -33,15 +33,10 @@
 #include "ivtv-gpio.h"
 #include "ivtv-controls.h"
 #include "ivtv-cards.h"
-#include "v4l2-common.h"
-#include "audiochip.h"
-#include "cx25840.h"
-
-/* from v4l1_compat.c */
-extern int
-ivtv_compat_translate_ioctl(struct inode *inode,
-			    struct file *file,
-			    int cmd, void *arg, v4l2_kioctl drv);
+#include "ivtv-kthreads.h"
+#include <media/saa7127.h>
+#include <media/tveeprom.h>
+#include <linux/i2c-id.h>
 
 u16 service2vbi(int type)
 {
@@ -133,34 +128,36 @@ u16 get_service_set(struct v4l2_sliced_vbi_format *fmt)
         return set;
 }
 
-static const struct v4l2_standard ivtv_stds[] = {
-        {
-                .index = 0,
-                .id = V4L2_STD_NTSC,
-                .name = "NTSC",
-                .frameperiod = {.numerator = 1001,
-                        .denominator = 30000},
-                .framelines = 525,
-                .reserved = {0, 0, 0, 0}
-        }, {
-                .index = 1,
-                .id = V4L2_STD_PAL,
-                .name = "PAL",
-                .frameperiod = {.numerator = 1,
-                        .denominator = 25},
-                .framelines = 625,
-                .reserved = {0, 0, 0, 0}
-        }, {
-                .index = 2,
-                .id = V4L2_STD_SECAM,
-                .name = "SECAM",
-                .frameperiod = {.numerator = 1,
-                        .denominator = 25},
-                .framelines = 625,
-                .reserved = {0, 0, 0, 0}
-        }
+static const struct {
+	v4l2_std_id  std;
+	char        *name;
+} enum_stds[] = {
+	{ V4L2_STD_PAL_BG | V4L2_STD_PAL_H, "PAL-BGH" },
+	{ V4L2_STD_PAL_DK,    "PAL-DK"    },
+	{ V4L2_STD_PAL_I,     "PAL-I"     },
+	{ V4L2_STD_PAL_M,     "PAL-M"     },
+	{ V4L2_STD_PAL_N,     "PAL-N"     },
+	{ V4L2_STD_PAL_Nc,    "PAL-Nc"    },
+	{ V4L2_STD_SECAM_B | V4L2_STD_SECAM_G | V4L2_STD_SECAM_H, "SECAM-BGH" },
+	{ V4L2_STD_SECAM_DK,  "SECAM-DK"  },
+	{ V4L2_STD_SECAM_L,   "SECAM-L"   },
+	{ V4L2_STD_SECAM_LC,  "SECAM-L'"  },
+	{ V4L2_STD_NTSC_M,    "NTSC-M"    },
+	{ V4L2_STD_NTSC_M_JP, "NTSC-J"    },
+	{ V4L2_STD_NTSC_M_KR, "NTSC-K"    },
+};
+
+static const struct v4l2_standard ivtv_std_60hz = 
+{
+	.frameperiod = {.numerator = 1001, .denominator = 30000},
+	.framelines = 525,
+};
+
+static const struct v4l2_standard ivtv_std_50hz = 
+{
+	.frameperiod = {.numerator = 1, .denominator = 25},
+	.framelines = 625,
 };
-static const int ivtv_stds_size = sizeof(ivtv_stds) / sizeof(ivtv_stds[0]);
 
 static void ivtv_stream_off(struct ivtv *itv, struct ivtv_stream *stream)
 {
@@ -270,7 +267,7 @@ static int ivtv_s_speed(struct ivtv *itv, struct ivtv_speed *speed)
 	/* If not decoding, just change speed setting */
 	if (atomic_read(&itv->decoding) == 0) {
 		if (ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-			     IVTV_API_DEC_PLAYBACK_SPEED, &result, 7, &data[0]))
+			     CX2341X_DEC_SET_PLAYBACK_SPEED, &result, 7, &data[0]))
 		{
 			IVTV_DEBUG_WARN(
 				   "DEC: error changing speed\n");
@@ -282,7 +279,7 @@ static int ivtv_s_speed(struct ivtv *itv, struct ivtv_speed *speed)
 
         /* Stop all DMA and decoding activity */
 	if (test_bit(IVTV_F_T_DEC_DONE, &itv->t_flags)) {
-        	if (ivtv_vapi(itv, IVTV_API_DEC_PAUSE_PLAYBACK, 1, 0)) {
+        	if (ivtv_vapi(itv, CX2341X_DEC_PAUSE_PLAYBACK, 1, 0)) {
                 	IVTV_DEBUG_WARN("DEC: error pausing\n");
         	}
 	}
@@ -303,7 +300,7 @@ static int ivtv_s_speed(struct ivtv *itv, struct ivtv_speed *speed)
 
 	/* Change Speed safely */
 	if (ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		     IVTV_API_DEC_PLAYBACK_SPEED, &result, 7, &data[0])) {
+		     CX2341X_DEC_SET_PLAYBACK_SPEED, &result, 7, &data[0])) {
 		IVTV_DEBUG_WARN("DEC: error changing speed\n");
 		return (int)result;
 	}
@@ -355,11 +352,9 @@ static int ivtv_itvc(struct ivtv *itv, unsigned int cmd, void *arg)
                 return -EINVAL;
 
 	spin_lock_irqsave(&ivtv_cards_lock, flags);
-	if (cmd == ITVC_GET_REG) {
+	if (cmd == IVTV_IOC_G_ITVC_REG) {
 		regs->val = readl(regs->reg + reg_start);
 	} else {
-//		printk("Write Register: 0x%08x = 0x%08x\n",
-//		       regs->reg, regs->val);
 		writel(regs->val, regs->reg + reg_start);
 	}
 	spin_unlock_irqrestore(&ivtv_cards_lock, flags);
@@ -472,7 +467,7 @@ static int ivtv_get_fmt(struct ivtv *itv, int streamtype, struct v4l2_format *fm
                         break;
                 }
 
-                itv->card->video_dec_func(itv, VIDIOC_G_FMT, fmt);
+                itv->video_dec_func(itv, VIDIOC_G_FMT, fmt);
                 vbifmt->service_set = get_service_set(vbifmt);
                 break;
         }
@@ -510,7 +505,7 @@ static int ivtv_try_or_set_fmt(struct ivtv *itv, int streamtype,
 		pix.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
                 pix.fmt.pix.width = itv->width = fmt->fmt.pix.width;
                 pix.fmt.pix.height = itv->height = fmt->fmt.pix.height;
-                itv->card->video_dec_func(itv, VIDIOC_S_FMT, &pix);
+                itv->video_dec_func(itv, VIDIOC_S_FMT, &pix);
 
                 if (itv->codec.stream_type == IVTV_STREAM_MPEG1 ||
                     itv->codec.stream_type == IVTV_STREAM_VCD) {
@@ -530,7 +525,7 @@ static int ivtv_try_or_set_fmt(struct ivtv *itv, int streamtype,
                         return -EBUSY;
                 }
 		itv->vbi_sliced_in->service_set = 0;
-                itv->card->video_dec_func(itv, VIDIOC_S_FMT, &itv->vbi_in);
+                itv->video_dec_func(itv, VIDIOC_S_FMT, &itv->vbi_in);
                 return ivtv_get_fmt(itv, streamtype, fmt);
         }
        
@@ -565,13 +560,13 @@ static int ivtv_try_or_set_fmt(struct ivtv *itv, int streamtype,
         if (atomic_read(&itv->capturing) > 0 && itv->vbi_sliced_in->service_set == 0) {
                 return -EBUSY;
         }
-        itv->card->video_dec_func(itv, VIDIOC_S_FMT, fmt);
+        itv->video_dec_func(itv, VIDIOC_S_FMT, fmt);
 	memcpy(itv->vbi_sliced_in, vbifmt, sizeof(*itv->vbi_sliced_in));
         return 0;
 }
 
 
-int ivtv_internal_ioctls(struct ivtv *itv, int streamtype, unsigned int cmd,
+static int ivtv_internal_ioctls(struct ivtv *itv, int streamtype, unsigned int cmd,
 			 void *arg)
 {
 	struct v4l2_register *reg = arg;
@@ -639,24 +634,32 @@ int ivtv_internal_ioctls(struct ivtv *itv, int streamtype, unsigned int cmd,
 	/* ioctls to allow direct access to the ITVC registers for testing */
 	case IVTV_IOC_G_ITVC_REG:
 		IVTV_DEBUG_IOCTL("IVTV_IOC_G_ITVC_REG\n");
-		return ivtv_itvc(itv, ITVC_GET_REG, arg);
+		return ivtv_itvc(itv, cmd, arg);
 
 	case IVTV_IOC_S_ITVC_REG:
 		IVTV_DEBUG_IOCTL("IVTV_IOC_S_ITVC_REG\n");
-		return ivtv_itvc(itv, ITVC_SET_REG, arg);
+		return ivtv_itvc(itv, cmd, arg);
 
 	/* ioctls to allow direct access to the decoder registers for testing */
 	case IVTV_IOC_G_DECODER_REG:
 		IVTV_DEBUG_IOCTL("IVTV_IOC_G_DECODER_REG\n");
-		reg->i2c_id = (itv->hw_flags & IVTV_HW_CX25840) ?
-			I2C_DRIVERID_CX25840 : I2C_DRIVERID_SAA711X;
-		return itv->card->video_dec_func(itv, VIDIOC_INT_G_REGISTER, reg);
+		if (itv->hw_flags & IVTV_HW_CX25840)
+			reg->i2c_id = I2C_DRIVERID_CX25840;
+		else if (itv->hw_flags & IVTV_HW_SAA711X)
+			reg->i2c_id = I2C_DRIVERID_SAA711X;
+		else
+			reg->i2c_id = I2C_DRIVERID_SAA717X;
+		return itv->video_dec_func(itv, VIDIOC_INT_G_REGISTER, reg);
 
 	case IVTV_IOC_S_DECODER_REG:
 		IVTV_DEBUG_IOCTL("IVTV_IOC_S_DECODER_REG\n");
-		reg->i2c_id = (itv->hw_flags & IVTV_HW_CX25840) ?
-			I2C_DRIVERID_CX25840 : I2C_DRIVERID_SAA711X;
-		return itv->card->video_dec_func(itv, VIDIOC_INT_S_REGISTER, reg);
+		if (itv->hw_flags & IVTV_HW_CX25840)
+			reg->i2c_id = I2C_DRIVERID_CX25840;
+		else if (itv->hw_flags & IVTV_HW_SAA711X)
+			reg->i2c_id = I2C_DRIVERID_SAA711X;
+		else
+			reg->i2c_id = I2C_DRIVERID_SAA717X;
+		return itv->video_dec_func(itv, VIDIOC_INT_S_REGISTER, reg);
 
 	/* ioctls to allow direct access to the encoder registers for testing */
 	case IVTV_IOC_G_ENCODER_REG:
@@ -675,20 +678,21 @@ int ivtv_internal_ioctls(struct ivtv *itv, int streamtype, unsigned int cmd,
 
 		IVTV_DEBUG_IOCTL("ZCOUNT\n");
 
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 		while (MOD_IN_USE)
 			MOD_DEC_USE_COUNT;
 
 		MOD_INC_USE_COUNT;
-#endif /* LINUX26 */
+#endif
 		break;
 	}
 
-	case IVTV_IOC_S_MSP_MATRIX:{
-		struct ivtv_msp_matrix *matrix = arg;
+	case IVTV_IOC_S_AUDIO_ROUTING: {
+		struct v4l2_routing *route = arg;
 
-		IVTV_DEBUG_IOCTL("IVTV_IOC_S_MSP_MATRIX\n");
-		return ivtv_audio_set_matrix(itv, matrix);
+		IVTV_DEBUG_IOCTL("IVTV_IOC_S_AUDIO_ROUTING\n");
+		ivtv_audio_set_route(itv, route);
+		break;
 	}
 
 	case IVTV_IOC_G_DEBUG_LEVEL:
@@ -722,28 +726,19 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 	switch (cmd) {
 	case VIDIOC_QUERYCAP:{
 		struct v4l2_capability *vcap = arg;
-		memset(vcap, 0, sizeof(*vcap));
 
 		IVTV_DEBUG_IOCTL("VIDIOC_QUERYCAP\n");
 
-		/* driver name */
-		strcpy(vcap->driver, IVTV_DRIVER_NAME);
-
-		/* card type */
-		strcpy(vcap->card, itv->card_name);
-
-		/* bus info... */
-                strcpy(vcap->bus_info, pci_name(itv->dev));
-
-		/* version */
-		vcap->version = IVTV_DRIVER_VERSION;
-
-		/* capabilities */
-		vcap->capabilities = itv->v4l2_cap;
+		memset(vcap, 0, sizeof(*vcap));
+		strcpy(vcap->driver, IVTV_DRIVER_NAME);     /* driver name */
+		strcpy(vcap->card, itv->card_name); 	    /* card type */
+                strcpy(vcap->bus_info, pci_name(itv->dev)); /* bus info... */
+		vcap->version = IVTV_DRIVER_VERSION; 	    /* version */
+		vcap->capabilities = itv->v4l2_cap; 	    /* capabilities */
 
 		/* reserved.. must set to 0! */
-		vcap->reserved[0] = vcap->reserved[1] = vcap->reserved[2] =
-		    vcap->reserved[3] = 0;
+		vcap->reserved[0] = vcap->reserved[1] = 
+			vcap->reserved[2] = vcap->reserved[3] = 0;
 		break;
 	}
 
@@ -752,32 +747,15 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 		IVTV_DEBUG_IOCTL("VIDIOC_ENUMAUDIO\n");
 
-		if (ivtv_get_audio_input(itv, vin->index) == NULL) {
-			return -EINVAL;
-		}
-
-		/* set it to defaults from our table */
-		*vin = *ivtv_get_audio_input(itv, vin->index);
-		break;
+		return ivtv_get_audio_input(itv, vin->index, vin);
 	}
 
 	case VIDIOC_G_AUDIO:{
 		struct v4l2_audio *vin = arg;
 
 		IVTV_DEBUG_IOCTL("VIDIOC_G_AUDIO\n");
-		if (test_bit(IVTV_F_I_RADIO_USER, &itv->i_flags)) {
-			vin->index = itv->audio_input_radio;
-			/* msp3400 input 2 maps to audio input 0 (tuner) */
-			if (vin->index == 2)
-				vin->index = 0;
-		} else {
-			vin->index = itv->audio_input_tv;
-		}
-
-		if (ivtv_get_audio_input(itv, vin->index) == NULL)
-			return -EINVAL;
-		*vin = *ivtv_get_audio_input(itv, vin->index);
-		break;
+		vin->index = itv->audio_input;
+		return ivtv_get_audio_input(itv, vin->index, vin);
 	}
 
 	case VIDIOC_S_AUDIO:{
@@ -785,16 +763,9 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 		IVTV_DEBUG_IOCTL("VIDIOC_S_AUDIO\n");
 
-		if (ivtv_get_audio_input(itv, vout->index) == NULL)
+		if (vout->index >= itv->nof_audio_inputs)
 			return -EINVAL;
-		if (test_bit(IVTV_F_I_RADIO_USER, &itv->i_flags)) {
-			/* audio input 0 (tuner) maps to msp3400 input 2 for the radio */
-			if (vout->index == 0)
-				vout->index = 2;
-			itv->audio_input_radio = vout->index;
-		} else {
-			itv->audio_input_tv = vout->index;
-		}
+		itv->audio_input = vout->index;
 		ivtv_audio_set_io(itv);
 		break;
 	}
@@ -804,13 +775,8 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 		IVTV_DEBUG_IOCTL("VIDIOC_ENUMAUDOUT\n");
 
-		if (ivtv_get_audio_output(itv, vin->index) == NULL) {
-			return -EINVAL;
-		}
-
 		/* set it to defaults from our table */
-		*vin = *ivtv_get_audio_output(itv, vin->index);
-		break;
+		return ivtv_get_audio_output(itv, vin->index, vin);
 	}
 
 	case VIDIOC_G_AUDOUT:{
@@ -818,10 +784,7 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 		IVTV_DEBUG_IOCTL("VIDIOC_G_AUDOUT\n");
 		vin->index = 0;
-		if (ivtv_get_audio_output(itv, vin->index) == NULL)
-			return -EINVAL;
-		*vin = *ivtv_get_audio_output(itv, vin->index);
-		break;
+		return ivtv_get_audio_output(itv, vin->index, vin);
 	}
 
 	case VIDIOC_S_AUDOUT:{
@@ -829,10 +792,7 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 		IVTV_DEBUG_IOCTL("VIDIOC_S_AUDOUT\n");
 
-		if (ivtv_get_audio_output(itv, vout->index) == NULL)
-			return -EINVAL;
-		/* Do nothing. */
-		break;
+		return ivtv_get_audio_output(itv, vout->index, vout);
 	}
 
 	case VIDIOC_ENUMINPUT:{
@@ -840,17 +800,8 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 		IVTV_DEBUG_IOCTL("VIDIOC_ENUMINPUT\n");
 
-		if (ivtv_get_input(itv, vin->index) == NULL) {
-			return -EINVAL;
-		}
-
 		/* set it to defaults from our table */
-		*vin = *ivtv_get_input(itv, vin->index);
-
-		/* The tuner is a special case */
-		if (vin->type == V4L2_INPUT_TYPE_TUNER)
-			vin->std = itv->tuner_std;
-		break;
+		return ivtv_get_input(itv, vin->index, vin);
 	}
 
 	case VIDIOC_ENUMOUTPUT:{
@@ -858,14 +809,7 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 		IVTV_DEBUG_IOCTL("VIDIOC_ENUMOUTPUT\n");
 
-		if (ivtv_get_output(itv, vout->index) == NULL) {
-			return -EINVAL;
-		}
-
-		/* set it to defaults from our table */
-		*vout = *ivtv_get_output(itv, vout->index);
-
-		break;
+		return ivtv_get_output(itv, vout->index, vout);
 	}
 
 	case VIDIOC_TRY_FMT:
@@ -902,41 +846,27 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 		IVTV_DEBUG_IOCTL("VIDIOC_S_INPUT\n");
 
-		if (ivtv_get_input(itv, inp) == NULL)
+		if (inp >= itv->nof_inputs)
 			return -EINVAL;
 
 		if (inp == itv->active_input) {
 			IVTV_DEBUG_INFO("Input unchanged\n");
-		} else {
-			IVTV_DEBUG_INFO(
-				   "Changing input from %d to %d\n",
-				   itv->active_input, inp);
-
-			itv->active_input = inp;
-			/* Set the audio input to whatever is appropriate for the
-			   input type. */
-			if (ivtv_get_input(itv, inp)->type ==
-			    V4L2_INPUT_TYPE_TUNER) {
-				itv->audio_input_tv = 0;	/* TV tuner */
-                        } else {
-                                if (itv->card->type == IVTV_CARD_PVR_150 ||
-                                    itv->card->type == IVTV_CARD_PG600) {
-                                        itv->audio_input_tv = 2; /* Line in */
-                                } else {
-                                        itv->audio_input_tv = 1; /* Line in */
-                                }
-                        }
-
-			/* prevent others from messing with the streams until
-			   we're finished changing inputs. */
-			ivtv_mute(itv);
-
-			itv->card->video_dec_func(itv, VIDIOC_S_INPUT, &inp);
-
-			/* Select new audio input */
-			ivtv_audio_set_io(itv);
-			ivtv_unmute(itv);
+			break;
 		}
+		IVTV_DEBUG_INFO("Changing input from %d to %d\n",
+			       	itv->active_input, inp);
+
+		itv->active_input = inp;
+		/* Set the audio input to whatever is appropriate for the
+		   input type. */
+		itv->audio_input = itv->card->video_inputs[inp].audio_index;
+
+		/* prevent others from messing with the streams until
+		   we're finished changing inputs. */
+		ivtv_mute(itv);
+		ivtv_video_set_io(itv);
+		ivtv_audio_set_io(itv);
+		ivtv_unmute(itv);
 		break;
 	}
 
@@ -951,10 +881,11 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 	case VIDIOC_S_OUTPUT:{
 		int outp = *(int *)arg;
+		struct v4l2_routing route;
 
 		IVTV_DEBUG_IOCTL("VIDIOC_S_OUTPUT\n");
 
-		if (ivtv_get_output(itv, outp) == NULL)
+		if (outp >= itv->card->nof_outputs)
 			return -EINVAL;
 
 		if (outp == itv->active_output) {
@@ -965,8 +896,9 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 			   itv->active_output, outp);
 
 		itv->active_output = outp;
-
-		ivtv_saa7127(itv, VIDIOC_S_OUTPUT, &outp);
+		route.input = SAA7127_INPUT_TYPE_NORMAL;
+		route.output = itv->card->video_outputs[outp].video_output;
+		ivtv_saa7127(itv, VIDIOC_INT_S_VIDEO_ROUTING, &route);
 		break;
 	}
 
@@ -977,18 +909,7 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 		if (vf->tuner != 0)
 			return -EINVAL;
-		if (test_bit(IVTV_F_I_RADIO_USER, &itv->i_flags)) {
-			vf->type = V4L2_TUNER_RADIO;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 13)
-			vf->frequency = itv->freq_radio / 1000;
-#else
-			vf->frequency = itv->freq_radio;
-#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 13) */
-		} else {
-			vf->type = V4L2_TUNER_ANALOG_TV;
-			vf->frequency = itv->freq_tv;
-		}
-		memset(vf->reserved, 0, sizeof(vf->reserved));
+		ivtv_call_i2c_clients(itv, cmd, arg);
 		break;
 	}
 
@@ -1000,40 +921,27 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		if (vf.tuner != 0)
 			return -EINVAL;
 
-		if (test_bit(IVTV_F_I_RADIO_USER, &itv->i_flags)) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 13)
-                        vf.frequency *= 1000;
-#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 13) */
-			itv->freq_radio = vf.frequency;
-			vf.type = V4L2_TUNER_RADIO; /* not always set, and needed by IF demod */
-		} else {
-			itv->freq_tv = vf.frequency;
-			vf.type = V4L2_TUNER_ANALOG_TV;
-		}
-
 		ivtv_mute(itv);
 		IVTV_DEBUG_INFO("v4l2 ioctl: set frequency %d\n", vf.frequency);
-                if (vf.type == V4L2_TUNER_ANALOG_TV) {
-		        ivtv_tv_tuner(itv, VIDIOC_S_FREQUENCY, &vf);
-                }
-                else {
-		        ivtv_radio_tuner(itv, VIDIOC_S_FREQUENCY, &vf);
-                }
-		if (itv->options.tda9887 == 0) ivtv_tda9887(itv, VIDIOC_S_FREQUENCY, &vf);
-		ivtv_audio_freq_changed(itv);
+		ivtv_call_i2c_clients(itv, cmd, &vf);
 		ivtv_unmute(itv);
 		break;
 	}
 
 	case VIDIOC_ENUMSTD:{
 		struct v4l2_standard *vs = arg;
+		int idx = vs->index;
 
 		IVTV_DEBUG_IOCTL("VIDIOC_ENUMSTD\n");
 
-		if (vs->index < 0 || vs->index >= ivtv_stds_size)
+		if (idx < 0 || idx >= ARRAY_SIZE(enum_stds))
 			return -EINVAL;
 
-		*vs = ivtv_stds[vs->index];
+		*vs = (enum_stds[idx].std & V4L2_STD_525_60) ? 
+				ivtv_std_60hz : ivtv_std_50hz;
+		vs->index = idx;
+		vs->id = enum_stds[idx].std;
+		strcpy(vs->name, enum_stds[idx].name);
 		break;
 	}
 
@@ -1045,18 +953,13 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 	case VIDIOC_S_STD:{
 		v4l2_std_id std = *(v4l2_std_id *) arg;
-		int x;
 
 		IVTV_DEBUG_IOCTL("VIDIOC_S_STD\n");
 
-		for (x = 0; x < ivtv_stds_size; x++) {
-			if (ivtv_stds[x].id & std)
-				break;
-		}
-		if (x == ivtv_stds_size)
+		if ((std & V4L2_STD_ALL) == 0)
 			return -EINVAL;
 
-		if (ivtv_stds[x].id == itv->std)
+		if (std == itv->std)
 			break;
 
 		/* Prevent others from messing around with streams while
@@ -1072,9 +975,7 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 			return -EBUSY;
 		}
 
-		IVTV_DEBUG_INFO(
-			   "Switching standard to %s.\n", ivtv_stds[x].name);
-		itv->std = ivtv_stds[x].id;
+		itv->std = std;
 		itv->width = 720;
 		if (itv->std & V4L2_STD_525_60) {
                         itv->is_50hz = 0;
@@ -1094,38 +995,20 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		if (itv->hw_flags & IVTV_HW_CX25840) {
 			itv->vbi_sliced_decoder_line_size = itv->is_60hz ? 272 : 284;
 		}
+		IVTV_DEBUG_INFO("Switching standard to %llx.\n", itv->std);
 
 		/* Tuner */
-		ivtv_tv_tuner(itv, VIDIOC_S_STD, &itv->std);
-
-		/* Tuner Audio */
-		ivtv_audio_set_std(itv);
-
-		/* Microtune support */
-		if (itv->options.tda9887 > 0) {
-			unsigned int c = TDA9887_INTERCARRIER;
-
-			ivtv_tda9887(itv, TDA9887_SET_CONFIG, &c);
-		} else if (itv->options.tda9887 == 0) {
-			ivtv_tda9887(itv, VIDIOC_S_STD, &itv->std);
-		}
-
-		/* Digitizer */
-		itv->card->video_dec_func(itv, VIDIOC_S_STD, &itv->std);
+		ivtv_call_i2c_clients(itv, VIDIOC_S_STD, &itv->std);
 
 		if (itv->v4l2_cap & V4L2_CAP_VIDEO_OUTPUT) {
-			/* Encoder */
-			ivtv_saa7127(itv, VIDIOC_S_STD, &itv->std);
-
 			/* set display standard */
-			ivtv_vapi(itv, IVTV_API_DEC_DISP_STANDARD, 1,
-				  !(itv->std & V4L2_STD_NTSC));
+			ivtv_vapi(itv, CX2341X_DEC_SET_STANDARD, 1, itv->is_50hz);
 		}
 		up(&stream->mlock);
 		break;
 	}
 
-	case VIDIOC_S_TUNER:{	/* Setting tuner can only set audio mode */
+	case VIDIOC_S_TUNER: {	/* Setting tuner can only set audio mode */
 		struct v4l2_tuner *vt = arg;
 
 		IVTV_DEBUG_IOCTL("VIDIOC_S_TUNER\n");
@@ -1133,16 +1016,12 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		if (vt->index != 0)
 			return -EINVAL;
 
-		if (test_bit(IVTV_F_I_RADIO_USER, &itv->i_flags)) {
-			itv->audmode_radio = vt->audmode;
-		} else {
-			itv->audmode_tv = vt->audmode;
-		}
-		ivtv_audio_set_audmode(itv, vt->audmode);
+		ivtv_call_i2c_clients(itv, VIDIOC_S_TUNER, vt);
+		ivtv_dualwatch_set_enabled(itv, vt->audmode == V4L2_TUNER_MODE_LANG1_LANG2);
 		break;
 	}
 
-	case VIDIOC_G_TUNER:{
+	case VIDIOC_G_TUNER: {
 		struct v4l2_tuner *vt = arg;
 
 		IVTV_DEBUG_IOCTL("VIDIOC_G_TUNER\n");
@@ -1150,40 +1029,16 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		if (vt->index != 0)
 			return -EINVAL;
 
-		vt->afc = 0;
-		memset(vt->reserved, 0, sizeof(vt->reserved));
+		memset(vt, 0, sizeof(*vt));
+		ivtv_call_i2c_clients(itv, VIDIOC_G_TUNER, vt);
 
 		if (test_bit(IVTV_F_I_RADIO_USER, &itv->i_flags)) {
-			struct v4l2_tuner sig;
-
 			strcpy(vt->name, "ivtv Radio Tuner");
 			vt->type = V4L2_TUNER_RADIO;
-			vt->capability = V4L2_TUNER_CAP_STEREO;
-			/* Japan:          76.0 MHz -  89.9 MHz
-			   Western Europe: 87.5 MHz - 108.0 MHz
-			   Russia:         65.0 MHz - 108.0 MHz */
-			vt->rangelow = 65 * 16;
-			vt->rangehigh = 108 * 16;
-			ivtv_radio_tuner(itv, VIDIOC_G_TUNER, &sig);
-			vt->signal = sig.signal;
-			vt->audmode = itv->audmode_radio;
 		} else {
-			struct v4l2_tuner sig;
-
 			strcpy(vt->name, "ivtv TV Tuner");
 			vt->type = V4L2_TUNER_ANALOG_TV;
-			vt->capability =
-			    V4L2_TUNER_CAP_NORM | V4L2_TUNER_CAP_STEREO |
-			    V4L2_TUNER_CAP_LANG1 | V4L2_TUNER_CAP_LANG2;
-			vt->rangelow = 0;
-			vt->rangehigh = 0xffffffffUL;
-
-			itv->card->video_dec_func(itv, VIDIOC_G_TUNER, &sig);
-
-			vt->signal = sig.signal;
-			vt->audmode = itv->audmode_tv;
 		}
-		vt->rxsubchans = ivtv_audio_get_rxsubchans(itv, vt);
 		break;
 	}
 
@@ -1241,25 +1096,11 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
         case VIDIOC_LOG_STATUS:
                 IVTV_INFO("=================  START STATUS CARD #%d  =================\n", itv->num);
                 if (itv->hw_flags & IVTV_HW_TVEEPROM) {
-//			struct tveeprom tv;
+			struct tveeprom tv;
 
-//			ivtv_read_eeprom(itv, &tv);
-                }
-                if (itv->hw_flags & IVTV_HW_CX25840) {
-                        itv->card->video_dec_func(itv, VIDIOC_LOG_STATUS, 0);
-                }
-                if (itv->hw_flags & IVTV_HW_SAA7115) {
-                        ivtv_saa7115(itv, VIDIOC_LOG_STATUS, 0);
-                }
-                if (itv->hw_flags & IVTV_HW_WM8775) {
-                        ivtv_wm8775(itv, VIDIOC_LOG_STATUS, 0);
-                }
-                if (itv->hw_flags & IVTV_HW_SAA7127) {
-                        ivtv_saa7127(itv, VIDIOC_LOG_STATUS, 0);
-                }
-                if (itv->hw_flags & IVTV_HW_TDA9887) {
-                        ivtv_tda9887(itv, VIDIOC_LOG_STATUS, 0);
+			ivtv_read_eeprom(itv, &tv);
                 }
+		ivtv_call_i2c_clients(itv, VIDIOC_LOG_STATUS, 0);
                 IVTV_INFO("==================  END STATUS CARD #%d  ==================\n", itv->num);
                 break;
 
@@ -1270,7 +1111,7 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 	return 0;
 }
 
-int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
+static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		     int streamtype, unsigned int cmd, void *arg)
 {
 	struct ivtv_stream *stream = &itv->streams[streamtype];
@@ -1312,7 +1153,7 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 			memset(fs, 0, sizeof(fs));
 			/* scr should be valid anytime */
 			if (ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-				     IVTV_API_DEC_TIMING_INFO, &result, 5,
+				     CX2341X_DEC_GET_TIMING_INFO, &result, 5,
 				     &data[0])) {
 				IVTV_DEBUG_WARN(
 					   "DEC: couldn't read clock\n");
@@ -1334,7 +1175,7 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 	case IVTV_IOC_PLAY:{
 		IVTV_DEBUG_IOCTL("IVTV_IOC_PLAY\n");
-		if (ivtv_vapi(itv, IVTV_API_DEC_START_PLAYBACK, 2, 0, 0))
+		if (ivtv_vapi(itv, CX2341X_DEC_START_PLAYBACK, 2, 0, 0))
 			IVTV_DEBUG_WARN(
 				   "DEC: error starting playback\n");
 		break;
@@ -1448,7 +1289,7 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 			return -EINVAL;
 
 		/* howfar: 0 = 1 frame, 1 = top field, 2 = bottom field */
-		if (ivtv_vapi(itv, IVTV_API_DEC_STEP_VIDEO, 1, howfar)) {
+		if (ivtv_vapi(itv, CX2341X_DEC_STEP_VIDEO, 1, howfar)) {
 			IVTV_DEBUG_WARN("DEC: error stepping\n");
 		}
 		break;
@@ -1532,7 +1373,7 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		if ((atomic_read(&itv->decoding) > 0) &&
 			test_bit(IVTV_F_T_DEC_DONE, &itv->t_flags)) 
 		{
-			if (ivtv_vapi(itv, IVTV_API_DEC_PAUSE_PLAYBACK, 1, 0)) {
+			if (ivtv_vapi(itv, CX2341X_DEC_PAUSE_PLAYBACK, 1, 0)) {
 				IVTV_DEBUG_WARN(
 					"DEC: error pausing\n");
 			}
@@ -1553,7 +1394,7 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 			memset(timing, 0, sizeof(timing));
 			/* scr should be valid anytime */
 			if (ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-				     IVTV_API_DEC_TIMING_INFO, &result, 5,
+				     CX2341X_DEC_GET_TIMING_INFO, &result, 5,
 				     &data[0])) {
 				IVTV_DEBUG_WARN(
 					   "DEC: couldn't read clock\n");
@@ -1567,7 +1408,7 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 		} else {
 			if (ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-				     IVTV_API_DEC_TIMING_INFO, &result, 5,
+				     CX2341X_DEC_GET_TIMING_INFO, &result, 5,
 				     &data[0])) {
 				IVTV_DEBUG_WARN(
 					   "DEC: couldnt read clock\n");
@@ -1583,7 +1424,6 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 	}
 
 	case IVTV_IOC_S_VBI_EMBED:{
-		int is_ntsc = (itv->std & V4L2_STD_NTSC) ? 1 : 0;
                 int embed = *(int *)arg;
 
 		IVTV_DEBUG_IOCTL("IVTV_IOC_S_VBI_EMBED\n");
@@ -1619,11 +1459,11 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
                 }
 		/* Need sliced data for mpeg insertion */
                 if (get_service_set(itv->vbi_sliced_in) == 0) {
-                        if (is_ntsc)
+                        if (itv->is_60hz)
         			itv->vbi_sliced_in->service_set = V4L2_SLICED_CAPTION_525;
                         else
         			itv->vbi_sliced_in->service_set = V4L2_SLICED_WSS_625;
-        		expand_service_set(itv->vbi_sliced_in, !is_ntsc);
+        		expand_service_set(itv->vbi_sliced_in, itv->is_50hz);
                 }
 		break;
 	}
@@ -1676,8 +1516,7 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
                 if ((codec->stream_type == IVTV_STREAM_MPEG1) ||
                     (codec->stream_type == IVTV_STREAM_VCD)) {
                         /* this is an MPEG1 stream */
-                        int tmp_height =
-                            (itv->std & V4L2_STD_NTSC) ? 480 : 576;
+                        int tmp_height = itv->is_60hz ? 480 : 576;
 
                         /* set vertical mpeg encoder resolution */
                         if (itv->height != tmp_height / 2) {
@@ -1700,7 +1539,7 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
                                         "v4l2 ioctl: set size failed, "
                                         "capture in progress\n");
                         } else {
-                                itv->card->video_dec_func(itv, VIDIOC_S_FMT, &fmt);
+                                itv->video_dec_func(itv, VIDIOC_S_FMT, &fmt);
                         }
                 }
 
@@ -1722,8 +1561,8 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		IVTV_DEBUG_IOCTL("IVTV_IOC_PAUSE_ENCODE\n");
 		if (!atomic_read(&itv->capturing))
 			return 0;
-		ivtv_vapi(itv, IVTV_API_PAUSE_ENCODER, 1);
 		ivtv_mute(itv);
+		ivtv_vapi(itv, CX2341X_ENC_PAUSE_ENCODER, 1);
 		break;
 	}
 
@@ -1731,8 +1570,8 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		IVTV_DEBUG_IOCTL("IVTV_IOC_RESUME_ENCODE\n");
 		if (!atomic_read(&itv->capturing))
 			return 0;
+		ivtv_vapi(itv, CX2341X_ENC_PAUSE_ENCODER, 0);
 		ivtv_unmute(itv);
-		ivtv_vapi(itv, IVTV_API_PAUSE_ENCODER, 0);
 		break;
 	}
 	case IVTV_IOC_PREP_FRAME_YUV: {
@@ -1781,6 +1620,12 @@ int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
                 itv->yuv_info.lace_threshold = yuv_mode->threshold;
 		break;
 	}
+	case IVTV_IOC_G_PTS: {
+		u64 *pts = arg;
+
+		*pts = stream->pts;
+		break;
+	}
 
 	default:
 		IVTV_DEBUG_WARN("unknown IVTV command %08x\n", cmd);
@@ -1808,7 +1653,7 @@ static int ivtv_v4l2_do_ioctl(struct inode *inode, struct file *filp,
 	case IVTV_IOC_G_DECODER_REG:
 	case IVTV_IOC_S_DECODER_REG:
 	case IVTV_IOC_ZCOUNT:
-	case IVTV_IOC_S_MSP_MATRIX:
+	case IVTV_IOC_S_AUDIO_ROUTING:
 	case IVTV_IOC_G_DEBUG_LEVEL:
 	case IVTV_IOC_S_DEBUG_LEVEL:
 	case IVTV_IOC_FWAPI:
@@ -1840,7 +1685,7 @@ static int ivtv_v4l2_do_ioctl(struct inode *inode, struct file *filp,
 	case VIDIOC_G_AUDOUT:
         case VIDIOC_G_SLICED_VBI_CAP:
         case VIDIOC_G_FBUF:
-	case VIDIOC_LOG_STATUS:
+        case VIDIOC_LOG_STATUS:
 		return ivtv_v4l2_ioctls(itv, id, streamtype, cmd, arg);
 
 	case VIDIOC_QUERYMENU:
@@ -1878,12 +1723,13 @@ static int ivtv_v4l2_do_ioctl(struct inode *inode, struct file *filp,
         case IVTV_IOC_PREP_FRAME_YUV:
         case IVTV_IOC_G_YUV_INTERLACE:
         case IVTV_IOC_S_YUV_INTERLACE:
+        case IVTV_IOC_G_PTS:
                 return ivtv_ivtv_ioctls(itv, id, streamtype, cmd, arg);
 
 	case 0x00005401:	/* Handle isatty() calls */
 		return -EINVAL;
 	default:
-		return ivtv_compat_translate_ioctl(inode, filp, cmd, arg,
+		return v4l_compat_translate_ioctl(inode, filp, cmd, arg,
 						   ivtv_v4l2_do_ioctl);
 	}
 	return 0;
@@ -1896,10 +1742,6 @@ int ivtv_v4l2_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
         struct ivtv *itv = id->itv;
 	int streamtype = id->type;
 
-	/* Converts old (0.1.9) non-conforming ioctls that were using
-	   'just some values I picked for now'. I hoped this would not be
-	   necessary, but too many people were already using existing apps
-	   (MythTV) written for this version of the driver. */
 	switch (cmd) {
 
 		/* This is temporary, I guess, it won't work unless here, so it's
@@ -1911,6 +1753,10 @@ int ivtv_v4l2_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
                 ivtv_stream_off(itv, &itv->streams[streamtype]);
 		return 0;
 
+	/* Converts old (0.1.9) non-conforming ioctls that were using
+	   'just some values I picked for now'. I hoped this would not be
+	   necessary, but too many people were already using existing apps
+	   (MythTV) written for this version of the driver. */
 	case 0xFFEE7703:
 		cmd = IVTV_IOC_G_CODEC;
 		break;
@@ -1941,26 +1787,6 @@ int ivtv_v4l2_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
 	case 0xFFEE7789:
 		cmd = IVTV_IOC_GET_FB;
 		break;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 2)
-	// For kernels from 2.6.2 this is converted by 
-	// video_usercopy from videodev.c. For older kernels
-	// we have to do it ourselves.
-	case VIDIOC_S_CTRL_OLD:
-		cmd = VIDIOC_S_CTRL;
-		break;
-	case VIDIOC_OVERLAY_OLD:
-		cmd = VIDIOC_OVERLAY;
-		break;
-	case VIDIOC_S_PARM_OLD:
-		cmd = VIDIOC_S_PARM;
-		break;
-	case VIDIOC_G_AUDIO_OLD:
-		cmd = VIDIOC_G_AUDIO;
-		break;
-	case VIDIOC_G_AUDOUT_OLD:
-		cmd = VIDIOC_G_AUDOUT;
-		break;
-#endif
 	case IVTV_IOC_G_DRIVER_INFO:{
 		// AEW - put here becuase we can't do this after video_usercopy
 		// has done its work of copying a tiny portion of what we really
