@@ -30,7 +30,7 @@
 
 typedef unsigned long uintptr_t;
 
-static void ivtv_FROM_DMA_done(struct ivtv *itv, int type, void *src);
+static void ivtv_FROM_DMA_done(struct ivtv *itv, int type);
 static int dma_from_device(struct ivtv *itv, struct ivtv_stream *st);
 
 IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
@@ -69,8 +69,10 @@ IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
 	/* Clear out IRQ */
 	if (combo) writel(combo, (itv->reg_mem + IVTV_REG_IRQSTATUS));
 
- 	if (combo & ~(0x88000400))
- 		IVTV_DEBUG_IRQ("======= valid IRQ bits: 0x%08x ======\n", combo);
+	/* Exclude 0x400 from the output, otherwise the log is flooded with
+	   these messages */
+	if (combo != 0x400)
+		IVTV_DEBUG_IRQ("======= valid IRQ bits: 0x%08x ======\n", combo);
 
 	if (combo & IVTV_IRQ_DEC_VBI_RE_INSERT2) {
 		/* VBI User data reinsertion */
@@ -100,13 +102,14 @@ IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
 		wake_up(&itv->r_intr_wq);
 	}
 	if (combo & IVTV_IRQ_ENC_DMA_COMPLETE) {
-		//IVTV_DEBUG_IRQ("IRQ: IVTV_IRQ_ENC_DMA_COMPLETE\n");
+		IVTV_DEBUG_IRQ("IRQ: IVTV_IRQ_ENC_DMA_COMPLETE\n");
 		atomic_set(&itv->w_intr, 1);
 		wake_up(&itv->w_intr_wq);
 	}
 	if (combo & IVTV_IRQ_DMA_ERR) {
-		atomic_set(&itv->w_intr, 1);
-		wake_up(&itv->w_intr_wq);
+		IVTV_DEBUG_WARN(
+			   "IRQ: IVTV_IRQ_DEC_DMA_ERR\n");
+		writel(0x0, itv->reg_mem + IVTV_REG_DMAXFER);
 	}
 
 	/* Define irq bits */
@@ -119,7 +122,7 @@ IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
 
 	if (combo & IVTV_IRQ_ENC_DMA) {
 		if (combo & IVTV_IRQ_ENC_START_CAP) {
-			//IVTV_DEBUG_IRQ("IRQ ENC DMA\n");
+			IVTV_DEBUG_IRQ("IRQ ENC DMA\n");
 
 			/* set dma needed flag */
 			set_bit(IVTV_F_T_ENC_DMA_NEEDED, &itv->t_flags);
@@ -239,7 +242,7 @@ IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
 
 	if (combo & IVTV_IRQ_ENC_VIM_RST) {
 		IVTV_DEBUG_IRQ("VIM Restart\n");
-		ivtv_vapi(itv, IVTV_API_REFRESH_INPUT, 0);
+		ivtv_vapi(itv, CX2341X_ENC_REFRESH_INPUT, 0);
 	}
 
 	if (combo & IVTV_IRQ_DEC_COPY_PROTECT) {
@@ -266,13 +269,11 @@ IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
 		return IRQ_HANDLED;
 }
 
-static void ivtv_FROM_DMA_done(struct ivtv *itv, int stmtype, void *src)
+static void ivtv_FROM_DMA_done(struct ivtv *itv, int stmtype)
 {
 	struct ivtv_stream *stream = NULL;
 	struct ivtv_buffer *buf;
 	int freed = 0;
-	int x = 0;
-	int offset = 0;
 
 	IVTV_DEBUG_DMA("ENC: DMA Done\n");
 
@@ -283,45 +284,15 @@ static void ivtv_FROM_DMA_done(struct ivtv *itv, int stmtype, void *src)
 		/* Sync Buffer */
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 8)
 		pci_dma_sync_single_for_cpu((struct pci_dev *)itv->dev,
-					    buf->dma_handle, buf->buffer.length + 256,
+					    buf->dma_handle, buf->buffer.length,
 					    stream->dma);
 #else
 		pci_dma_sync_single(itv->dev, buf->dma_handle,
-				    buf->buffer.length + 256, stream->dma);
+				    buf->buffer.length, stream->dma);
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 8) */
 
-		if (x == 0) {
-			offset = stream->last_offset;
-			if (((u32 *)(buf->buffer.m.userptr))[offset / 4] != 0x000001fe) {
-				for (offset = 0; offset < 64; offset++) {
-					if (((u32 *)(buf->buffer.m.userptr))[offset] == 0x000001fe) {
-						break;
-					}
-				}
-				offset *= 4;
-				if (offset == 256) {
-					IVTV_DEBUG_WARN("Couldn't find start of buffer within the first 256 bytes\n");
-					offset = 0;
-				}
-				if (stream->last_offset != offset)
-					IVTV_DEBUG_WARN("offset %d -> %d\n", stream->last_offset, offset);
-				stream->last_offset = offset;
-			}
-			writel(0, src);
-			if (offset) {
-				buf->buffer.bytesused -= offset;
-				memcpy((void *)buf->buffer.m.userptr,
-					(void *)(buf->buffer.m.userptr + offset), 
-					buf->buffer.bytesused + offset);
-			}
-			*(u32 *)(buf->buffer.m.userptr) = stream->buf_backup;
-		}
-		if (list_empty(&stream->dma_q.list)) {
-			buf->buffer.bytesused += stream->last_offset;
-		}
-		x++;
 		/* flag byteswap ABCD -> DCBA for MPG & VBI data outside irq */
-		if (stmtype == IVTV_ENC_STREAM_TYPE_MPG)
+		if (stmtype == IVTV_ENC_STREAM_TYPE_MPG) 
 			set_bit(IVTV_F_B_NEED_BUF_SWAP, &buf->b_flags);
 		else if ((stmtype == IVTV_ENC_STREAM_TYPE_VBI ||
 					stmtype == IVTV_DEC_STREAM_TYPE_VBI)) 
@@ -359,17 +330,17 @@ static void ivtv_FROM_DMA_done(struct ivtv *itv, int stmtype, void *src)
 void ivtv_sched_DMA(struct ivtv *itv)
 {
 	u32 data[IVTV_MBOX_MAX_DATA], result;
-	u32 type, size, offset, streamtype;
+	u32 type, size, offset, streamtype, pad = 0;
 	u32 UVsize = 0, UVoffset = 0;
 	u64 pts_stamp = 0;
 	struct ivtv_stream *st;
 	int x;
-	void *src = (void *)itv->enc_mem;
 	int uvflag = 0;
 	long sequence;
 	u32 bytes_needed = 0, bytes_read = 0, bytes_received = 0;
 	struct ivtv_buffer *buf;
 	LIST_HEAD(free_list);
+	int xfer_pad;
 	unsigned long then;
 	DECLARE_WAITQUEUE(wait, current);
 	int rc = 0;
@@ -436,7 +407,6 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		pts_stamp |=
 		    (u64) (*(u32 *) (itv->dec_mem + offset - 12)) << 32;
 		offset += IVTV_DECODER_OFFSET;
-		src = (void *)(itv->dec_mem - IVTV_DECODER_OFFSET);
 		IVTV_DEBUG_INFO(
 			   "DMA/PCM type 0x%08x,size 0x%08x,offset 0x%08x "
 			   "PTS 0x%09llx\n", type, size, offset, pts_stamp);
@@ -461,7 +431,6 @@ void ivtv_sched_DMA(struct ivtv *itv)
 
 	st = &itv->streams[streamtype];
 	st->pts = pts_stamp;
-	st->dmatype = type;
 
 	if (st->id == -1) {
 		return;
@@ -553,9 +522,6 @@ void ivtv_sched_DMA(struct ivtv *itv)
 	sequence = ++st->seq;
 
 	st->ubytes = size;
-	src += offset;
-	st->buf_backup = readl(src);
-	writel(0x000001fe, src);
 
 	for (x = 0; bytes_read < bytes_needed; x++) {
 		/* extract the buffers we procured earlier */
@@ -571,10 +537,18 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		bytes_read += buf->buffer.length;
 
 		if (size < buf->buffer.length) {
+			xfer_pad = buf->buffer.length /*256 */ ;
+			pad = size;
 			buf->buffer.bytesused = size;
-			st->SGarray[x].size = buf->buffer.length;
+			if (size > xfer_pad && size % xfer_pad)	/* Align */
+				size = size + (size % xfer_pad);
+			if (size < xfer_pad)	/* Too small */
+				size = xfer_pad;
+			st->SGarray[x].size = size;
 			size = 0;
+
 		} else {
+			pad = 0;
 			buf->buffer.bytesused = buf->buffer.length;
 			st->SGarray[x].size = buf->buffer.length;
 			size -= st->SGarray[x].size;
@@ -594,11 +568,11 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		/* Sync SG buffers */
 		pci_dma_sync_single_for_device((struct pci_dev *)itv->dev,
 					       buf->dma_handle,
-					       buf->buffer.length + 256, st->dma);
+					       buf->buffer.length, st->dma);
 #else
 		/* Sync SG buffers */
 		pci_dma_sync_single(itv->dev, buf->dma_handle,
-				    buf->buffer.length + 256, st->dma);
+				    buf->buffer.length, st->dma);
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 8) */
 
 		ivtv_enq_buf_nolock(&st->dma_q, buf);
@@ -611,7 +585,6 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		}
 	}
 	st->SG_length = x;
-	st->SGarray[x - 1].size += 256;
 
 	/* This should wrap gracefully */
 	st->trans_id++;
@@ -639,10 +612,10 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		   st->SG_length, bytes_read, type);
 
 	if (pio_mode) {
-		ivtv_vapi(itv, IVTV_API_SCHED_DMA_TO_HOST, 3, 0, 0, st->type);
+		ivtv_vapi(itv, CX2341X_ENC_SCHED_DMA_TO_HOST, 3, 0, 0, st->type);
 
 		/* Gather Buffers from DMA xfer to read */
-		ivtv_FROM_DMA_done(itv, st->type, src);
+		ivtv_FROM_DMA_done(itv, st->type);
 	} else {
 		//down(&itv->DMA_lock);
 		if (dma_from_device(itv, st) != 0) {
@@ -651,7 +624,7 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		//up(&itv->DMA_lock);
 
 		/* Gather Buffers from DMA xfer to read */
-		ivtv_FROM_DMA_done(itv, st->type, src);
+		ivtv_FROM_DMA_done(itv, st->type);
 	}
 
 	clear_bit(IVTV_F_S_DMAP, &st->s_flags);
@@ -665,15 +638,14 @@ static int dma_from_device(struct ivtv *itv, struct ivtv_stream *st)
 	unsigned long flags;
 	int redo_dma = 0;
 	int ret = 0;
-	int err;
+	int sg_offset;
 
 	/* wait for DMA complete status */
 	then = jiffies;
 	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x03)) {
 		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ * 1)) {
-			IVTV_DEBUG_WARN(
-				   "ENC: REG_DMASTATUS 1 wait failed\n");
+		if ((jiffies - then) > (HZ * 3)) {
+			IVTV_DEBUG_WARN("ENC: REG_DMASTATUS wait failed\n");
 			break;
 		}
 	}
@@ -682,57 +654,70 @@ static int dma_from_device(struct ivtv *itv, struct ivtv_stream *st)
 	then = jiffies;
 	while ((readl(itv->reg_mem + IVTV_REG_DMAXFER) & 0x03)) {
 		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ * 1)) {
-			IVTV_DEBUG_WARN(
-				   "ENC: REG_DMAXFER 1 wait failed\n");
+		if ((jiffies - then) > (HZ * 3)) {
+			IVTV_DEBUG_WARN("ENC: REG_DMAXFER wait failed\n");
 			break;
 		}
 	}
 
-redo_dma:
-	err = 0;
+      redo_dma:
 
 	spin_lock_irqsave(&itv->DMA_slock, flags);
 	/* put SG Handle into register 0x0c */
 	ivtv_write_reg(st->SG_handle, itv->reg_mem + IVTV_REG_ENCDMAADDR);
-	if (readl(itv->reg_mem + IVTV_REG_ENCDMAADDR) != st->SG_handle)
+	if (readl(itv->reg_mem + IVTV_REG_ENCDMAADDR) != st->SG_handle) {
 		ivtv_write_reg(st->SG_handle,
 			       itv->reg_mem + IVTV_REG_ENCDMAADDR);
+	}
 
 	/* Send DMA with register 0x00, using the enc DMA bit */
-	if (readl(itv->reg_mem + IVTV_REG_ENCDMAADDR) == st->SG_handle)
+	if (readl(itv->reg_mem + IVTV_REG_ENCDMAADDR) == st->SG_handle) {
 		ivtv_write_reg(readl(itv->reg_mem + IVTV_REG_DMAXFER) |
 			       0x02, itv->reg_mem + IVTV_REG_DMAXFER);
-	else {
-		err = 1;
 	}
 	spin_unlock_irqrestore(&itv->DMA_slock, flags);
 
 	/* wait for DMA to start */
-
 	then = jiffies;
-
-	while (!err && (readl(itv->reg_mem + IVTV_REG_DMAXFER) & 0x02)) {
-		u32 stat = readl(itv->reg_mem + IVTV_REG_DMASTATUS);
+	while ((readl(itv->reg_mem + IVTV_REG_DMAXFER) & 0x02)) {
 		/* DMA Error */
-		if (stat & 0x1a)
-  			break;
-  
-		if ((jiffies - then) > (HZ / 5)) {
-  			IVTV_DEBUG_WARN("ENC: REG_DMAXFER 2 wait failed\n");
-			err = 1;
-  			break;
-  		}
+		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1A) {
+			break;
+		}
+
+		ivtv_sleep_timeout(HZ / 100, 1);
+		if ((jiffies - then) > (HZ * 3)) {
+			IVTV_DEBUG_WARN("ENC: REG_DMAXFER 2 wait failed\n");
+			break;
+		}
+	}
+
+	/* wait for DMA to be finished, check interrupt bit */
+	if (st->SG_length <= 8)
+		sg_offset = ((12 * st->SG_length) - 4);
+	else
+		sg_offset = ((12 * (st->SG_length % 8)) - 4);
+	then = jiffies;
+	while (!(readl(itv->reg_mem +
+			       IVTV_REG_ENCSG1SRC + sg_offset) & 0x80000000)) {
+		/* DMA Error */
+		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1A)
+			break;
+
+		ivtv_sleep_timeout(HZ / 100, 1);
+		if ((jiffies - then) > (HZ * 3)) {
+			IVTV_DEBUG_WARN("DEC: REG_ENCSG1LEN wait failed\n");
+			break;
+		}
 	}
 
 	/* Wait for Write Interrupt */
-	while (!err && !(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1a) &&
+	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1A) &&
 	       wait_event_interruptible(itv->w_intr_wq,
 					atomic_read(&itv->w_intr))) {
 		/* DMA Error */
-		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1a) {
+		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1A)
 			break;
-		}
 
 		if (atomic_read(&itv->w_intr))
 			break;
@@ -742,27 +727,24 @@ redo_dma:
 
 	/* wait for DMA complete status */
 	then = jiffies;
-
-	while (!err && !(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x02)) {
+	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x02)) {
 		/* DMA Error */
-		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x18) {
+		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1A) {
 			break;
 		}
 
 		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ / 50)) {
-			err = 1;
-			IVTV_DEBUG_WARN("ENC: REG_DMASTATUS 2 wait failed\n");
+		if ((jiffies - then) > (HZ * 3)) {
+			IVTV_DEBUG_WARN("ENC: REG_DMASTATUS2 wait failed\n");
 			break;
 		}
 	}
 
 	/* DMA Error */
-	if (err || (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x18)) {
-		IVTV_DEBUG_WARN("ENC: (%d) DMA Error 0x%08x %08x\n",
-  			   redo_dma, readl(itv->reg_mem +
-						   IVTV_REG_DMASTATUS),
-			   readl(itv->reg_mem + IVTV_REG_IRQSTATUS));
+	if ((readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x18)) {
+		IVTV_DEBUG_WARN("ENC: (%d) DMA Error 0x%08x\n",
+			   redo_dma, readl(itv->reg_mem +
+						   IVTV_REG_DMASTATUS));
 
 		/* Reset DMA Error, cancel last DMA? */
 		spin_lock_irqsave(&itv->DMA_slock, flags);
@@ -771,11 +753,10 @@ redo_dma:
 			       & 0x03, itv->reg_mem + IVTV_REG_DMASTATUS);
 		spin_unlock_irqrestore(&itv->DMA_slock, flags);
 		if (redo_dma < 3) {
-  			redo_dma++;
+			redo_dma++;
 			goto redo_dma;
 		}
-		IVTV_DEBUG_WARN(
-			   "ENC: REDO DMA took too many tries.\n");
+		IVTV_DEBUG_WARN("ENC: REDO DMA took too many tries.\n");
 		ret = -ENOMEM;
 	}
 
@@ -788,7 +769,6 @@ void ivtv_sched_VBI(struct ivtv *itv, int streamtype)
 	u64 pts_stamp;
 	struct ivtv_stream *st;
 	int x;
-	void *src;
 	struct ivtv_buffer *buf = 0;
 	LIST_HEAD(free_list);
 	long sequence;
@@ -819,7 +799,6 @@ void ivtv_sched_VBI(struct ivtv *itv, int streamtype)
 		itv->vbi_frame += itv->vbi_fpi;
 		size = itv->vbi_enc_size * itv->vbi_fpi;
 		offset = bufptr;
-		src = (void *)(itv->enc_mem + offset);
 
 		IVTV_DEBUG_DMA(
 			"VBI(%d) BufPTR: 0x%08x offset 0x%08x size 0x%08x\n",
@@ -841,7 +820,6 @@ void ivtv_sched_VBI(struct ivtv *itv, int streamtype)
 		pio_mode = 1; /* Seems DMA mode has errors */
 		offset = readl(itv->dec_mem + itv->vbi_dec_start);
 		offset += itv->vbi_dec_start;
-		src = (void *)(itv->dec_mem + offset);
 		size = readl(itv->dec_mem + itv->vbi_dec_start + 4);
 		size += 8;
 		pts_stamp = 0;
@@ -939,8 +917,6 @@ skip_vbi_wait:
 	sequence = ++st->seq;
 
 	st->ubytes = bytes_received;
-	st->buf_backup = readl(src);
-	writel(0x000001fe, src);
 
 	for (x = 0; bytes_read < bytes_received; x++) {
 		/* extract the buffers we procured earlier */
@@ -993,11 +969,11 @@ skip_vbi_wait:
 		/* Sync SG buffers */
 		pci_dma_sync_single_for_device((struct pci_dev *)itv->dev,
 					       buf->dma_handle,
-					       buf->buffer.length + 256, st->dma);
+					       buf->buffer.length, st->dma);
 #else
 		/* Sync SG buffers */
 		pci_dma_sync_single(itv->dev, buf->dma_handle,
-				    buf->buffer.length + 256, st->dma);
+				    buf->buffer.length, st->dma);
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 8) */
 
 		ivtv_enq_buf_nolock(&st->dma_q, buf);
@@ -1031,7 +1007,7 @@ skip_vbi_wait:
 
 	if (pio_mode) {
 		/* Gather Buffers from DMA xfer to read */
-		ivtv_FROM_DMA_done(itv, st->type, src);
+		ivtv_FROM_DMA_done(itv, st->type);
 	} else {
 		//down(&itv->DMA_lock);
 		if (dma_from_device(itv, st) != 0) {
@@ -1039,7 +1015,7 @@ skip_vbi_wait:
 		}
 		//up(&itv->DMA_lock);
 
-		ivtv_FROM_DMA_done(itv, st->type, src);
+		ivtv_FROM_DMA_done(itv, st->type);
 	}
 
 	clear_bit(IVTV_F_S_DMAP, &st->s_flags);
