@@ -19,8 +19,5 @@
  */
 
 IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs);
-void ivtv_DMA_tasklet(unsigned long arg);
-void ivtv_dec_VSYNC_tasklet(unsigned long arg);
-int ivtv_ignore_DMA_req(struct ivtv *itv, u32 type);
 void ivtv_sched_DMA(struct ivtv *itv);
 void ivtv_sched_VBI(struct ivtv *itv, int streamtype);
