@@ -58,56 +58,51 @@ static u32 ivtv_round_dma(u32 data)
 static void ivtv_dualwatch_set_dual(struct ivtv *itv)
 {
 	u32 audio_bitmask;
+	u32 new_bitmap;
 	int goal_stereo_flag;
 	int stereo_flag;
+	const int stereo_mask = IVTV_CODEC_AUDIO_MPEG_STEREO_MASK;
+	const int dual = IVTV_CODEC_AUDIO_MPEG_STEREO_DUAL;
+	const int stereo = IVTV_CODEC_AUDIO_MPEG_STEREO_STEREO;
+	const int joint = IVTV_CODEC_AUDIO_MPEG_STEREO_JOINT;
 
 	if (!itv->dualwatch.encoding)
 		return;
 
 	audio_bitmask = itv->dualwatch.audio_bitmask;
-	stereo_flag = audio_bitmask & IVTV_CODEC_AUDIO_MPEG_STEREO_MASK;
+	stereo_flag = audio_bitmask & stereo_mask;
 	if (itv->dualwatch.enabled) {
 		struct v4l2_tuner vt;
+
 		memset(&vt, 0, sizeof(vt));
-		ivtv_msp34xx(itv, VIDIOC_G_TUNER, &vt);
+		ivtv_call_i2c_clients(itv, VIDIOC_G_TUNER, &vt);
 		if (vt.rxsubchans & V4L2_TUNER_SUB_LANG2) {
-			goal_stereo_flag = IVTV_CODEC_AUDIO_MPEG_STEREO_DUAL;
+			goal_stereo_flag = dual;
 		} else {
-			goal_stereo_flag = IVTV_CODEC_AUDIO_MPEG_STEREO_STEREO;
+			goal_stereo_flag = stereo;
 		}
 	} else {
-		if (stereo_flag == IVTV_CODEC_AUDIO_MPEG_STEREO_DUAL) {
-			goal_stereo_flag = IVTV_CODEC_AUDIO_MPEG_STEREO_STEREO;
-		} else {
-			goal_stereo_flag = stereo_flag;
-		}
+		goal_stereo_flag = (stereo_flag == dual) ? stereo : stereo_flag;
 	}
 
-	if (stereo_flag == IVTV_CODEC_AUDIO_MPEG_STEREO_STEREO
-	    && ((itv->codec.audio_bitmask & IVTV_CODEC_AUDIO_MPEG_STEREO_MASK)
-		== IVTV_CODEC_AUDIO_MPEG_STEREO_JOINT)) {
-		goal_stereo_flag = IVTV_CODEC_AUDIO_MPEG_STEREO_JOINT;
+	if (stereo_flag == stereo &&
+	    ((itv->codec.audio_bitmask & stereo_mask) == joint)) {
+		goal_stereo_flag = joint;
 	}
 
-	if (goal_stereo_flag != stereo_flag) {
-		u32 new_bitmap =
-		    goal_stereo_flag
-		    | (itv->dualwatch.
-		       audio_bitmask & ~IVTV_CODEC_AUDIO_MPEG_STEREO_MASK);
+	if (goal_stereo_flag == stereo_flag) 
+		return;
 
-		IVTV_DEBUG_INFO(
-			   "dualwatch: change stereo flag from 0x%x to 0x%x. new audio_bitmask=0x%ux\n",
+	new_bitmap = goal_stereo_flag | (itv->dualwatch.audio_bitmask & ~stereo_mask);
+
+	IVTV_DEBUG_INFO("dualwatch: change stereo flag from 0x%x to 0x%x. new audio_bitmask=0x%ux\n",
 			   stereo_flag, goal_stereo_flag, new_bitmap);
 
-		if (ivtv_vapi
-		    (itv, IVTV_API_ASSIGN_AUDIO_PROPERTIES, 1,
-		     new_bitmap) == 0) {
-			itv->dualwatch.audio_bitmask = new_bitmap;
-		} else {
-			IVTV_DEBUG_INFO(
-				   "dualwatch: changing stereo flag failed\n");
-		}
+	if (ivtv_vapi(itv, CX2341X_ENC_SET_AUDIO_PROPERTIES, 1, new_bitmap) == 0) {
+		itv->dualwatch.audio_bitmask = new_bitmap;
+		return;
 	}
+	IVTV_DEBUG_INFO("dualwatch: changing stereo flag failed\n");
 }
 
 /* start the thread if necessary */
@@ -231,13 +226,13 @@ static void ivtv_dualwatch_init_kthread(struct ivtv *ivtv)
 {
 	struct ivtv_dualwatch *kthread = &ivtv->dualwatch;
 
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	daemonize();
 	exit_files(current);
 	reparent_to_init();
 #else
 	daemonize("ivtv_dualwatch");
-#endif /* LINUX26 */
+#endif
 
 	/* lock the kernel. A new kernel thread starts without
 	   the big kernel lock, regardless of the lock state
@@ -287,16 +282,6 @@ static void ivtv_dualwatch_exit_kthread(struct ivtv *ivtv)
 	 */
 }
 
-void fw_work_handler(void *arg)
-{
-	struct ivtv *itv = (struct ivtv *)arg;
-
-	/* Check Firmware */
-	ivtv_firmware_flags(itv, 100);
-
-	return;
-}
-
 /* Must be spinlocked */
 static int gather_free_buffers(struct ivtv *itv, int streamtype,
 			       struct list_head *free_list, int bytes_needed)
@@ -361,7 +346,7 @@ static int gather_free_buffers(struct ivtv *itv, int streamtype,
 	return bytes_received;
 }
 
-void enc_work_handler(void *arg)
+static void enc_work_handler(void *arg)
 {
 	struct ivtv *itv = (struct ivtv *)arg;
 
@@ -375,7 +360,7 @@ void enc_work_handler(void *arg)
 	return;
 }
 
-void enc_vbi_work_handler(void *arg)
+static void enc_vbi_work_handler(void *arg)
 {
         struct ivtv *itv = (struct ivtv *)arg;
 
@@ -389,7 +374,7 @@ void enc_vbi_work_handler(void *arg)
         return;
 }
 
-void dec_vbi_work_handler(void *arg)
+static void dec_vbi_work_handler(void *arg)
 {
         struct ivtv *itv = (struct ivtv *)arg;
 
@@ -403,7 +388,7 @@ void dec_vbi_work_handler(void *arg)
         return;
 }
 
-void dec_work_handler(void *arg)
+static void dec_work_handler(void *arg)
 {
 	struct ivtv *itv = (struct ivtv *)arg;
 	int type = IVTV_DEC_STREAM_TYPE_MPG;
@@ -981,7 +966,7 @@ void dec_work_handler(void *arg)
 		   I think we can do this part ourselves but would have to
 		   fully calculate xfer info ourselves and not use interrupts 
 		 */
-		ivtv_vapi(itv, IVTV_API_DEC_DMA_FROM_HOST, 3,
+		ivtv_vapi(itv, CX2341X_DEC_SCHED_DMA_FROM_HOST, 3,
 			  0, bytes_written, hw_stream_type);
 
 		/* Dequeue buffers */
@@ -998,7 +983,7 @@ void dec_work_handler(void *arg)
                         	ivtv_write_reg(0x01, itv->reg_mem + 0x2800);
 			}
 
-			ivtv_vapi(itv, IVTV_API_DEC_DMA_FROM_HOST, 3,
+			ivtv_vapi(itv, CX2341X_DEC_SCHED_DMA_FROM_HOST, 3,
 				  0, bytes_written, hw_stream_type);
 
 			clear_bit(IVTV_F_S_DMAP,
@@ -1052,7 +1037,7 @@ int ivtv_dec_thread(void *arg)
 {
 	struct ivtv *itv = (struct ivtv *)arg;
 
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	daemonize();
 	reparent_to_init();
 	strcpy(current->comm, "ivtv-dec");
@@ -1061,7 +1046,7 @@ int ivtv_dec_thread(void *arg)
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 5)
 	current->flags |= PF_NOFREEZE;
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 5) */
-#endif /* LINUX26 */
+#endif
 
 #ifndef SCHED_YIELD
 	set_user_nice(current, -10);
@@ -1110,7 +1095,7 @@ int ivtv_enc_thread(void *arg)
 {
 	struct ivtv *itv = (struct ivtv *)arg;
 
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	daemonize();
 	reparent_to_init();
 	strcpy(current->comm, "ivtv-enc");
@@ -1119,7 +1104,7 @@ int ivtv_enc_thread(void *arg)
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 5)
 	current->flags |= PF_NOFREEZE;
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 5) */
-#endif /* LINUX26 */
+#endif
 
 #ifndef SCHED_YIELD
 	set_user_nice(current, -10);
@@ -1168,7 +1153,7 @@ int ivtv_enc_vbi_thread(void *arg)
 {
         struct ivtv *itv = (struct ivtv *)arg;
 
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
         daemonize();
         reparent_to_init();
         strcpy(current->comm, "ivtv-enc-vbi");
@@ -1177,7 +1162,7 @@ int ivtv_enc_vbi_thread(void *arg)
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 5)
         current->flags |= PF_NOFREEZE;
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 5) */
-#endif /* LINUX26 */
+#endif
 
 #ifndef SCHED_YIELD
         set_user_nice(current, -10);
@@ -1226,7 +1211,7 @@ int ivtv_dec_vbi_thread(void *arg)
 {
         struct ivtv *itv = (struct ivtv *)arg;
 
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
         daemonize();
         reparent_to_init();
         strcpy(current->comm, "ivtv-dec-vbi");
@@ -1235,7 +1220,7 @@ int ivtv_dec_vbi_thread(void *arg)
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 5)
         current->flags |= PF_NOFREEZE;
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 5) */
-#endif /* LINUX26 */
+#endif
 
 #ifndef SCHED_YIELD
         set_user_nice(current, -10);
