@@ -153,96 +153,96 @@ static const struct {
         const char *name;
 } cmd_names[] = {
 	/* MPEG decoder API */
-	API_ENTRY(IVTV_API_DEC_PING_FW)
-	API_ENTRY(IVTV_API_DEC_START_PLAYBACK)
-	API_ENTRY(IVTV_API_DEC_STOP_PLAYBACK)
-	API_ENTRY(IVTV_API_DEC_PLAYBACK_SPEED)
-	API_ENTRY(IVTV_API_DEC_STEP_VIDEO)
-	API_ENTRY(IVTV_API_DEC_DMA_BLOCKSIZE)
-	API_ENTRY(IVTV_API_DEC_XFER_INFO)
-	API_ENTRY(IVTV_API_DEC_DMA_STATUS)
-	API_ENTRY(IVTV_API_DEC_DMA_FROM_HOST)
-	API_ENTRY(IVTV_API_DEC_PAUSE_PLAYBACK)
-	API_ENTRY(IVTV_API_DEC_HALT_FW)
-	API_ENTRY(IVTV_API_DEC_DISP_STANDARD)
-	API_ENTRY(IVTV_API_DEC_GETVER)
-	API_ENTRY(IVTV_API_DEC_STREAM_INPUT)
-	API_ENTRY(IVTV_API_DEC_TIMING_INFO)
-	API_ENTRY(IVTV_API_DEC_SELECT_AUDIO)
-	API_ENTRY(IVTV_API_DEC_EVENT_NOTIFICATION)
-	API_ENTRY(IVTV_API_DEC_DISPLAY_BUFFERS)
-	API_ENTRY(IVTV_API_DEC_EXTRACT_VBI)
-	API_ENTRY(IVTV_API_DEC_DECODE_SOURCE)
-	API_ENTRY(IVTV_API_DEC_AUDIO_OUTPUT)
-	API_ENTRY(IVTV_API_DEC_SET_AV_DELAY)
-	API_ENTRY(IVTV_API_DEC_BUFFER)
+	API_ENTRY(CX2341X_DEC_PING_FW)
+	API_ENTRY(CX2341X_DEC_START_PLAYBACK)
+	API_ENTRY(CX2341X_DEC_STOP_PLAYBACK)
+	API_ENTRY(CX2341X_DEC_SET_PLAYBACK_SPEED)
+	API_ENTRY(CX2341X_DEC_STEP_VIDEO)
+	API_ENTRY(CX2341X_DEC_SET_DMA_BLOCK_SIZE)
+	API_ENTRY(CX2341X_DEC_GET_XFER_INFO)
+	API_ENTRY(CX2341X_DEC_GET_DMA_STATUS)
+	API_ENTRY(CX2341X_DEC_SCHED_DMA_FROM_HOST)
+	API_ENTRY(CX2341X_DEC_PAUSE_PLAYBACK)
+	API_ENTRY(CX2341X_DEC_HALT_FW)
+	API_ENTRY(CX2341X_DEC_SET_STANDARD)
+	API_ENTRY(CX2341X_DEC_GET_VERSION)
+	API_ENTRY(CX2341X_DEC_SET_STREAM_INPUT)
+	API_ENTRY(CX2341X_DEC_GET_TIMING_INFO)
+	API_ENTRY(CX2341X_DEC_SET_AUDIO_MODE)
+	API_ENTRY(CX2341X_DEC_SET_EVENT_NOTIFICATION)
+	API_ENTRY(CX2341X_DEC_SET_DISPLAY_BUFFERS)
+	API_ENTRY(CX2341X_DEC_EXTRACT_VBI)
+	API_ENTRY(CX2341X_DEC_SET_DECODER_SOURCE)
+	API_ENTRY(CX2341X_DEC_SET_AUDIO_OUTPUT)
+	API_ENTRY(CX2341X_DEC_SET_AV_DELAY)
+	API_ENTRY(CX2341X_DEC_SET_PREBUFFERING)
 
 	/* MPEG encoder API */
-	API_ENTRY(IVTV_API_ENC_PING_FW)
-	API_ENTRY(IVTV_API_BEGIN_CAPTURE)
-	API_ENTRY(IVTV_API_END_CAPTURE)
-	API_ENTRY(IVTV_API_ASSIGN_AUDIO_ID)
-	API_ENTRY(IVTV_API_ASSIGN_VIDEO_ID)
-	API_ENTRY(IVTV_API_ASSIGN_PCR_ID)
-	API_ENTRY(IVTV_API_ASSIGN_FRAMERATE)
-	API_ENTRY(IVTV_API_ASSIGN_FRAME_SIZE)
-	API_ENTRY(IVTV_API_ASSIGN_BITRATES)
-	API_ENTRY(IVTV_API_ASSIGN_GOP_PROPERTIES)
-	API_ENTRY(IVTV_API_ASSIGN_ASPECT_RATIO)
-	API_ENTRY(IVTV_API_ASSIGN_DNR_FILTER_MODE)
-	API_ENTRY(IVTV_API_ASSIGN_DNR_FILTER_PROPS)
-	API_ENTRY(IVTV_API_ASSIGN_CORING_LEVELS)
-	API_ENTRY(IVTV_API_ASSIGN_SPATIAL_FILTER_TYPE)
-	API_ENTRY(IVTV_API_ASSIGN_3_2_PULLDOWN)
-	API_ENTRY(IVTV_API_SELECT_VBI_LINE)
-	API_ENTRY(IVTV_API_ASSIGN_STREAM_TYPE)
-	API_ENTRY(IVTV_API_ASSIGN_OUTPUT_PORT)
-	API_ENTRY(IVTV_API_ASSIGN_AUDIO_PROPERTIES)
-	API_ENTRY(IVTV_API_ENC_HALT_FW)
-	API_ENTRY(IVTV_API_ENC_GETVER)
-	API_ENTRY(IVTV_API_ASSIGN_GOP_CLOSURE)
-	API_ENTRY(IVTV_API_ENC_GET_SEQ_END)
-	API_ENTRY(IVTV_API_ASSIGN_PGM_INDEX_INFO)
-	API_ENTRY(IVTV_API_CONFIG_VBI)
-	API_ENTRY(IVTV_API_ASSIGN_DMA_BLOCKLEN)
-	API_ENTRY(IVTV_API_PREV_DMA_INFO_MB_10)
-	API_ENTRY(IVTV_API_PREV_DMA_INFO_MB_9)
-	API_ENTRY(IVTV_API_SCHED_DMA_TO_HOST)
-	API_ENTRY(IVTV_API_INITIALIZE_INPUT)
-	API_ENTRY(IVTV_API_ASSIGN_FRAME_DROP_RATE)
-	API_ENTRY(IVTV_API_PAUSE_ENCODER)
-	API_ENTRY(IVTV_API_REFRESH_INPUT)
-	API_ENTRY(IVTV_API_ASSIGN_COPYRIGHT)
-	API_ENTRY(IVTV_API_EVENT_NOTIFICATION)
-	API_ENTRY(IVTV_API_ASSIGN_NUM_VSYNC_LINES)
-	API_ENTRY(IVTV_API_ASSIGN_PLACEHOLDER)
-	API_ENTRY(IVTV_API_MUTE_VIDEO)
-	API_ENTRY(IVTV_API_MUTE_AUDIO)
-	API_ENTRY(IVTV_API_ENC_UNKNOWN)
-	API_ENTRY(IVTV_API_ENC_MISC)
+	API_ENTRY(CX2341X_ENC_PING_FW)
+	API_ENTRY(CX2341X_ENC_START_CAPTURE)
+	API_ENTRY(CX2341X_ENC_STOP_CAPTURE)
+	API_ENTRY(CX2341X_ENC_SET_AUDIO_ID)
+	API_ENTRY(CX2341X_ENC_SET_VIDEO_ID)
+	API_ENTRY(CX2341X_ENC_SET_PCR_ID)
+	API_ENTRY(CX2341X_ENC_SET_FRAME_RATE)
+	API_ENTRY(CX2341X_ENC_SET_FRAME_SIZE)
+	API_ENTRY(CX2341X_ENC_SET_BIT_RATE)
+	API_ENTRY(CX2341X_ENC_SET_GOP_PROPERTIES)
+	API_ENTRY(CX2341X_ENC_SET_ASPECT_RATIO)
+	API_ENTRY(CX2341X_ENC_SET_DNR_FILTER_MODE)
+	API_ENTRY(CX2341X_ENC_SET_DNR_FILTER_PROPS)
+	API_ENTRY(CX2341X_ENC_SET_CORING_LEVELS)
+	API_ENTRY(CX2341X_ENC_SET_SPATIAL_FILTER_TYPE)
+	API_ENTRY(CX2341X_ENC_SET_3_2_PULLDOWN)
+	API_ENTRY(CX2341X_ENC_SET_VBI_LINE)
+	API_ENTRY(CX2341X_ENC_SET_STREAM_TYPE)
+	API_ENTRY(CX2341X_ENC_SET_OUTPUT_PORT)
+	API_ENTRY(CX2341X_ENC_SET_AUDIO_PROPERTIES)
+	API_ENTRY(CX2341X_ENC_HALT_FW)
+	API_ENTRY(CX2341X_ENC_GET_VERSION)
+	API_ENTRY(CX2341X_ENC_SET_GOP_CLOSURE)
+	API_ENTRY(CX2341X_ENC_GET_SEQ_END)
+	API_ENTRY(CX2341X_ENC_SET_PGM_INDEX_INFO)
+	API_ENTRY(CX2341X_ENC_SET_VBI_CONFIG)
+	API_ENTRY(CX2341X_ENC_SET_DMA_BLOCK_SIZE)
+	API_ENTRY(CX2341X_ENC_GET_PREV_DMA_INFO_MB_10)
+	API_ENTRY(CX2341X_ENC_GET_PREV_DMA_INFO_MB_9)
+	API_ENTRY(CX2341X_ENC_SCHED_DMA_TO_HOST)
+	API_ENTRY(CX2341X_ENC_INITIALIZE_INPUT)
+	API_ENTRY(CX2341X_ENC_SET_FRAME_DROP_RATE)
+	API_ENTRY(CX2341X_ENC_PAUSE_ENCODER)
+	API_ENTRY(CX2341X_ENC_REFRESH_INPUT)
+	API_ENTRY(CX2341X_ENC_SET_COPYRIGHT)
+	API_ENTRY(CX2341X_ENC_SET_EVENT_NOTIFICATION)
+	API_ENTRY(CX2341X_ENC_SET_NUM_VSYNC_LINES)
+	API_ENTRY(CX2341X_ENC_SET_PLACEHOLDER)
+	API_ENTRY(CX2341X_ENC_MUTE_VIDEO)
+	API_ENTRY(CX2341X_ENC_MUTE_AUDIO)
+	API_ENTRY(CX2341X_ENC_UNKNOWN)
+	API_ENTRY(CX2341X_ENC_MISC)
 
 	/* OSD API */
-	API_ENTRY(IVTV_API_FB_GET_FRAMEBUFFER)
-	API_ENTRY(IVTV_API_FB_GET_PIXEL_FORMAT)
-	API_ENTRY(IVTV_API_FB_SET_PIXEL_FORMAT)
-	API_ENTRY(IVTV_API_FB_GET_STATE)
-	API_ENTRY(IVTV_API_FB_SET_STATE)
-	API_ENTRY(IVTV_API_FB_GET_OSD_COORDS)
-	API_ENTRY(IVTV_API_FB_SET_OSD_COORDS)
-	API_ENTRY(IVTV_API_FB_GET_SCREEN_COORDS)
-	API_ENTRY(IVTV_API_FB_SET_SCREEN_COORDS)
-	API_ENTRY(IVTV_API_FB_GET_GLOBAL_ALPHA)
-	API_ENTRY(IVTV_API_FB_SET_GLOBAL_ALPHA)
-	API_ENTRY(IVTV_API_FB_SET_BLEND_COORDS)
-	API_ENTRY(IVTV_API_FB_GET_FLICKER_STATE)
-	API_ENTRY(IVTV_API_FB_SET_FLICKER_STATE)
-	API_ENTRY(IVTV_API_FB_BLT_COPY)
-	API_ENTRY(IVTV_API_FB_BLT_FILL)
-	API_ENTRY(IVTV_API_FB_BLT_TEXT)
-	API_ENTRY(IVTV_API_FB_SET_FRAMEBUFFER_WINDOW)
-	API_ENTRY(IVTV_API_FB_SET_CHROMA_KEY)
-	API_ENTRY(IVTV_API_FB_GET_ALPHA_CONTENT_INDEX)
-	API_ENTRY(IVTV_API_FB_SET_ALPHA_CONTENT_INDEX)
+	API_ENTRY(CX2341X_OSD_GET_FRAMEBUFFER)
+	API_ENTRY(CX2341X_OSD_GET_PIXEL_FORMAT)
+	API_ENTRY(CX2341X_OSD_SET_PIXEL_FORMAT)
+	API_ENTRY(CX2341X_OSD_GET_STATE)
+	API_ENTRY(CX2341X_OSD_SET_STATE)
+	API_ENTRY(CX2341X_OSD_GET_OSD_COORDS)
+	API_ENTRY(CX2341X_OSD_SET_OSD_COORDS)
+	API_ENTRY(CX2341X_OSD_GET_SCREEN_COORDS)
+	API_ENTRY(CX2341X_OSD_SET_SCREEN_COORDS)
+	API_ENTRY(CX2341X_OSD_GET_GLOBAL_ALPHA)
+	API_ENTRY(CX2341X_OSD_SET_GLOBAL_ALPHA)
+	API_ENTRY(CX2341X_OSD_SET_BLEND_COORDS)
+	API_ENTRY(CX2341X_OSD_GET_FLICKER_STATE)
+	API_ENTRY(CX2341X_OSD_SET_FLICKER_STATE)
+	API_ENTRY(CX2341X_OSD_BLT_COPY)
+	API_ENTRY(CX2341X_OSD_BLT_FILL)
+	API_ENTRY(CX2341X_OSD_BLT_TEXT)
+	API_ENTRY(CX2341X_OSD_SET_FRAMEBUFFER_WINDOW)
+	API_ENTRY(CX2341X_OSD_SET_CHROMA_KEY)
+	API_ENTRY(CX2341X_OSD_GET_ALPHA_CONTENT_INDEX)
+	API_ENTRY(CX2341X_OSD_SET_ALPHA_CONTENT_INDEX)
 };
 
 static const char *get_cmd_name(int cmd)
@@ -299,8 +299,8 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 		return -EINVAL;
 
 	switch (cmd) {
-	case IVTV_API_SCHED_DMA_TO_HOST:
-	case IVTV_API_DEC_DMA_FROM_HOST:
+	case CX2341X_ENC_SCHED_DMA_TO_HOST:
+	case CX2341X_DEC_SCHED_DMA_FROM_HOST:
 		down(sem);
 		gotsem = 1;
 
@@ -322,7 +322,7 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 			       (unsigned char *)&local_box->timeout);
 
 		/* Dec DMA Timestamp */
-		if (cmd == IVTV_API_DEC_DMA_FROM_HOST)
+		if (cmd == CX2341X_DEC_SCHED_DMA_FROM_HOST)
 			itv->dec_dma_stat.ts = jiffies;
 
 		for (i = 0; i < args; i++) {
@@ -338,12 +338,12 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 			break;
 		goto ivtv_api_done;
 		/* Start/Stop capture calls for encoding are sensitive */
-	case IVTV_API_BEGIN_CAPTURE:
-	case IVTV_API_DEC_START_PLAYBACK:
-	case IVTV_API_END_CAPTURE:
-	case IVTV_API_EVENT_NOTIFICATION:
-	case IVTV_API_ENC_PING_FW:
-	case IVTV_API_DEC_PING_FW:
+	case CX2341X_ENC_START_CAPTURE:
+	case CX2341X_DEC_START_PLAYBACK:
+	case CX2341X_ENC_STOP_CAPTURE:
+	case CX2341X_ENC_SET_EVENT_NOTIFICATION:
+	case CX2341X_ENC_PING_FW:
+	case CX2341X_DEC_PING_FW:
 		down(sem);
 		gotsem = 1;
 
@@ -378,72 +378,72 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 		break;
 		/* These commands are not stored */
 		/* These don't need a result */
-	case IVTV_API_ENC_UNKNOWN:
-	case IVTV_API_ENC_MISC:
-	case IVTV_API_FB_BLT_COPY:
-	case IVTV_API_FB_BLT_FILL:
-	case IVTV_API_FB_BLT_TEXT:
-	case IVTV_API_DEC_DMA_BLOCKSIZE:
-	case IVTV_API_ASSIGN_DMA_BLOCKLEN:
-	case IVTV_API_INITIALIZE_INPUT:
-	case IVTV_API_REFRESH_INPUT:
-		if (cmd == IVTV_API_REFRESH_INPUT)
+	case CX2341X_ENC_UNKNOWN:
+	case CX2341X_ENC_MISC:
+	case CX2341X_OSD_BLT_COPY:
+	case CX2341X_OSD_BLT_FILL:
+	case CX2341X_OSD_BLT_TEXT:
+	case CX2341X_DEC_SET_DMA_BLOCK_SIZE:
+	case CX2341X_ENC_SET_DMA_BLOCK_SIZE:
+	case CX2341X_ENC_INITIALIZE_INPUT:
+	case CX2341X_ENC_REFRESH_INPUT:
+		if (cmd == CX2341X_ENC_REFRESH_INPUT)
 			intr = 1;
-	case IVTV_API_ASSIGN_NUM_VSYNC_LINES:
-	case IVTV_API_MUTE_VIDEO:
-	case IVTV_API_MUTE_AUDIO:
+	case CX2341X_ENC_SET_NUM_VSYNC_LINES:
+	case CX2341X_ENC_MUTE_VIDEO:
+	case CX2341X_ENC_MUTE_AUDIO:
 		needsresult = 0;
 
 	/* These commands need a result and are not stored */
-	case IVTV_API_PAUSE_ENCODER:
-	case IVTV_API_ENC_HALT_FW:
-		if (cmd == IVTV_API_ENC_HALT_FW) {
+	case CX2341X_ENC_PAUSE_ENCODER:
+	case CX2341X_ENC_HALT_FW:
+		if (cmd == CX2341X_ENC_HALT_FW) {
 			api_timeout = 100;
 			no_reset = 1;
 		}
-	case IVTV_API_ENC_GETVER:
-	case IVTV_API_DEC_GETVER:
-	case IVTV_API_DEC_HALT_FW:
-		if (cmd == IVTV_API_DEC_HALT_FW)
+	case CX2341X_ENC_GET_VERSION:
+	case CX2341X_DEC_GET_VERSION:
+	case CX2341X_DEC_HALT_FW:
+		if (cmd == CX2341X_DEC_HALT_FW)
 			api_timeout = 100;
-	case IVTV_API_DEC_XFER_INFO:
-		if (cmd == IVTV_API_DEC_XFER_INFO) {
+	case CX2341X_DEC_GET_XFER_INFO:
+		if (cmd == CX2341X_DEC_GET_XFER_INFO) {
 			api_timeout = 1000;
 			no_reset = 1;
 		}
-	case IVTV_API_DEC_STOP_PLAYBACK:
-		if (cmd == IVTV_API_DEC_STOP_PLAYBACK) {
+	case CX2341X_DEC_STOP_PLAYBACK:
+		if (cmd == CX2341X_DEC_STOP_PLAYBACK) {
 			api_timeout = 100;
 			no_reset = 1;
 		}
-	case IVTV_API_DEC_PLAYBACK_SPEED:
-	case IVTV_API_DEC_STEP_VIDEO:
-	case IVTV_API_DEC_PAUSE_PLAYBACK:
-	case IVTV_API_DEC_TIMING_INFO:
-	case IVTV_API_DEC_DMA_STATUS:
-	case IVTV_API_FB_GET_FRAMEBUFFER:
-	case IVTV_API_FB_GET_GLOBAL_ALPHA:
-	case IVTV_API_FB_GET_SCREEN_COORDS:
-	case IVTV_API_FB_GET_OSD_COORDS:
-	case IVTV_API_FB_GET_STATE:
-		if (cmd == IVTV_API_FB_GET_STATE) {
+	case CX2341X_DEC_SET_PLAYBACK_SPEED:
+	case CX2341X_DEC_STEP_VIDEO:
+	case CX2341X_DEC_PAUSE_PLAYBACK:
+	case CX2341X_DEC_GET_TIMING_INFO:
+	case CX2341X_DEC_GET_DMA_STATUS:
+	case CX2341X_OSD_GET_FRAMEBUFFER:
+	case CX2341X_OSD_GET_GLOBAL_ALPHA:
+	case CX2341X_OSD_GET_SCREEN_COORDS:
+	case CX2341X_OSD_GET_OSD_COORDS:
+	case CX2341X_OSD_GET_STATE:
+		if (cmd == CX2341X_OSD_GET_STATE) {
 			no_reset = 1;
 		}
-	case IVTV_API_FB_GET_PIXEL_FORMAT:
-	case IVTV_API_FB_GET_ALPHA_CONTENT_INDEX:
-	case IVTV_API_DEC_DECODE_SOURCE:
-	case IVTV_API_FB_SET_GLOBAL_ALPHA:
-	case IVTV_API_FB_SET_STATE:
-		if (cmd == IVTV_API_FB_SET_STATE) {
+	case CX2341X_OSD_GET_PIXEL_FORMAT:
+	case CX2341X_OSD_GET_ALPHA_CONTENT_INDEX:
+	case CX2341X_DEC_SET_DECODER_SOURCE:
+	case CX2341X_OSD_SET_GLOBAL_ALPHA:
+	case CX2341X_OSD_SET_STATE:
+		if (cmd == CX2341X_OSD_SET_STATE) {
 			api_timeout = 1000;
 			no_reset = 1;
 		}
-	case IVTV_API_FB_SET_FLICKER_STATE:
-	case IVTV_API_FB_SET_OSD_COORDS:
-	case IVTV_API_FB_SET_SCREEN_COORDS:
-	case IVTV_API_CONFIG_VBI:
-	case IVTV_API_SELECT_VBI_LINE:
-	case IVTV_API_DEC_EXTRACT_VBI:
+	case CX2341X_OSD_SET_FLICKER_STATE:
+	case CX2341X_OSD_SET_OSD_COORDS:
+	case CX2341X_OSD_SET_SCREEN_COORDS:
+	case CX2341X_ENC_SET_VBI_CONFIG:
+	case CX2341X_ENC_SET_VBI_LINE:
+	case CX2341X_DEC_EXTRACT_VBI:
 		if (!intr) {
 			down(sem);
 			gotsem = 1;
@@ -476,33 +476,33 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 
 		break;
 		/* These don't need a result */
-	case IVTV_API_DEC_SELECT_AUDIO:
-	case IVTV_API_DEC_DISPLAY_BUFFERS:
-	case IVTV_API_DEC_AUDIO_OUTPUT:
-	case IVTV_API_DEC_SET_AV_DELAY:
-	case IVTV_API_DEC_BUFFER:
-	case IVTV_API_DEC_STREAM_INPUT:
-	case IVTV_API_ASSIGN_PLACEHOLDER:
-	case IVTV_API_ASSIGN_FRAME_DROP_RATE:
-	case IVTV_API_ASSIGN_SPATIAL_FILTER_TYPE:
-	case IVTV_API_ASSIGN_CORING_LEVELS:
-	case IVTV_API_ASSIGN_DNR_FILTER_PROPS:
-	case IVTV_API_ASSIGN_DNR_FILTER_MODE:
-	case IVTV_API_ASSIGN_AUDIO_PROPERTIES:
-	case IVTV_API_ASSIGN_GOP_CLOSURE:
-	case IVTV_API_ASSIGN_3_2_PULLDOWN:
-	case IVTV_API_ASSIGN_GOP_PROPERTIES:
-	case IVTV_API_ASSIGN_BITRATES:
-	case IVTV_API_ASSIGN_ASPECT_RATIO:
-	case IVTV_API_ASSIGN_FRAME_SIZE:
-	case IVTV_API_ASSIGN_FRAMERATE:
-	case IVTV_API_ASSIGN_OUTPUT_PORT:
-	case IVTV_API_ASSIGN_STREAM_TYPE:
-	case IVTV_API_FB_SET_PIXEL_FORMAT:
-	case IVTV_API_FB_SET_BLEND_COORDS:
-	case IVTV_API_FB_SET_FRAMEBUFFER_WINDOW:
-	case IVTV_API_FB_SET_CHROMA_KEY:
-	case IVTV_API_FB_SET_ALPHA_CONTENT_INDEX:
+	case CX2341X_DEC_SET_AUDIO_MODE:
+	case CX2341X_DEC_SET_DISPLAY_BUFFERS:
+	case CX2341X_DEC_SET_AUDIO_OUTPUT:
+	case CX2341X_DEC_SET_AV_DELAY:
+	case CX2341X_DEC_SET_PREBUFFERING:
+	case CX2341X_DEC_SET_STREAM_INPUT:
+	case CX2341X_ENC_SET_PLACEHOLDER:
+	case CX2341X_ENC_SET_FRAME_DROP_RATE:
+	case CX2341X_ENC_SET_SPATIAL_FILTER_TYPE:
+	case CX2341X_ENC_SET_CORING_LEVELS:
+	case CX2341X_ENC_SET_DNR_FILTER_PROPS:
+	case CX2341X_ENC_SET_DNR_FILTER_MODE:
+	case CX2341X_ENC_SET_AUDIO_PROPERTIES:
+	case CX2341X_ENC_SET_GOP_CLOSURE:
+	case CX2341X_ENC_SET_3_2_PULLDOWN:
+	case CX2341X_ENC_SET_GOP_PROPERTIES:
+	case CX2341X_ENC_SET_BIT_RATE:
+	case CX2341X_ENC_SET_ASPECT_RATIO:
+	case CX2341X_ENC_SET_FRAME_SIZE:
+	case CX2341X_ENC_SET_FRAME_RATE:
+	case CX2341X_ENC_SET_OUTPUT_PORT:
+	case CX2341X_ENC_SET_STREAM_TYPE:
+	case CX2341X_OSD_SET_PIXEL_FORMAT:
+	case CX2341X_OSD_SET_BLEND_COORDS:
+	case CX2341X_OSD_SET_FRAMEBUFFER_WINDOW:
+	case CX2341X_OSD_SET_CHROMA_KEY:
+	case CX2341X_OSD_SET_ALPHA_CONTENT_INDEX:
 		needsresult = 0;
 	default:		/* Stored Commands */
 		if (!intr) {
@@ -658,7 +658,7 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 	return -EBUSY;
 }
 
-int ivtv_api_result(struct ivtv *itv, int cmd, int args, u32 * data)
+static int ivtv_api_result(struct ivtv *itv, int cmd, int args, u32 * data)
 {
 	u32 result;
 
