@@ -179,10 +179,10 @@ console hijacks, and allow you to unload the driver.
 #include "ivtv-mailbox.h"
 #include "ivtv-cards.h"
 
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 #include <video/fbcon.h>
 #include <video/fbcon-cfb32.h>
-#endif /* LINUX26 */
+#endif
 #include "ivtv-osd.h"
 
 typedef unsigned long uintptr_t;
@@ -229,7 +229,7 @@ static int ivtv_api_fb_get_framebuffer(struct ivtv *itv, u32 * fbbase,
 	int rc;
 
 	rc = ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		      IVTV_API_FB_GET_FRAMEBUFFER, &result, 0, &data[0]);
+		      CX2341X_OSD_GET_FRAMEBUFFER, &result, 0, &data[0]);
 	*fbbase = data[0];
 	*fblength = data[1];
 	return rc;
@@ -240,7 +240,7 @@ static int ivtv_api_fb_get_pixel_format(struct ivtv *itv)
 	u32 data[IVTV_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		 IVTV_API_FB_GET_PIXEL_FORMAT, &result, 0, &data[0]);
+		 CX2341X_OSD_GET_PIXEL_FORMAT, &result, 0, &data[0]);
 	return data[0];
 }
 
@@ -250,7 +250,7 @@ static int ivtv_api_fb_set_pixel_format(struct ivtv *itv, int format)
 	data[0] = format;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		 IVTV_API_FB_SET_PIXEL_FORMAT, &result, 1, &data[0]);
+		 CX2341X_OSD_SET_PIXEL_FORMAT, &result, 1, &data[0]);
 	return result;
 }
 
@@ -259,7 +259,7 @@ static int ivtv_api_fb_get_status(struct ivtv *itv)
 {
 	u32 data[IVTV_MBOX_MAX_DATA], result;
 
-	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, IVTV_API_FB_GET_STATE,
+	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_GET_STATE,
 		 &result, 0, &data[0]);
 
 	/* bit 1 is on/off bit */
@@ -274,7 +274,7 @@ static int ivtv_api_fb_get_state(struct ivtv *itv)
 {
 	u32 data[IVTV_MBOX_MAX_DATA], result;
 
-	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, IVTV_API_FB_GET_STATE,
+	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_GET_STATE,
 		 &result, 0, &data[0]);
 	return data[0];
 }
@@ -284,7 +284,7 @@ static int ivtv_api_fb_set_state(struct ivtv *itv, int enabled)
 	u32 params[IVTV_MBOX_MAX_DATA], result;
 	params[0] = enabled;
 
-	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, IVTV_API_FB_SET_STATE,
+	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_SET_STATE,
 		 &result, 1, &params[0]);
 	return result;
 }
@@ -301,7 +301,7 @@ static int ivtv_api_fb_set_framebuffer_window(struct ivtv *itv,
 	data[3] = top;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		 IVTV_API_FB_SET_FRAMEBUFFER_WINDOW, &result, 4, &data[0]);
+		 CX2341X_OSD_SET_FRAMEBUFFER_WINDOW, &result, 4, &data[0]);
 	return result;
 }
 
@@ -310,7 +310,7 @@ static int ivtv_api_fb_get_osd_coords(struct ivtv *itv,
 {
 	u32 data[IVTV_MBOX_MAX_DATA], result;
 
-	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, IVTV_API_FB_GET_OSD_COORDS,
+	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_GET_OSD_COORDS,
 		 &result, 0, &data[0]);
 
 	osd->offset = data[0] - itv->osd_info->video_rbase;
@@ -335,7 +335,7 @@ static int ivtv_api_fb_set_osd_coords(struct ivtv *itv, const struct ivtv_osd_co
 	data[4] = osd->y;
 
 	/* FIXME maybe wait on vsync? */
-	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, IVTV_API_FB_SET_OSD_COORDS,
+	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_SET_OSD_COORDS,
 		 &result, 5, &data[0]);
 
 	return result;
@@ -346,7 +346,7 @@ static int ivtv_api_fb_get_screen_coords(struct ivtv *itv, struct rectangle *r)
 	u32 data[IVTV_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		 IVTV_API_FB_GET_SCREEN_COORDS, &result, 0, &data[0]);
+		 CX2341X_OSD_GET_SCREEN_COORDS, &result, 0, &data[0]);
 
 	r->x0 = data[0];
 	r->y0 = data[1];
@@ -367,7 +367,7 @@ static int ivtv_api_fb_set_screen_coords(struct ivtv *itv,
 	data[3] = r->y1;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		 IVTV_API_FB_SET_SCREEN_COORDS, &result, 4, &data[0]);
+		 CX2341X_OSD_SET_SCREEN_COORDS, &result, 4, &data[0]);
 	return result;
 }
 
@@ -376,7 +376,7 @@ static int ivtv_api_fb_get_global_alpha(struct ivtv *itv)
 	u32 data[IVTV_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		 IVTV_API_FB_GET_GLOBAL_ALPHA, &result, 0, &data[0]);
+		 CX2341X_OSD_GET_GLOBAL_ALPHA, &result, 0, &data[0]);
 	return data[1];
 }
 
@@ -395,7 +395,7 @@ static int ivtv_api_fb_set_global_alpha(struct ivtv *itv,
 	data[1] = alpha;
 	data[2] = !enable_local;
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		 IVTV_API_FB_SET_GLOBAL_ALPHA, &result, 3, &data[0]);
+		 CX2341X_OSD_SET_GLOBAL_ALPHA, &result, 3, &data[0]);
 
 	return result;
 }
@@ -409,7 +409,7 @@ static void  ivtv_api_fb_set_colorKey(struct ivtv *itv,int state,  uint32_t colo
 	data[0] = state;
 	data[1] = color;
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		 IVTV_API_FB_SET_CHROMA_KEY, &result, 2, &data[0]);
+		 CX2341X_OSD_SET_CHROMA_KEY, &result, 2, &data[0]);
 
 }
 static int ivtv_api_fb_get_flicker_state(struct ivtv *itv)
@@ -417,7 +417,7 @@ static int ivtv_api_fb_get_flicker_state(struct ivtv *itv)
 	u32 data[IVTV_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		 IVTV_API_FB_GET_FLICKER_STATE, &result, 0, &data[0]);
+		 CX2341X_OSD_GET_FLICKER_STATE, &result, 0, &data[0]);
 	return data[0];
 }
 
@@ -428,7 +428,7 @@ static int ivtv_api_fb_set_flicker_state(struct ivtv *itv, int enabled)
 	params[0] = enabled;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-		 IVTV_API_FB_SET_FLICKER_STATE, &result, 1, &params[0]);
+		 CX2341X_OSD_SET_FLICKER_STATE, &result, 1, &params[0]);
 	return result;
 }
 
@@ -451,7 +451,7 @@ static int ivtv_api_fb_blt_fill(struct ivtv *itv, int rasterop,
 	data[8] = value;
 
 	result =
-	    ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, IVTV_API_FB_BLT_FILL,
+	    ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_BLT_FILL,
 		     &result, 9, &data[0]);
 	return result;
 }
@@ -481,7 +481,7 @@ static int ivtv_api_fb_blt_copy(struct ivtv *itv, int rasterop,
 	data[9] = sourceaddr;
 
 	result =
-	    ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, IVTV_API_FB_BLT_COPY,
+	    ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_BLT_COPY,
 		     &result, 10, &data[0]);
 	return result;
 }
@@ -523,22 +523,22 @@ static struct fb_var_screeninfo ivtvfb_defined = {
 	0L, 0L, 0L, 0L, 0L,
 	0L, 0L, 0,		/* No sync info */
 	FB_VMODE_NONINTERLACED,
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	{0, 0, 0, 0, 0, 0}
 #else
 	0,
 	{0, 0, 0, 0, 0}
-#endif /* LINUX26 */
+#endif
 };
 
 static struct fb_info fb_info;
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 static struct display disp;
 static union {
 	u32 cfb32[16];
 } fbcon_cmap;
 static struct display_switch ivtvfb_sw;
-#endif /* LINUX26 */
+#endif
 
 #ifdef CONFIG_MTRR
 static int mtrr = 1;		//++MTY
@@ -578,10 +578,9 @@ static int _ivtvfb_get_fix(struct ivtv *itv, struct fb_fix_screeninfo *fix)
 	return 0;
 }
 
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 static int ivtvfb_update_var(int con, struct fb_info *info)
 {
-	struct ivtv *itv = (struct ivtv *) info->par;
 	IVTV_OSD_DEBUG_INFO("ivtvfb_update_var\n");
 	return 0;
 }
@@ -589,15 +588,13 @@ static int ivtvfb_update_var(int con, struct fb_info *info)
 static int ivtvfb_get_fix(struct fb_fix_screeninfo *fix, int con,
 			  struct fb_info *info)
 {
-	struct ivtv *itv = (struct ivtv *) info->par;
 	IVTV_OSD_DEBUG_INFO("ivtvfb_get_fix\n");
-	return _ivtvfb_get_fix(itv, fix);
+	return _ivtvfb_get_fix(fix);
 }
 
 static int ivtvfb_get_var(struct fb_var_screeninfo *var, int con,
 			  struct fb_info *info)
 {
-	struct ivtv *itv = (struct ivtv *) info->par;
 	IVTV_OSD_DEBUG_INFO("ivtvfb_get_var\n");
 	if (con == -1)
 		memcpy(var, &ivtvfb_defined, sizeof(struct fb_var_screeninfo));
@@ -606,10 +603,9 @@ static int ivtvfb_get_var(struct fb_var_screeninfo *var, int con,
 	return 0;
 }
 
-static void ivtvfb_set_disp(struct ivtv *itv, int con)
+static void ivtvfb_set_disp(int con)
 {
 	struct fb_fix_screeninfo fix;
-	struct fb_info dummy;
 	struct display *display;
 	struct display_switch *sw;
 
@@ -619,8 +615,7 @@ static void ivtvfb_set_disp(struct ivtv *itv, int con)
 	else
 		display = &disp;	/* used during initialization */
 
-	dummy.par = itv;
-	ivtvfb_get_fix(&fix, con, &dummy);
+	ivtvfb_get_fix(&fix, con, 0);
 
 	memset(display, 0, sizeof(struct display));
 	display->screen_base = itv->osd_info->video_vbase;
@@ -646,13 +641,12 @@ static void ivtvfb_set_disp(struct ivtv *itv, int con)
 static int ivtvfb_set_var(struct fb_var_screeninfo *var, int con,
 			  struct fb_info *info)
 {
-	struct ivtv *itv = (struct ivtv *) info->par;
 	int first = 0;
 	IVTV_OSD_DEBUG_INFO("ivtvfb_set_var\n");
 	if (con >= 0)
 		first = 1;
 
-	if (_ivtvfb_set_var(itv, var)) {
+	if (_ivtvfb_set_var(var)) {
 		if (first) {
 			IVTV_OSD_ERR
 			    ("changing the video mode is not supported\n");
@@ -728,7 +722,7 @@ static int ivtv_fb_sync(struct fb_info *info)
 	return 0;
 }
 
-#endif /* LINUX26 */
+#endif
 
 static int ivtv_fb_blt_copy(struct ivtv *itv, int x, int y, int width,
 			    int height, int source_offset, int source_stride)
@@ -1191,13 +1185,17 @@ static int ivtv_prep_frame(struct ivtv *itv, int cmd, void *source, unsigned lon
         return ret;
 }
 
-#ifndef LINUX26
-int ivtv_fb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
+static int ivtv_fb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
 		  unsigned long arg, int con, struct fb_info *info)
 #else
-int ivtv_fb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
+static int ivtv_fb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
 		  unsigned long arg, struct fb_info *info)
-#endif /* LINUX26 */
+#else
+static int ivtv_fb_ioctl(struct fb_info *info, unsigned int cmd, unsigned long arg)
+#endif
+#endif
 {
 
 	int rc;
@@ -1205,14 +1203,14 @@ int ivtv_fb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
 	struct ivtv *itv = (struct ivtv *) info->par;
 
 	switch (cmd) {
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	case 0x7777: {
 		while (MOD_IN_USE)
 			MOD_DEC_USE_COUNT;
 		MOD_INC_USE_COUNT;
 		return 0;
 	}
-#endif /* LINUX26 */
+#endif
 	case IVTVFB_IOCTL_GET_STATE: {
 		struct ivtvfb_ioctl_state_info state;
 
@@ -1398,9 +1396,12 @@ ivtv_fb_write(struct file *file, const char *ubuf, size_t count, loff_t * ppos)
 #endif
 }
 
-static int
-ivtv_fb_mmap(struct fb_info *info,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
+static int ivtv_fb_mmap(struct fb_info *info,
 	     struct file *file, struct vm_area_struct *vma)
+#else
+static int ivtv_fb_mmap(struct fb_info *info, struct vm_area_struct *vma)
+#endif
 {
 	struct ivtv *itv = (struct ivtv *) info->par;
 	IVTV_OSD_DEBUG_WARN( "MMAP is not safe for ivtv usage\n");
@@ -1409,7 +1410,7 @@ ivtv_fb_mmap(struct fb_info *info,
 
 static struct fb_ops ivtvfb_ops = {
       owner:THIS_MODULE,
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
       fb_get_fix:ivtvfb_get_fix,
       fb_get_var:ivtvfb_get_var,
       fb_set_var:ivtvfb_set_var,
@@ -1426,7 +1427,7 @@ static struct fb_ops ivtvfb_ops = {
       fb_cursor:ivtv_fb_cursor,
       fb_rotate:ivtv_fb_rotate,
       fb_sync:ivtv_fb_sync,
-#endif /* LINUX26 */
+#endif
       fb_ioctl:ivtv_fb_ioctl,
       fb_pan_display:NULL,
       fb_mmap:ivtv_fb_mmap,
@@ -1434,12 +1435,12 @@ static struct fb_ops ivtvfb_ops = {
 
 /* 0 unblank, 1 blank, 2 no vsync, 3 no hsync, 4 off */
 
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 static void ivtvfb_blank(int blank, struct fb_info *info)
 {
 	/* Not supported */
 }
-#endif /* LINUX26 */
+#endif
 
 static int ivtvfb_setup(struct ivtv *itv)
 {
@@ -1452,17 +1453,17 @@ static int ivtvfb_setup(struct ivtv *itv)
 	IVTV_OSD_DEBUG_INFO("Current pixel format = %d\n",
 		       ivtv_api_fb_get_pixel_format(itv));
 
-	itv->osd_info->display_width = 720;
+ 	itv->osd_info->display_width = 720;
+
 	if (itv->std & V4L2_STD_625_50) {
-		itv->osd_info->display_height = 576;
+ 		itv->osd_info->display_height = 576;
 	} else {
-		itv->osd_info->display_height = 480;
+ 		itv->osd_info->display_height = 480;
 	}
 
 	/* set number of internal decoder buffers */
 	ivtv_vapi(itv,
-		  IVTV_API_DEC_DISPLAY_BUFFERS, 1, itv->dec_options.decbuffers);
-
+		  CX2341X_DEC_SET_DISPLAY_BUFFERS, 1, itv->dec_options.decbuffers);
 
 	rc = ivtv_api_fb_get_osd_coords(itv, &osd);
 	IVTV_OSD_DEBUG_INFO(
@@ -1539,9 +1540,9 @@ static int ivtvfb_setup(struct ivtv *itv)
 
 int __init ivtvfb_init(void)
 {
-#ifdef LINUX26
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	struct fb_fix_screeninfo fix;
-#endif /* LINUX26 */
+#endif
 	int i;
         
         struct ivtv *itv;
@@ -1685,11 +1686,10 @@ int __init ivtvfb_init(void)
 	fb_info.node = -1;
 	fb_info.flags = FBINFO_FLAG_DEFAULT;
 	fb_info.fbops = &ivtvfb_ops;
-        fb_info.par = itv;
-
+	fb_info.par = itv;
         if (osd_init == 0)
-            memset(itv->osd_info->video_vbase, 0, itv->osd_info->display_width * itv->osd_info->display_height*4);
-#ifndef LINUX26
+		memset(itv->osd_info->video_vbase, 0, itv->osd_info->display_width * itv->osd_info->display_height*4);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	strcpy(fb_info.modename, "cx23415 TV out");
 	fb_info.fontname[0] = '\0';
 	fb_info.changevar = NULL;
@@ -1697,7 +1697,7 @@ int __init ivtvfb_init(void)
 	fb_info.switch_con = NULL;	//&ivtvfb_switch; (fbcon will ignore it then)
 	fb_info.updatevar = &ivtvfb_update_var;
 	fb_info.blank = &ivtvfb_blank;
-	ivtvfb_set_disp(itv, -1);
+	ivtvfb_set_disp(-1);
 #else
 	_ivtvfb_get_fix(itv,&fix);
 	fb_info.var = ivtvfb_defined;
@@ -1705,12 +1705,12 @@ int __init ivtvfb_init(void)
 	fb_info.screen_base = itv->osd_info->video_vbase;
 	fb_info.fbops = &ivtvfb_ops;
 	fb_alloc_cmap(&fb_info.cmap, 0, 0);
-#endif /* LINUX26 */
+#endif
 
 	if (register_framebuffer(&fb_info) < 0)
 		return -EINVAL;
 
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	itv->fb_id = GET_FB_IDX(fb_info.node);
 
 	IVTV_OSD_INFO("fb%d: %s frame buffer device\n",
@@ -1720,7 +1720,7 @@ int __init ivtvfb_init(void)
 
 	IVTV_OSD_INFO("fb%d: %s frame buffer device\n",
 			   itv->fb_id, fix.id);
-#endif /* LINUX26 */
+#endif
 
 	/* Allocate DMA */
 	ivtv_alloc_user_dma(itv,
@@ -1732,11 +1732,11 @@ int __init ivtvfb_init(void)
 	}
 
 	/* Allow module to unload */
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	while (MOD_IN_USE)
 		MOD_DEC_USE_COUNT;
 	MOD_INC_USE_COUNT;
-#endif /* LINUX26 */
+#endif
 
 	return 0;
 }
