@@ -53,14 +53,14 @@ struct osd_info {
 	// Frame buffer stuff
 	struct fb_info ivtvfb_info;
 	struct fb_var_screeninfo ivtvfb_defined;
-#ifdef LINUX26
-	struct fb_fix_screeninfo ivtvfb_fix;
-#else
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 	struct display disp;
 	union {
 		u32 cfb32[16];
 	} fbcon_cmap;
 	struct display_switch ivtvfb_sw;
+#else
+	struct fb_fix_screeninfo ivtvfb_fix;
 #endif /* LINUX26 */
 
 
