@@ -60,29 +60,6 @@ int ivtv_buf_fill_nolock(struct ivtv_stream *stream, u32 count, int inc)
 	return stream->buf_fill;
 }
 
-int ivtv_buf_fill_lock_irq(struct ivtv_stream *stream, u32 count, int inc)
-{
-	unsigned long flags;
-	int ret;
-
-	spin_lock_irqsave(&stream->slock, flags);
-	ret = ivtv_buf_fill_nolock(stream, count, inc);
-	spin_unlock_irqrestore(&stream->slock, flags);
-
-	return ret;
-}
-
-int ivtv_buf_fill_lock_bh(struct ivtv_stream *stream, u32 count, int inc)
-{
-	int ret;
-
-	spin_lock_bh(&stream->slock);
-	ret = ivtv_buf_fill_nolock(stream, count, inc);
-	spin_unlock_bh(&stream->slock);
-
-	return ret;
-}
-
 int ivtv_buf_fill_lock(struct ivtv_stream *stream, u32 count, int inc)
 {
 	int ret;
@@ -102,30 +79,6 @@ void ivtv_enq_buf_nolock(struct ivtv_buffer_list *queue,
 	atomic_inc(&queue->elements);
 }
 
-/* Adds buffers to the tail, effectively making a queue */
-int ivtv_enq_buf_lock_irq(struct ivtv_buffer_list *queue,
-			  struct ivtv_buffer *buf, struct ivtv_stream *stream)
-{
-	unsigned long flags;
-
-	spin_lock_irqsave(&stream->slock, flags);
-	ivtv_enq_buf_nolock(queue, buf);
-	spin_unlock_irqrestore(&stream->slock, flags);
-
-	return 0;
-}
-
-/* Adds buffers to the tail, effectively making a queue */
-int ivtv_enq_buf_lock_bh(struct ivtv_buffer_list *queue,
-			 struct ivtv_buffer *buf, struct ivtv_stream *stream)
-{
-	spin_lock_bh(&stream->slock);
-	ivtv_enq_buf_nolock(queue, buf);
-	spin_unlock_bh(&stream->slock);
-
-	return 0;
-}
-
 /* Adds buffers to the tail, effectively making a queue */
 int ivtv_enq_buf_lock(struct ivtv_buffer_list *queue,
 		      struct ivtv_buffer *buf, struct ivtv_stream *stream)
@@ -137,134 +90,6 @@ int ivtv_enq_buf_lock(struct ivtv_buffer_list *queue,
 	return 0;
 }
 
-void ivtv_del_buf_nolock(struct ivtv_buffer_list *queue,
-			 struct ivtv_buffer *buffer)
-{
-	WARN_ON(list_empty(&buffer->list));
-	list_del_init(&buffer->list);
-	atomic_dec(&queue->elements);
-}
-
-void ivtv_del_buf_lock_irq(struct ivtv_buffer_list *queue,
-			   struct ivtv_buffer *buffer,
-			   struct ivtv_stream *stream)
-{
-	unsigned long flags;
-
-	spin_lock_irqsave(&stream->slock, flags);
-	ivtv_del_buf_nolock(queue, buffer);
-	spin_unlock_irqrestore(&stream->slock, flags);
-}
-
-void ivtv_del_buf_lock_bh(struct ivtv_buffer_list *queue,
-			  struct ivtv_buffer *buffer,
-			  struct ivtv_stream *stream)
-{
-	spin_lock_bh(&stream->slock);
-	ivtv_del_buf_nolock(queue, buffer);
-	spin_unlock_bh(&stream->slock);
-}
-
-void ivtv_del_buf_lock(struct ivtv_buffer_list *queue,
-		       struct ivtv_buffer *buffer, struct ivtv_stream *stream)
-{
-	down(&stream->mlock);
-	ivtv_del_buf_nolock(queue, buffer);
-	up(&stream->mlock);
-}
-
-void ivtv_move_buf_lock_irq(struct ivtv_buffer_list *from,
-			    struct ivtv_buffer_list *to,
-			    struct ivtv_buffer *buffer,
-			    struct ivtv_stream *stream)
-{
-	unsigned long flags;
-
-	WARN_ON(list_empty(&buffer->list));
-
-	spin_lock_irqsave(&stream->slock, flags);
-	list_move_tail(&buffer->list, &to->list);
-	atomic_dec(&from->elements);
-	atomic_inc(&to->elements);
-	spin_unlock_irqrestore(&stream->slock, flags);
-}
-
-void ivtv_move_buf_lock_bh(struct ivtv_buffer_list *from,
-			   struct ivtv_buffer_list *to,
-			   struct ivtv_buffer *buffer,
-			   struct ivtv_stream *stream)
-{
-	WARN_ON(list_empty(&buffer->list));
-
-	spin_lock_bh(&stream->slock);
-	list_move_tail(&buffer->list, &to->list);
-	atomic_dec(&from->elements);
-	atomic_inc(&to->elements);
-	spin_unlock_bh(&stream->slock);
-}
-
-void ivtv_move_buf_lock(struct ivtv_buffer_list *from,
-			struct ivtv_buffer_list *to, struct ivtv_buffer *buffer,
-			struct ivtv_stream *stream)
-{
-	WARN_ON(list_empty(&buffer->list));
-
-	down(&stream->mlock);
-	list_move_tail(&buffer->list, &to->list);
-	atomic_dec(&from->elements);
-	atomic_inc(&to->elements);
-	up(&stream->mlock);
-}
-
-/* returns first item in queue, doesn't dequeue */
-struct ivtv_buffer *ivtv_deq_peek_head_nolock(struct ivtv *itv, struct ivtv_buffer_list *queue,
-					      int type)
-{
-	/* make sure list has something to DeQ */
-	if (!list_empty(&queue->list))
-		return list_entry(queue->list.next, struct ivtv_buffer, list);
-
-	IVTV_DEBUG_DMA("deqpeek: %d DeQ from empty list\n", type);
-	return NULL;
-}
-
-struct ivtv_buffer *ivtv_deq_peek_head_lock(struct ivtv *itv, struct ivtv_buffer_list *queue,
-					    struct ivtv_stream *stream)
-{
-	struct ivtv_buffer *buffer;
-
-	down(&stream->mlock);
-	buffer = ivtv_deq_peek_head_nolock(itv, queue, stream->type);
-	up(&stream->mlock);
-
-	return buffer;
-}
-
-struct ivtv_buffer *ivtv_deq_peek_head_lock_bh(struct ivtv *itv, struct ivtv_buffer_list *queue,
-					       struct ivtv_stream *stream)
-{
-	struct ivtv_buffer *buffer;
-
-	spin_lock_bh(&stream->slock);
-	buffer = ivtv_deq_peek_head_nolock(itv, queue, stream->type);
-	spin_unlock_bh(&stream->slock);
-
-	return buffer;
-}
-
-struct ivtv_buffer *ivtv_deq_peek_head_lock_irq(struct ivtv *itv, struct ivtv_buffer_list *queue,
-						struct ivtv_stream *stream)
-{
-	struct ivtv_buffer *buffer;
-	unsigned long flags;
-
-	spin_lock_irqsave(&stream->slock, flags);
-	buffer = ivtv_deq_peek_head_nolock(itv, queue, stream->type);
-	spin_unlock_irqrestore(&stream->slock, flags);
-
-	return buffer;
-}
-
 /* removes buffer from the head */
 struct ivtv_buffer *ivtv_deq_buf_nolock(struct ivtv *itv, struct ivtv_buffer_list *queue,
 					int type)
@@ -295,69 +120,18 @@ struct ivtv_buffer *ivtv_deq_buf_lock(struct ivtv *itv, struct ivtv_buffer_list
 	return buf;
 }
 
-struct ivtv_buffer *ivtv_deq_buf_lock_irq(struct ivtv *itv, struct ivtv_buffer_list *queue,
-					  struct ivtv_stream *stream)
-{
-	struct ivtv_buffer *buf;
-	unsigned long flags;
-
-	spin_lock_irqsave(&stream->slock, flags);
-	buf = ivtv_deq_buf_nolock(itv, queue, stream->type);
-	spin_unlock_irqrestore(&stream->slock, flags);
-
-	return buf;
-}
-
-struct ivtv_buffer *ivtv_deq_buf_lock_bh(struct ivtv *itv, struct ivtv_buffer_list *queue,
-					 struct ivtv_stream *stream)
-{
-	struct ivtv_buffer *buf;
-
-	spin_lock_bh(&stream->slock);
-	buf = ivtv_deq_buf_nolock(itv, queue, stream->type);
-	spin_unlock_bh(&stream->slock);
-
-	return buf;
-}
-
-void ivtv_free_page_buf (struct ivtv_stream *st, struct ivtv_buffer *buf)
-{
-        const unsigned int tail_size = st->bufsize % PAGE_SIZE;
-        const unsigned int full_pages =
-                st->bufsize / PAGE_SIZE * 1;
-        const unsigned int tails_per_page = (tail_size > 0) ?
-                PAGE_SIZE / tail_size : 0;
-        const unsigned int tail_pages = (tail_size > 0) ?
-                1 / tails_per_page +
-                ((1 % tails_per_page) ? 1 : 0) : 0;
-        const unsigned int total_pages = full_pages + tail_pages;
-        unsigned int i;
-
-        for (i = 0; i < total_pages; i++) {
-                free_page (buf->page[i]);
-        }
-        kfree (buf->vpage);
-        kfree (buf->page);
-        return;
-}
-
 void ivtv_free_buffer(struct ivtv *itv, struct ivtv_buffer *item,
 		      struct ivtv_stream *st)
 {
 	if (item->buffer.m.userptr != 0) {
-#if 0
-		ivtv_free_page_buf(st, item);
-#else
 		/* UnMap Buffer */
 		if (st->dma != PCI_DMA_NONE) {
 			pci_unmap_single(itv->dev, item->dma_handle,
-					 item->buffer.length + 256, st->dma);
+					 item->buffer.length, st->dma);
 		}
 
 		kfree((void *)item->buffer.m.userptr);
 		item->buffer.m.userptr = 0;
-#endif /* 0 */
-
 		atomic_dec(&st->allocated_buffers);
 		st->buf_total -= item->buffer.length;
 	}
@@ -368,7 +142,7 @@ void ivtv_free_buffer(struct ivtv *itv, struct ivtv_buffer *item,
 	}
 }
 
-void ivtv_free_queues_nolock(struct ivtv *itv, int type)
+static void ivtv_free_queues_nolock(struct ivtv *itv, int type)
 {
 	struct ivtv_stream *st = &itv->streams[type];
 	struct ivtv_buffer *item;
@@ -383,26 +157,7 @@ void ivtv_free_queues_nolock(struct ivtv *itv, int type)
 		ivtv_free_buffer(itv, item, st);
 }
 
-void ivtv_free_queues_lock_irq(struct ivtv *itv, int type)
-{
-	struct ivtv_stream *st = &itv->streams[type];
-	unsigned long flags;
-
-	spin_lock_irqsave(&st->slock, flags);
-	ivtv_free_queues_nolock(itv, type);
-	spin_unlock_irqrestore(&st->slock, flags);
-}
-
-void ivtv_free_queues_lock_bh(struct ivtv *itv, int type)
-{
-	struct ivtv_stream *st = &itv->streams[type];
-
-	spin_lock_bh(&st->slock);
-	ivtv_free_queues_nolock(itv, type);
-	spin_unlock_bh(&st->slock);
-}
-
-void ivtv_free_queues_lock(struct ivtv *itv, int type)
+static void ivtv_free_queues_lock(struct ivtv *itv, int type)
 {
 	struct ivtv_stream *st = &itv->streams[type];
 
@@ -482,24 +237,13 @@ struct ivtv_buffer *ivtv_init_buffer(struct ivtv *itv, struct ivtv_stream *st)
 		IVTV_DEBUG_WARN("No memory on ivtv_buffer alloc!\n");
 		return NULL;
 	}
-#if 0
-	if ((buf->buffer.m.userptr = 
-		(unsigned long)ivtv_page_buf(st, buf)) == 0) 
-	{
-
-		ivtv_free_page_buf(st, buf);
-		IVTV_DEBUG_WARN("No memory on buffer alloc!\n");
-		return NULL;
-	}
-#else
- 	buf->buffer.m.userptr = (unsigned long)kmalloc(st->bufsize+256, GFP_KERNEL);
+	buf->buffer.m.userptr = (unsigned long)kmalloc(st->bufsize, GFP_KERNEL);
 
 	if (buf->buffer.m.userptr == 0) {
 		kfree(buf);
 		IVTV_DEBUG_WARN("No memory on buffer alloc!\n");
 		return NULL;
 	}
-#endif /* 0 */
 	buf->b_flags = 0;
 	buf->ts = 0;
 	buf->buffer.length = st->bufsize;
@@ -544,7 +288,7 @@ struct ivtv_buffer *ivtv_init_buffer(struct ivtv *itv, struct ivtv_stream *st)
 	if (st->dma != PCI_DMA_NONE) {
 		buf->dma_handle = pci_map_single(itv->dev,
 						 (void *)buf->buffer.m.userptr,
-						 buf->buffer.length + 256, st->dma);
+						 buf->buffer.length, st->dma);
 	}
 
 	return buf;
@@ -578,7 +322,7 @@ static int ivtv_init_queue_nolock(struct ivtv *itv,
 	return x;
 }
 
-int ivtv_init_queues_nolock(struct ivtv *itv, int streamtype, int buffers,
+static int ivtv_init_queues_nolock(struct ivtv *itv, int streamtype, int buffers,
 			    int bufsize)
 {
 	struct ivtv_stream *st = &itv->streams[streamtype];
@@ -596,43 +340,6 @@ int ivtv_init_queues_nolock(struct ivtv *itv, int streamtype, int buffers,
 	return 0;
 }
 
-int ivtv_init_queues_lock_bh(struct ivtv *itv, int streamtype, int buffers,
-			     int bufsize)
-{
-	struct ivtv_stream *st = &itv->streams[streamtype];
-	int ret;
-
-	spin_lock_bh(&st->slock);
-	ret = ivtv_init_queues_nolock(itv, streamtype, buffers, bufsize);
-	spin_unlock_bh(&st->slock);
-	return ret;
-}
-
-int ivtv_init_queues_lock(struct ivtv *itv, int streamtype, int buffers,
-			  int bufsize)
-{
-	struct ivtv_stream *st = &itv->streams[streamtype];
-	int ret;
-
-	down(&st->mlock);
-	ret = ivtv_init_queues_nolock(itv, streamtype, buffers, bufsize);
-	up(&st->mlock);
-	return ret;
-}
-
-int ivtv_move_queue(struct ivtv *itv, struct ivtv_buffer_list *src,
-		    struct ivtv_buffer_list *dst, struct ivtv_stream *stream)
-{
-	struct ivtv_buffer *buf;
-
-	down(&stream->mlock);
-	while ((buf = ivtv_deq_buf_nolock(itv, src, stream->type)))
-		ivtv_enq_buf_nolock(dst, buf);
-	up(&stream->mlock);
-
-	return 0;
-}
-
 void ivtv_flush_queues_nolock(struct ivtv *itv, int type)
 {
 	struct ivtv_stream *st = &itv->streams[type];
@@ -713,25 +420,6 @@ void ivtv_flush_queues_nolock(struct ivtv *itv, int type)
 	}
 }
 
-void ivtv_flush_queues_lock_bh(struct ivtv *itv, int type)
-{
-	struct ivtv_stream *st = &itv->streams[type];
-
-	spin_lock_bh(&st->slock);
-	ivtv_flush_queues_nolock(itv, type);
-	spin_unlock_bh(&st->slock);
-}
-
-void ivtv_flush_queues_lock_irq(struct ivtv *itv, int type)
-{
-	struct ivtv_stream *st = &itv->streams[type];
-	unsigned long flags;
-
-	spin_lock_irqsave(&st->slock, flags);
-	ivtv_flush_queues_nolock(itv, type);
-	spin_unlock_irqrestore(&st->slock, flags);
-}
-
 void ivtv_flush_queues_lock(struct ivtv *itv, int type)
 {
 	struct ivtv_stream *st = &itv->streams[type];
@@ -853,9 +541,9 @@ int ivtv_prep_user_dma(struct ivtv *itv, struct ivtv_user_dma *dma,
 		dma->SGlist[i].length = PAGE_SIZE - offset;
 		dma->SGlist[i].offset = offset;
 		dma->SGlist[i].page = dma->map[i];
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 		dma->SGlist[i].address = 0;
-#endif /* LINUX26 */
+#endif
 		offset = 0;
 	}
 
@@ -969,67 +657,6 @@ void ivtv_free_user_dma(struct ivtv *itv, struct ivtv_user_dma *dma)
 	return;
 }
 
-int user_dma_to_device(struct ivtv *itv, unsigned long ivtv_dest_addr,
-		       char *userbuf, int size_in_bytes, int streamtype)
-{
-	struct ivtv_stream *stream = &itv->streams[streamtype];
-	u32 data[IVTV_MBOX_MAX_DATA];
-	int ret = 0;
-
-	/* Map User DMA */
-	if (ivtv_prep_user_dma(itv,
-			       &stream->udma, ivtv_dest_addr, userbuf,
-			       size_in_bytes) <= 0) {
-		IVTV_DEBUG_WARN(
-			   "user_dma_to_device, "
-			   "Error with get_user_pages: %d bytes, %d pages returned\n",
-			   size_in_bytes, stream->udma.page_count);
-
-		/* get_user_pages must have failed completely */
-		return -EIO;
-	}
-
-	IVTV_DEBUG_INFO(
-		   "user_dma_to_device, %d bytes, %d pages\n",
-		   size_in_bytes, stream->udma.page_count);
-
-	/* Decoder DMA State */
-	itv->dec_dma_stat.last_xfer = stream->udma.sg_size;
-	atomic_set(&itv->dec_dma_stat.type, 1);
-
-	data[0] = stream->udma.SG_handle;
-	data[1] = stream->udma.sg_size;
-	data[2] = 0x01;
-
-	IVTV_DEBUG_INFO(
-		   "Sched DMA: addr: 0x%08x, array_size 0x%08x, type 0x%08x\n",
-		   data[0], data[1], data[2]);
-
-	down(&itv->DMA_lock);
-	if (dma_to_device(itv, stream, stream->udma.SG_handle, 0) != 0) {
-		IVTV_DEBUG_WARN("Error User DMA\n");
-	}
-	up(&itv->DMA_lock);
-
-	/* Unmap Last DMA Xfer */
-	if (ivtv_unmap_user_dma(itv, &stream->udma) == 1) {
-		IVTV_DEBUG_WARN("Error: unmap osd dma\n");
-	}
-
-	/* Lock DEC DMA */
-	/* Tell Firmware we are done */
-	ivtv_vapi(itv, IVTV_API_SCHED_DMA_TO_HOST, 3, 0, 0, 1);
-
-	/* Unlock stream */
-	clear_bit(DMA_IN_USE, &stream->udma.u_flags);
-	clear_bit(IVTV_F_S_DMAP, &stream->s_flags);
-
-	/* Wake up next OSD/DEC DMA in wait */
-	wake_up(&stream->udma.waitq);
-
-	return ret;
-}
-
 int dec_gather_free_buffers(struct ivtv *itv, int streamtype,
 			    struct list_head *free_list, int bytes_needed)
 {
@@ -1154,39 +781,6 @@ int enc_gather_free_buffers(struct ivtv *itv, int streamtype,
 	return 0;
 }
 
-int dec_dma_wait(struct ivtv *itv, struct ivtv_stream *stream)
-{
-	int x = 0;
-	DECLARE_WAITQUEUE(wait, current);
-
-	add_wait_queue(&stream->waitq, &wait);
-	do {
-		set_current_state(TASK_INTERRUPTIBLE);
-
-		/* Lock Decoder */
-		if ((readl((unsigned char *)itv->reg_mem +
-				   IVTV_REG_DMASTATUS)
-		     & 0x01) && !(readl((unsigned char *)
-						itv->reg_mem +
-						IVTV_REG_DMASTATUS) & 0x14)
-		    && !test_and_set_bit(IVTV_F_S_DMAP, &stream->s_flags)) {
-			break;
-		}
-
-		if (ivtv_sleep_timeout(HZ / 100, 1)) {
-			/*IVTV_DEBUG_WARN(
-			   "DEC: User stopped waitq.\n");
-
-			   x = -EINTR;
-			   break; */
-		}
-	} while (!x);
-	set_current_state(TASK_RUNNING);
-	remove_wait_queue(&stream->waitq, &wait);
-
-	return x;
-}
-
 void ivtv_TO_DMA_done(struct ivtv *itv, int stmtype)
 {
 	struct ivtv_stream *stream = NULL;
@@ -1256,7 +850,6 @@ int dma_to_device(struct ivtv *itv, struct ivtv_stream *st,
 	int ret = 0;
 	/* wait for DMA complete status */
 	then = jiffies;
-
 	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x03)) {
 		ivtv_sleep_timeout(HZ / 100, 1);
 		if ((jiffies - then) > (HZ * 3)) {
@@ -1270,7 +863,7 @@ int dma_to_device(struct ivtv *itv, struct ivtv_stream *st,
 	then = jiffies;
 	while ((readl(itv->reg_mem + IVTV_REG_DMAXFER) & 0x03)) {
 		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ / 5)) {
+		if ((jiffies - then) > (HZ * 3)) {
 			IVTV_DEBUG_WARN(
 				   "DMA_TO: REG_DMAXFER wait failed\n");
 			break;
@@ -1301,20 +894,33 @@ int dma_to_device(struct ivtv *itv, struct ivtv_stream *st,
 
 	/* wait for DMA to start */
 	then = jiffies;
-
 	while ((readl(itv->reg_mem + IVTV_REG_DMAXFER) & 0x01)) {
 		/* DMA Error */
 		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x15) {
 			break;
 		}
 
-		if ((jiffies - then) > (HZ /5)) {
+		ivtv_sleep_timeout(HZ / 100, 1);
+		if ((jiffies - then) > (HZ * 3)) {
 			IVTV_DEBUG_WARN(
 				   "DMA_TO: REG_DMAXFER 2 wait failed\n");
 			break;
 		}
 	}
 
+	then = jiffies;
+	while (!(readl(itv->reg_mem + IVTV_REG_DECSG1LEN) & 0x80000000)) {
+		/* DMA Error */
+		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x15)
+			break;
+
+		ivtv_sleep_timeout(HZ / 100, 1);
+		if ((jiffies - then) > (HZ * 3)) {
+			IVTV_DEBUG_WARN(
+				   "DEC: REG_DECSG1LEN wait failed\n");
+			break;
+		}
+	}
 	/* Wait for Read Interrupt */
 	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x15) &&
 	       wait_event_interruptible(itv->r_intr_wq,
@@ -1329,7 +935,6 @@ int dma_to_device(struct ivtv *itv, struct ivtv_stream *st,
 	atomic_set(&itv->r_intr, 0);
 	/* wait for DMA complete status */
 	then = jiffies;
-
 	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x01)) {
 		/* DMA Error */
 		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x15) {
