@@ -48,7 +48,6 @@
 #include "ivtv-irq.h"
 #include "ivtv-streams.h"
 #include "ivtv-cards.h"
-#include "audiochip.h"
 #include <linux/smp_lock.h>
 
 #define IVTV_V4L2_MAX_MINOR 15
@@ -158,7 +157,6 @@ static int ivtv_stream_init(struct ivtv *itv, int streamtype,
 	s->bufsize = bufsize;
 	s->buf_total = 0;
 	s->buf_fill = 0;
-	s->dmatype = 0;
 	atomic_set(&s->allocated_buffers, 0);
 	INIT_LIST_HEAD(&s->free_q.list);
 	INIT_LIST_HEAD(&s->full_q.list);
@@ -198,9 +196,9 @@ static int ivtv_reg_dev(struct ivtv *itv, int streamtype, int minor, int reg_typ
 			itv->num, ivtv_stream_name(streamtype));
 
 	s->v4l2dev->minor = minor;
-#ifdef LINUX26
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	s->v4l2dev->dev = &itv->dev->dev;
-#endif /* LINUX26 */
+#endif
 	s->v4l2dev->release = video_device_release;
 
 	/* We're done if this is a 'hidden' stream (OSD) */
@@ -461,20 +459,19 @@ void ivtv_setup_v4l2_encode_stream(struct ivtv *itv, int type)
 	IVTV_ASSERT(ivtv_sem_count(&s->mlock) <= 0);
 
 	/* assign stream type */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_STREAM_TYPE, 1, itv->codec.stream_type);
+	ivtv_vapi(itv, CX2341X_ENC_SET_STREAM_TYPE, 1, itv->codec.stream_type);
 
 	/* assign output port */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_OUTPUT_PORT, 1, 0);	/* 0 = Memory */
+	ivtv_vapi(itv, CX2341X_ENC_SET_OUTPUT_PORT, 1, 0);	/* 0 = Memory */
 
 	/* assign framerate */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_FRAMERATE, 1,
-		  (itv->std & V4L2_STD_625_50) ? 1 : 0);
+	ivtv_vapi(itv, CX2341X_ENC_SET_FRAME_RATE, 1, itv->is_50hz);
 
 	/* assign frame size */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_FRAME_SIZE, 2, itv->height, itv->width);
+	ivtv_vapi(itv, CX2341X_ENC_SET_FRAME_SIZE, 2, itv->height, itv->width);
 
 	/* assign aspect ratio */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_ASPECT_RATIO, 1, itv->codec.aspect);
+	ivtv_vapi(itv, CX2341X_ENC_SET_ASPECT_RATIO, 1, itv->codec.aspect);
 
 	/* automatically calculate bitrate on the fly */
 	if (itv->codec.bitrate > 15000000) {
@@ -484,28 +481,28 @@ void ivtv_setup_v4l2_encode_stream(struct ivtv *itv, int type)
 	}
 
 	/* assign bitrates */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_BITRATES, 5, itv->codec.bitrate_mode,
+	ivtv_vapi(itv, CX2341X_ENC_SET_BIT_RATE, 5, itv->codec.bitrate_mode,
 		  itv->codec.bitrate,	/* bps */
 		  itv->codec.bitrate_peak / 400,	/* peak/400 */
 		  0x00, 0x00, 0x70);	/* encoding buffer, ckennedy */
 
 	/* assign gop properties */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_GOP_PROPERTIES, 2,
+	ivtv_vapi(itv, CX2341X_ENC_SET_GOP_PROPERTIES, 2,
 		  itv->codec.framespergop, itv->codec.bframes);
 
 	/* assign 3 2 pulldown */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_3_2_PULLDOWN, 1, itv->codec.pulldown);
+	ivtv_vapi(itv, CX2341X_ENC_SET_3_2_PULLDOWN, 1, itv->codec.pulldown);
 
 	/* assign gop closure */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_GOP_CLOSURE, 1, itv->codec.gop_closure);
+	ivtv_vapi(itv, CX2341X_ENC_SET_GOP_CLOSURE, 1, itv->codec.gop_closure);
 
 	/* assign audio properties */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_AUDIO_PROPERTIES, 1,
+	ivtv_vapi(itv, CX2341X_ENC_SET_AUDIO_PROPERTIES, 1,
 		  itv->codec.audio_bitmask);
 	ivtv_dualwatch_start_encoding(itv);
 
 	/* assign dnr filter mode */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_DNR_FILTER_MODE, 2,
+	ivtv_vapi(itv, CX2341X_ENC_SET_DNR_FILTER_MODE, 2,
 		  itv->codec.dnr_mode, itv->codec.dnr_type);
 
 	/* assign dnr filter props */
@@ -517,17 +514,17 @@ void ivtv_setup_v4l2_encode_stream(struct ivtv *itv, int type)
                 // Always set to 0 for scaled images.
                 dnr_temporal = 0;
         }
-	ivtv_vapi(itv, IVTV_API_ASSIGN_DNR_FILTER_PROPS, 2,
+	ivtv_vapi(itv, CX2341X_ENC_SET_DNR_FILTER_PROPS, 2,
 		  itv->codec.dnr_spatial, dnr_temporal);
 
 	/* assign coring levels (luma_h, luma_l, chroma_h, chroma_l) */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_CORING_LEVELS, 4, 0, 255, 0, 255);
+	ivtv_vapi(itv, CX2341X_ENC_SET_CORING_LEVELS, 4, 0, 255, 0, 255);
 
 	/* assign spatial filter type: luma_t: 1 = horiz_only, chroma_t: 1 = horiz_only */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_SPATIAL_FILTER_TYPE, 2, 1, 1);
+	ivtv_vapi(itv, CX2341X_ENC_SET_SPATIAL_FILTER_TYPE, 2, 1, 1);
 
 	/* assign frame drop rate */
-	ivtv_vapi(itv, IVTV_API_ASSIGN_FRAME_DROP_RATE, 1, 0);
+	ivtv_vapi(itv, CX2341X_ENC_SET_FRAME_DROP_RATE, 1, 0);
 }
 
 int ivtv_passthrough_mode(struct ivtv *itv, int enable)
@@ -566,7 +563,7 @@ int ivtv_passthrough_mode(struct ivtv *itv, int enable)
 		ivtv_setup_v4l2_decode_stream(itv, IVTV_DEC_STREAM_TYPE_YUV);
 
 		/* Start Decoder */
-		ivtv_vapi(itv, IVTV_API_DEC_START_PLAYBACK, 2, 0, 1);
+		ivtv_vapi(itv, CX2341X_DEC_START_PLAYBACK, 2, 0, 1);
 		atomic_inc(&itv->decoding);
 		ivtv_start_vbi_timer(itv);
 
@@ -580,11 +577,11 @@ int ivtv_passthrough_mode(struct ivtv *itv, int enable)
 
                 /* Setup the sliced VBI registers */
 		fmt.type = V4L2_BUF_TYPE_SLICED_VBI_CAPTURE;
-                vbi_setup_lcr(itv, itv->vbi_passthrough, !(itv->std & V4L2_STD_NTSC), &fmt.fmt.sliced);
-                itv->card->video_dec_func(itv, VIDIOC_S_FMT, &fmt);
+                vbi_setup_lcr(itv, itv->vbi_passthrough, itv->is_50hz, &fmt.fmt.sliced);
+                itv->video_dec_func(itv, VIDIOC_S_FMT, &fmt);
 
                 /* Start Passthrough Mode */
-		x = ivtv_vapi(itv, IVTV_API_BEGIN_CAPTURE, 2, 2, 11);
+		x = ivtv_vapi(itv, CX2341X_ENC_START_CAPTURE, 2, 2, 11);
 
 		if (x)
 			IVTV_DEBUG_WARN(
@@ -601,8 +598,8 @@ int ivtv_passthrough_mode(struct ivtv *itv, int enable)
 
 	/* Stop Passthrough Mode */
 	down(&dec_stream->mlock);
-	x = ivtv_vapi(itv, IVTV_API_END_CAPTURE, 3, 1, 1, 11);
-	x = ivtv_vapi(itv, IVTV_API_DEC_STOP_PLAYBACK, 3, 1, 0, 0);
+	x = ivtv_vapi(itv, CX2341X_ENC_STOP_CAPTURE, 3, 1, 1, 11);
+	x = ivtv_vapi(itv, CX2341X_DEC_STOP_PLAYBACK, 3, 1, 0, 0);
 	if (x)
 		IVTV_DEBUG_WARN(
 			   "passthrough stop decode error. Code %d\n", x);
@@ -617,14 +614,14 @@ int ivtv_passthrough_mode(struct ivtv *itv, int enable)
 	return 0;
 }
 
-void ivtv_vbi_setup(struct ivtv *itv, int mode)
+static void ivtv_vbi_setup(struct ivtv *itv, int mode)
 {
 	int raw = itv->vbi_sliced_in->service_set == 0;
 	u32 data[IVTV_MBOX_MAX_DATA], result;
 	int lines;
 	int x;
 	int i;
-	int h = (itv->std & V4L2_STD_NTSC) ? 480 : 576;
+	int h = itv->is_60hz ? 480 : 576;
 
 	/* If Embed then streamtype must be Program */
 	if (!(mode & 0x04) && (mode & 0x02) && 
@@ -633,20 +630,20 @@ void ivtv_vbi_setup(struct ivtv *itv, int mode)
 		itv->codec.stream_type = 0;
 
         	/* assign stream type */
-        	ivtv_vapi(itv, IVTV_API_ASSIGN_STREAM_TYPE, 1, 
+        	ivtv_vapi(itv, CX2341X_ENC_SET_STREAM_TYPE, 1, 
 			itv->codec.stream_type);
 	}
 
 	/* Reset VBI */
-	ivtv_vapi(itv, IVTV_API_SELECT_VBI_LINE, 5, 0xffff , 0, 0, 0, 0);
+	ivtv_vapi(itv, CX2341X_ENC_SET_VBI_LINE, 5, 0xffff , 0, 0, 0, 0);
 
 	/* Don't use VBI if scaling */
 	if (raw && itv->height != h) {
-		itv->card->video_dec_func(itv, VIDIOC_S_FMT, &itv->vbi_in);
+		itv->video_dec_func(itv, VIDIOC_S_FMT, &itv->vbi_in);
 		return;
 	}
 
-        if (itv->std & V4L2_STD_NTSC) {
+        if (itv->is_60hz) {
                 itv->vbi_count = 12;
                 itv->vbi_start[0] = 10;
                 itv->vbi_start[1] = 273;
@@ -657,7 +654,7 @@ void ivtv_vbi_setup(struct ivtv *itv, int mode)
         }
 
 	// setup VBI registers
-	itv->card->video_dec_func(itv, VIDIOC_S_FMT, &itv->vbi_in);
+	itv->video_dec_func(itv, VIDIOC_S_FMT, &itv->vbi_in);
 
 	itv->vbi_search_ba = 0;
 
@@ -674,7 +671,7 @@ void ivtv_vbi_setup(struct ivtv *itv, int mode)
 			lines += 2;
 	}
 
-        ivtv_vapi(itv, IVTV_API_ASSIGN_NUM_VSYNC_LINES,
+        ivtv_vapi(itv, CX2341X_ENC_SET_NUM_VSYNC_LINES,
         	2, itv->digitizer, itv->digitizer);
 
 	itv->vbi_enc_size =
@@ -698,9 +695,7 @@ void ivtv_vbi_setup(struct ivtv *itv, int mode)
 	// start/stop codes was deduced from this, but they do not appear in the driver.
 	// Other code pairs that I found are: 0x250E6249/0x13545454 and 0x25256262/0x38137F54.
 	// However, I have no idea what these values are for.
-        if (itv->card->type == IVTV_CARD_PVR_150 ||
-               itv->card->type == IVTV_CARD_PG600)
-	{
+        if (itv->hw_flags & IVTV_HW_CX25840) {
         	/* Setup VBI for the cx25840 digitizer */
         	if (raw) {
 			data[3] = 0x20602060;
@@ -732,7 +727,7 @@ void ivtv_vbi_setup(struct ivtv *itv, int mode)
 		"Setup VBI API header 0x%08x pkts %d buffs %d ln %d sz %d\n", 
 			data[0], data[1], data[2], data[5], data[6]);
 
-	x = ivtv_api(itv, itv->enc_mbox, &itv->enc_msem, IVTV_API_CONFIG_VBI,
+	x = ivtv_api(itv, itv->enc_mbox, &itv->enc_msem, CX2341X_ENC_SET_VBI_CONFIG,
 		     &result, 7, &data[0]);
 	if (x)
 		IVTV_DEBUG_WARN("init error 21. Code %d\n", x);
@@ -753,14 +748,14 @@ void ivtv_vbi_setup(struct ivtv *itv, int mode)
 	for (i = 2; i <= 24; i++) {
 		int valid;
 
-                if (itv->std & V4L2_STD_NTSC) {
+                if (itv->is_60hz) {
 		        valid = i >= 10 && i < 22;
                 } else {
 		        valid = i >= 6 && i < 24;
                 }
-		ivtv_vapi(itv, IVTV_API_SELECT_VBI_LINE, 5, i - 1,
+		ivtv_vapi(itv, CX2341X_ENC_SET_VBI_LINE, 5, i - 1,
                                 valid, 0 , 0, 0);
-		ivtv_vapi(itv, IVTV_API_SELECT_VBI_LINE, 5, (i - 1) | 0x80000000,
+		ivtv_vapi(itv, CX2341X_ENC_SET_VBI_LINE, 5, (i - 1) | 0x80000000,
                                 valid, 0, 0, 0);
 	}
 
@@ -774,11 +769,11 @@ void ivtv_vbi_setup(struct ivtv *itv, int mode)
 
 static void unknown_setup_api(struct ivtv *itv)
 {
-	ivtv_vapi(itv, IVTV_API_ENC_UNKNOWN, 1, 0);
-	ivtv_vapi(itv, IVTV_API_ENC_MISC, 2, 7, 0);
-	ivtv_vapi(itv, IVTV_API_ENC_MISC, 2, 8, 0);
-	ivtv_vapi(itv, IVTV_API_ENC_MISC, 2, 4, 1);
-	ivtv_vapi(itv, IVTV_API_ENC_MISC, 2, 12, 0);
+	ivtv_vapi(itv, CX2341X_ENC_UNKNOWN, 1, 0);
+	ivtv_vapi(itv, CX2341X_ENC_MISC, 2, 7, 0);
+	ivtv_vapi(itv, CX2341X_ENC_MISC, 2, 8, 0);
+	ivtv_vapi(itv, CX2341X_ENC_MISC, 2, 4, 1);
+	ivtv_vapi(itv, CX2341X_ENC_MISC, 2, 12, 0);
 }
 
 int ivtv_init_digitizer(struct ivtv *itv)
@@ -787,17 +782,17 @@ int ivtv_init_digitizer(struct ivtv *itv)
 
         /* Disable digitizer */
         IVTV_DEBUG_INFO("Disabling digitizer\n");
-	itv->card->video_dec_func(itv, VIDIOC_STREAMOFF, &dummy);
+	itv->video_dec_func(itv, VIDIOC_STREAMOFF, &dummy);
 
         /* initialize or refresh input */
         if (atomic_read(&itv->capturing) == 0)
-                ivtv_vapi(itv, IVTV_API_INITIALIZE_INPUT, 0);
+                ivtv_vapi(itv, CX2341X_ENC_INITIALIZE_INPUT, 0);
         else
-                ivtv_vapi(itv, IVTV_API_REFRESH_INPUT, 0);
+                ivtv_vapi(itv, CX2341X_ENC_REFRESH_INPUT, 0);
 
         /* enable digitizer (saa7115) */
         IVTV_DEBUG_INFO("Enabling digitizer\n");
-	itv->card->video_dec_func(itv, VIDIOC_STREAMON, &dummy);
+	itv->video_dec_func(itv, VIDIOC_STREAMON, &dummy);
 
 	return 0;
 }
@@ -888,7 +883,7 @@ int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type)
 	st->stolen_bufs = 0;
 
 	/* mute/unmute video */
-	ivtv_vapi(itv, IVTV_API_MUTE_VIDEO, 1,
+	ivtv_vapi(itv, CX2341X_ENC_MUTE_VIDEO, 1,
 		  test_bit(IVTV_F_I_RADIO_USER, &itv->i_flags) ? 1 : 0);
 
 	/* Clear Streamoff flags in case left from last capture */
@@ -903,13 +898,13 @@ int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type)
 		/* 131072, 262144, 524288. 0 defaults to 131072 */
 		data[1] = itv->dma_cfg.fw_enc_dma_type;	/* 0=bytes, 1=frames */
 		ivtv_api(itv, itv->enc_mbox, &itv->enc_msem,
-	     		IVTV_API_ASSIGN_DMA_BLOCKLEN, &result, 2, &data[0]);
+	     		CX2341X_ENC_SET_DMA_BLOCK_SIZE, &result, 2, &data[0]);
 
 		/* Stuff from Windows, we don't know what it is */
 		unknown_setup_api(itv);
 
        		/* assign placeholder */
-       		ivtv_vapi(itv, IVTV_API_ASSIGN_PLACEHOLDER, 12, 
+       		ivtv_vapi(itv, CX2341X_ENC_SET_PLACEHOLDER, 12, 
 			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
 
 		/* Setup VBI */
@@ -932,7 +927,7 @@ int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type)
                         	itv->dec_vbi_pkt = 0;
                 	}
         	} else {
-			ivtv_vapi(itv, IVTV_API_ASSIGN_NUM_VSYNC_LINES,
+			ivtv_vapi(itv, CX2341X_ENC_SET_NUM_VSYNC_LINES,
 	  			2, itv->digitizer, itv->digitizer);
 		}
 
@@ -940,7 +935,7 @@ int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type)
 		data[0] = itv->idx_sdf_mask;
 		data[1] = itv->idx_sdf_num;
 		ivtv_api(itv, itv->enc_mbox, &itv->enc_msem,
-	 		IVTV_API_ASSIGN_PGM_INDEX_INFO, &result, 2, &data[0]);
+	 		CX2341X_ENC_SET_PGM_INDEX_INFO, &result, 2, &data[0]);
 		itv->idx_sdf_offset = data[0];
 		itv->idx_sdf_num = data[1];
 
@@ -952,13 +947,15 @@ int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type)
 		ivtv_setup_v4l2_encode_stream(itv, type);
 	}
 
-	if (atomic_read(&itv->capturing) == 0)
-		ivtv_vapi(itv, IVTV_API_ENC_MISC, 2, 3, itv->has_cx25840);
+	if (atomic_read(&itv->capturing) == 0) {
+		ivtv_vapi(itv, CX2341X_ENC_MISC, 2, 3, 
+				(itv->hw_flags & IVTV_HW_CX25840) ? 1 : 0);
+	}
 
 	/* Vsync Setup */
 	if (itv->has_cx23415 && !test_and_set_bit(IVTV_F_I_DIG_RST, &itv->i_flags)) {
 		/* event notification (on) */
-		ivtv_vapi(itv, IVTV_API_EVENT_NOTIFICATION,
+		ivtv_vapi(itv, CX2341X_ENC_SET_EVENT_NOTIFICATION,
 	  		4, 0, 1, 0x10000000, -1);
 		ivtv_clear_irq_mask(itv, IVTV_IRQ_ENC_VIM_RST);
 	}
@@ -984,7 +981,7 @@ int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type)
         }
 
 	/* begin_capture */
-	if (ivtv_vapi(itv, IVTV_API_BEGIN_CAPTURE, 2, captype, subtype)) 
+	if (ivtv_vapi(itv, CX2341X_ENC_START_CAPTURE, 2, captype, subtype)) 
 	{
 		IVTV_DEBUG_WARN( "Error starting capture!\n");
 		return -EINVAL;
@@ -1028,21 +1025,21 @@ int ivtv_setup_v4l2_decode_stream(struct ivtv *itv, int type)
 	ivtv_disable_vbi(itv);
 
 	/* set audio mode to stereo */
-	ivtv_vapi(itv, IVTV_API_DEC_SELECT_AUDIO, 2, 0, 0);
+	ivtv_vapi(itv, CX2341X_DEC_SET_AUDIO_MODE, 2, 0, 0);
 
 	/* set number of internal decoder buffers */
 	ivtv_vapi(itv,
-		  IVTV_API_DEC_DISPLAY_BUFFERS, 1, itv->dec_options.decbuffers);
+		  CX2341X_DEC_SET_DISPLAY_BUFFERS, 1, itv->dec_options.decbuffers);
 
 	/* prebuffering */
 	data[0] = itv->dec_options.prebuffer;	/* 0 = no prebuffering,
 						   1 = enabled, see docs */
-	ivtv_vapi(itv, IVTV_API_DEC_BUFFER, 1, itv->dec_options.prebuffer);
+	ivtv_vapi(itv, CX2341X_DEC_SET_PREBUFFERING, 1, itv->dec_options.prebuffer);
 
 	/* extract from user packets */
 	data[0] = itv->dec_vbi_pkt;
         ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
-                     IVTV_API_DEC_EXTRACT_VBI, &result, 2, &data[0]);
+                     CX2341X_DEC_EXTRACT_VBI, &result, 2, &data[0]);
 	itv->vbi_dec_start = data[0];
 	itv->vbi_dec_size = data[1];
 
@@ -1052,7 +1049,7 @@ int ivtv_setup_v4l2_decode_stream(struct ivtv *itv, int type)
 		itv->dec_vbi_pkt);
 
         /* event notification (on)*/
-        ivtv_vapi(itv, IVTV_API_DEC_EVENT_NOTIFICATION, 4,
+        ivtv_vapi(itv, CX2341X_DEC_SET_EVENT_NOTIFICATION, 4,
                 5, 1, 0x04000000, -1);
 
 	/* set decoder source settings */
@@ -1080,7 +1077,7 @@ int ivtv_setup_v4l2_decode_stream(struct ivtv *itv, int type)
 							   bitmap. see docs. */
 	x = ivtv_api(itv, itv->dec_mbox,
 		     &itv->dec_msem,
-		     IVTV_API_DEC_DECODE_SOURCE, &result, 4, &data[0]);
+		     CX2341X_DEC_SET_DECODER_SOURCE, &result, 4, &data[0]);
 	if (x)
 		IVTV_DEBUG_WARN(
 			   "COULDN'T INITIALIZE DECODER SOURCE %d\n", x);
@@ -1133,7 +1130,7 @@ int ivtv_start_v4l2_decode_stream(struct ivtv *itv, int type)
 	/* set dma size */
 	if (itv->dma_cfg.fw_dec_dma_xfer > 0) {
 		x = ivtv_vapi(itv,
-			      IVTV_API_DEC_DMA_BLOCKSIZE, 1,
+			      CX2341X_DEC_SET_DMA_BLOCK_SIZE, 1,
 			      itv->dma_cfg.fw_dec_dma_xfer);
 		if (x)
 			IVTV_DEBUG_WARN(
@@ -1177,7 +1174,7 @@ int ivtv_start_v4l2_decode_stream(struct ivtv *itv, int type)
 	}
         set_bit(IVTV_F_T_DEC_STARTED, &itv->t_flags);
 	/* start playback */
-	if (ivtv_vapi(itv, IVTV_API_DEC_START_PLAYBACK, 2,
+	if (ivtv_vapi(itv, CX2341X_DEC_START_PLAYBACK, 2,
 			     itv->dec_options.gop_offset,
 			     itv->dec_options.mute_frames))
 	{
@@ -1290,7 +1287,7 @@ int ivtv_stop_capture(struct ivtv *itv, int type)
 	}
 
 	/* end_capture */
-	x = ivtv_vapi(itv, IVTV_API_END_CAPTURE, 3, stopmode,	
+	x = ivtv_vapi(itv, CX2341X_ENC_STOP_CAPTURE, 3, stopmode,	
 		/* when: 0 =  end of GOP  1 = NOW! */
 		      cap_type,	/* type: 0 = mpeg */
 		      st->subtype);	/* subtype: 3 = video+audio */
@@ -1303,7 +1300,7 @@ int ivtv_stop_capture(struct ivtv *itv, int type)
 		/* event notification (off) */
 		if (test_and_clear_bit(IVTV_F_I_DIG_RST, &itv->i_flags)) {
 			/* type: 0 = refresh */
-			x = ivtv_vapi(itv, IVTV_API_EVENT_NOTIFICATION, 4, 0, 0,	/* on/off: 0 = off */
+			x = ivtv_vapi(itv, CX2341X_ENC_SET_EVENT_NOTIFICATION, 4, 0, 0,	/* on/off: 0 = off */
 				      0x10000000,	/* intr: 0x10000000 */
 				      -1);	/* mbox_id: -1: none */
 			if (x)
@@ -1312,7 +1309,7 @@ int ivtv_stop_capture(struct ivtv *itv, int type)
 					   "event notification %d\n", x);
 			ivtv_set_irq_mask(itv, IVTV_IRQ_ENC_VIM_RST);
 		}
-                ivtv_vapi(itv, IVTV_API_ENC_MISC, 2, 3, 0);
+                ivtv_vapi(itv, CX2341X_ENC_MISC, 2, 3, 0);
 
 	}
 
@@ -1491,7 +1488,7 @@ int ivtv_stop_v4l2_decode_stream(struct ivtv *itv, int type)
 		maxcount = 0;
 
         /* event notification (off)*/
-	ivtv_vapi(itv, IVTV_API_DEC_EVENT_NOTIFICATION, 4, 
+	ivtv_vapi(itv, CX2341X_DEC_SET_EVENT_NOTIFICATION, 4, 
 		5, 0, 0x04000000, -1);
 
         /* Clear all Buffers */
@@ -1515,7 +1512,7 @@ int ivtv_stop_v4l2_decode_stream(struct ivtv *itv, int type)
 		}
         }
 	/* Stop Decoder */
-       	if (ivtv_vapi(itv, IVTV_API_DEC_STOP_PLAYBACK, 4,
+       	if (ivtv_vapi(itv, CX2341X_DEC_STOP_PLAYBACK, 4,
         	/*  0 = last frame, 1 = black */
                	hide_last_frame,
                	itv->dec_options.pts_low,
