@@ -116,9 +116,9 @@ static int ivtvyuv_prep_user_dma(struct ivtv *itv, struct ivtv_user_dma *dma,
 		}
 		dma->SGlist[i].offset = offset;
 		dma->SGlist[i].page = dma->map[i];
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 		dma->SGlist[i].address = 0;
-#endif /* LINUX26 */
+#endif
 		offset = 0;
 	}
 
@@ -135,9 +135,9 @@ static int ivtvyuv_prep_user_dma(struct ivtv *itv, struct ivtv_user_dma *dma,
 		}
 		dma->SGlist[i].offset = offset;
 		dma->SGlist[i].page = dma->map[i];
-#ifndef LINUX26
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 		dma->SGlist[i].address = 0;
-#endif /* LINUX26 */
+#endif
 		offset = 0;
 	}
         y_page_count = dma->page_count;
