@@ -39,7 +39,7 @@
 #include <linux/types.h>
 
 #define __user
-#include "videodev2.h"
+#include <linux/videodev2.h>
 
 #include "ivtv.h"
 
@@ -61,7 +61,7 @@ int chann(int fd, int chan_num);
 int chanf(int fd, int frequency);
 void usage(void);
 
-int splice(unsigned char *, int, int);
+int enc_splice(unsigned char *, int, int);
 #define SPLICE_START 1
 #define SPLICE_END   2
 #define GOP_COUNTER  3
@@ -853,7 +853,7 @@ void stop_encoder(int signal)
 }
 
 /* Code borrowed from MythTV */
-int splice(unsigned char *buf, int len, int mode)
+int enc_splice(unsigned char *buf, int len, int mode)
 {
 	unsigned char *bufptr = buf;
 	unsigned int state = 0xFFFFFFFF, v = 0;
