@@ -37,7 +37,6 @@
 #include <linux/types.h>
 #include <sys/klog.h>
 
-#define __user
 #include "videodev2.h"
 
 #define IVTV_INTERNAL
@@ -48,6 +47,11 @@
 
 #include "ivtv-version.h"
 
+/* GPIO */
+#define IVTV_REG_GPIO_IN_OFFSET    (0x9008 + 0x02000000)
+#define IVTV_REG_GPIO_OUT_OFFSET   (0x900c + 0x02000000)
+#define IVTV_REG_GPIO_DIR_OFFSET   (0x9020 + 0x02000000)
+
 /* Options */
 enum Option {
 	OptListAudioInputs = 'A',
@@ -67,7 +71,7 @@ enum Option {
 	OptReloadFW = 'H',
 	OptHelp = 'h',
 	OptSetGPIO = 'i',
-	OptInputDetect = 'I',
+	OptListGPIO = 'I',
 	OptListEncRegisters = 'J',
 	OptSetEncRegister = 'j',
 	OptPassThrough = 'K',
@@ -90,7 +94,7 @@ enum Option {
 	OptGetStandard = 'U',
 	OptSetStandard = 'u',
 	OptGetDriverInfo = 'V',
-	OptSetIO = 'v',
+	OptSetAudioRoute = 'v',
 	OptGetVbiPassThrough = 'W',
 	OptSetVbiPassThrough = 'w',
 	OptGetVbiEmbed = 'X',
@@ -156,7 +160,7 @@ static struct option long_options[] = {
 	{"reload", no_argument, 0, OptReloadFW},
 	{"help", no_argument, 0, OptHelp},
 	{"set-gpio", required_argument, 0, OptSetGPIO},
-	{"input-detect", no_argument, 0, OptInputDetect},
+	{"list-gpio", no_argument, 0, OptListGPIO},
 	{"list-enc-registers", no_argument, 0, OptListEncRegisters},
 	{"set-enc-registers", required_argument, 0, OptSetEncRegister},
 	{"passthrough", required_argument, 0, OptPassThrough},
@@ -177,7 +181,7 @@ static struct option long_options[] = {
 	{"get-standard", no_argument, 0, OptGetStandard},
 	{"set-standard", required_argument, 0, OptSetStandard},
 	{"info", no_argument, 0, OptGetDriverInfo},
-	{"set-io", required_argument, 0, OptSetIO},
+	{"audio-route", required_argument, 0, OptSetAudioRoute},
 	{"get-vbi-passthrough", no_argument, 0, OptGetVbiPassThrough},
 	{"set-vbi-passthrough", required_argument, 0, OptSetVbiPassThrough},
 	{"get-vbi-embed", no_argument, 0, OptGetVbiEmbed},
@@ -283,10 +287,14 @@ void usage(void)
 	printf("       contrast      =<#> Picture contrast or luma gain. [0 - 127]\n");
 	printf("       volume        =<#> Overall audio volume. [0 - 65535]\n");
 	printf("       mute          =<#> Mute audio, i.e. set the volume to zero [boolean]\n");
+	printf("       balance       =<#> Audio balance. [0 - 65535]\n");
+	printf("       bass          =<#> Audio bass. [0 - 65535]\n");
+	printf("       treble        =<#> Audio treble. [0 - 65535]\n");
+	printf("       loudness      =<#> Audio loudness. [0 - 1]\n");
 	printf("  -Z, --get-sapmode  query the current Secondary Audio Program [VIDIOC_G_TUNER]\n");
 	printf("  -z, --set-sapmode=<mode>\n");
 	printf("                     set the current Secondary Audio Program to <mode> [VIDIOC_S_TUNER]\n");
-	printf("                     Possible values: 0 (mono), 1 (stereo), 2 (lang2), 3 (lang1)\n");
+	printf("                     Possible values: 0 (mono), 1 (stereo), 2 (lang2), 3 (lang1), 4 (both)\n");
 	printf("  --get-yuv-mode     display the current yuv mode\n");
 	printf("  --set-yuv-mode=[param],[param]\n");
         printf("                     set the current yuv mode\n");
@@ -307,10 +315,10 @@ void usage(void)
 	printf("  -g, --set-dec-register=reg=<reg>,val=<val>\n");
 	printf("                     set decoder registers [DECODER_SET_REG]\n");
 	printf("  -H, --reload       forces reload on cards firmware [IVTV_IOC_RELOAD_FW]\n");
-	printf("  -I, --input-detect\n");
-	printf("                     detect input video signal\n");
-	printf("  -i, --set-gpio=<mode>\n");
-	printf("                     set GPIO output to <mode> for debugging [0x0000-0xffff]\n");
+	printf("  -I, --list-gpio\n");
+	printf("                     show GPIO input/direction/output bits\n");
+	printf("  -i, --set-gpio=[dir=<dir>,]val=<val>\n");
+	printf("                     set GPIO direction bits to <dir> and set output to <val>\n");
 	printf("  -J, --list-enc-registers\n");
 	printf("                     dump encoder registers [ENCODER_GET_REG]\n");
 	printf("  -j, --set-enc-register=reg=<reg>,val=<val>\n");
@@ -322,8 +330,8 @@ void usage(void)
 	printf("                     set ITVC registers [ITVC_SET_REG]\n");
 	printf("  -S, --streamoff    send StreamOff API [VIDIOC_STREAMOFF]\n");
 	printf("  -V, --info         gets driver information (version) [IVTV_IOC_G_DRIVER_INFO]\n");
-	printf("  -v, --set-io=input=<in>,output=<out>\n");
-	printf("                     set the MSP34xx input/output mapping [MSP_SET_MATRIX]\n");
+	printf("  -v, --audio-route=input=<in>,output=<out>\n");
+	printf("                     set the audio input/output routing [IVTV_IOC_S_AUDIO_ROUTING]\n");
 	printf("  --log-status       log the board status in the kernel log\n");
 	exit(0);
 }
@@ -474,27 +482,27 @@ int main(int argc, char **argv)
 		"audio",
 #define SUB_BFRAMES			2
 		"bframes",
-#define SUB_BITRATE_MODE	3
+#define SUB_BITRATE_MODE		3
 		"bitrate_mode",
 #define SUB_BITRATE			4
 		"bitrate",
-#define SUB_BITRATE_PEAK	5
+#define SUB_BITRATE_PEAK		5
 		"bitrate_peak",
-#define SUB_DNR_MODE		6
+#define SUB_DNR_MODE			6
 		"dnr_mode",
-#define SUB_DNR_SPATIAL		7
+#define SUB_DNR_SPATIAL			7
 		"dnr_spatial",
-#define SUB_DNR_TEMPORAL	8
+#define SUB_DNR_TEMPORAL		8
 		"dnr_temporal",
-#define SUB_DNR_TYPE		9
+#define SUB_DNR_TYPE			9
 		"dnr_type",
-#define SUB_FRAMERATE		10
+#define SUB_FRAMERATE			10
 		"framerate",
-#define SUB_FRAMESPERGOP	11
+#define SUB_FRAMESPERGOP		11
 		"framespergop",
 #define SUB_FREQ			12
 		"freq",
-#define SUB_GOP_CLOSURE		13
+#define SUB_GOP_CLOSURE			13
 		"gop_closure",
 #define SUB_HEIGHT			14
 		"height",
@@ -502,9 +510,9 @@ int main(int argc, char **argv)
 		"input",
 #define SUB_OUTPUT			16
 		"output",
-#define SUB_PULLDOWN		17
+#define SUB_PULLDOWN			17
 		"pulldown",
-#define SUB_STEAM_TYPE		18
+#define SUB_STREAM_TYPE			18
 		"stream_type",
 #define SUB_TUNER			19
 		"tuner",
@@ -514,11 +522,11 @@ int main(int argc, char **argv)
 		"reg",
 #define SUB_VAL				22
 		"val",
-#define SUB_BRIGHTNESS		23
+#define SUB_BRIGHTNESS			23
 		"brightness",
-#define SUB_CONTRAST		24
+#define SUB_CONTRAST			24
 		"contrast",
-#define SUB_SATURATION		25
+#define SUB_SATURATION			25
 		"saturation",
 #define SUB_HUE				26
 		"hue",
@@ -526,16 +534,26 @@ int main(int argc, char **argv)
 		"volume",
 #define SUB_MUTE			28
 		"mute",
-#define SUB_AUDIO_MODE		29
+#define SUB_AUDIO_MODE			29
 		"audio_mode",
-#define SUB_MAX		30
+#define SUB_MAX				30
 		"max",
-#define SUB_MIN		31
+#define SUB_MIN				31
 		"min",
-#define SUB_YUV_MODE		32
+#define SUB_YUV_MODE			32
 		"mode",
 #define SUB_YUV_THRESHOLD		33
 		"threshold",
+#define SUB_BALANCE			34
+		"balance",
+#define SUB_BASS			35
+		"bass",
+#define SUB_TREBLE			36
+		"treble",
+#define SUB_LOUDNESS			37
+		"loudness",
+#define SUB_DIR				38
+		"dir",
 		NULL
 	};
 
@@ -570,9 +588,10 @@ int main(int argc, char **argv)
 	v4l2_std_id std;	/* get_std/set_std */
 	struct v4l2_frequency vf;	/* get_freq/set_freq */
 	struct v4l2_standard vs;	/* list_std */
-	struct ivtv_msp_matrix mspm;	/* set_io */
-	struct v4l2_tuner vt;	/* set_audio_mode */
-	__u32 gpio_out = 0x0;	/* GPIO output data */
+	struct v4l2_routing route;	/* audio_route */
+	unsigned short gpio_out = 0x0;	/* GPIO output data */
+	unsigned short gpio_dir = 0x0;	/* GPIO direction bits */
+	int gpio_set_dir = 0;
 	int passthrough = 0;
 	int vbi_passthrough = 0;
 	int debug_level = 0;
@@ -753,7 +772,7 @@ int main(int argc, char **argv)
 					    strtol(value, 0L, 0);
 					set_codecs |= CBitratePeak;
 					break;
-				case SUB_STEAM_TYPE:
+				case SUB_STREAM_TYPE:
 					if (value == NULL) {
 						fprintf(stderr,
 							"No value given to suboption <stream_type>\n");
@@ -1069,7 +1088,7 @@ int main(int argc, char **argv)
 		case OptGetDriverInfo:{
 			break;
 		}
-		case OptSetIO:
+		case OptSetAudioRoute:
 			subs = optarg;
 			while (*subs != '\0') {
 				switch (getsubopt(&subs, subopts, &value)) {
@@ -1079,7 +1098,7 @@ int main(int argc, char **argv)
 						    ("No value given to suboption <input>\n");
 						usage();
 					}
-					mspm.input = strtol(value, 0L, 0);
+					route.input = strtol(value, 0L, 0);
 					break;
 				case SUB_OUTPUT:
 					if (value == NULL) {
@@ -1087,7 +1106,7 @@ int main(int argc, char **argv)
 						    ("No value given to suboption <output>\n");
 						usage();
 					}
-					mspm.output = strtol(value, 0L, 0);
+					route.output = strtol(value, 0L, 0);
 					break;
 				default:
 					printf
@@ -1124,6 +1143,22 @@ int main(int argc, char **argv)
 					ctrl.id = V4L2_CID_AUDIO_VOLUME;
 					ctrl.value = strtol(value, 0L, 0);
 					break;
+				case SUB_BALANCE:
+					ctrl.id = V4L2_CID_AUDIO_BALANCE;
+					ctrl.value = strtol(value, 0L, 0);
+					break;
+				case SUB_BASS:
+					ctrl.id = V4L2_CID_AUDIO_BASS;
+					ctrl.value = strtol(value, 0L, 0);
+					break;
+				case SUB_TREBLE:
+					ctrl.id = V4L2_CID_AUDIO_TREBLE;
+					ctrl.value = strtol(value, 0L, 0);
+					break;
+				case SUB_LOUDNESS:
+					ctrl.id = V4L2_CID_AUDIO_LOUDNESS;
+					ctrl.value = strtol(value, 0L, 0);
+					break;
 				case SUB_MUTE:
 					ctrl.id = V4L2_CID_AUDIO_MUTE;
 					ctrl.value = strtol(value, 0L, 0);
@@ -1138,9 +1173,37 @@ int main(int argc, char **argv)
 			}
 			break;
 		case OptSetGPIO:
-			gpio_out = strtol(optarg, 0L, 0);
-			break;
-		case OptInputDetect:
+			subs = optarg;
+			while (*subs != '\0') {
+				switch (getsubopt(&subs, subopts, &value)) {
+				case SUB_DIR:
+					if (value == NULL) {
+						fprintf(stderr,
+							"No value given to suboption <dir>\n");
+						usage();
+						exit(1);
+					}
+					gpio_dir = strtol(value, 0L, 0);
+					gpio_set_dir = 1;
+					break;
+				case SUB_VAL:
+					if (value == NULL) {
+						fprintf(stderr,
+							"No value given to suboption <val>\n");
+						usage();
+						exit(1);
+					}
+					gpio_out =
+					    (unsigned short)strtol(value, 0L, 0);
+					break;
+				default:
+					fprintf(stderr,
+						"Invalid suboptions specified\n");
+					usage();
+					exit(1);
+					break;
+				}
+			}
 			break;
 		case OptSetSAPMode:
 			mode = strtol(optarg, 0L, 0);
@@ -1175,8 +1238,7 @@ int main(int argc, char **argv)
 
 	info.size = sizeof(struct ivtv_driver_info);
 	if (ioctl(fd, IVTV_IOC_G_DRIVER_INFO, &info) != 0) {
-                fprintf(stderr, "not an ivtv driver device\n");
-                exit(1);
+                fprintf(stderr, "not an ivtv driver device!\n");
         }
 
 	/* Setting Opts */
@@ -1306,7 +1368,7 @@ int main(int argc, char **argv)
 	}
 
 	if (options[OptSetStandard]) {
-                if (std < 3) {
+                if (std < 16) {
                         vs.index = std;
                         if (ioctl(fd, VIDIOC_ENUMSTD, &vs) >= 0) {
                                 std = vs.id;
@@ -1327,12 +1389,8 @@ int main(int argc, char **argv)
                 printf("   hw_flags = %x\n", info.hw_flags);
 	}
 
-	if (options[OptSetIO]) {
-		if ((mspm.input < 9) & (mspm.output < 3)) {
-			doioctl(fd, IVTV_IOC_S_MSP_MATRIX, &mspm, "IVTV_IOC_S_MSP_MATRIX");
-		} else
-			fprintf(stderr, "Invalid input/output setting %d/%d\n",
-				mspm.input, mspm.output);
+	if (options[OptSetAudioRoute]) {
+		doioctl(fd, IVTV_IOC_S_AUDIO_ROUTING, &route, "IVTV_IOC_S_AUDIO_ROUTING");
 	}
 
 	if (options[OptSetCtrl]) {
@@ -1640,103 +1698,32 @@ int main(int argc, char **argv)
 	}
 
 	if (options[OptSetGPIO]) {
-		vt.index = 0;
-		vt.audmode = 0x10000 | gpio_out;
-		if (doioctl(fd, VIDIOC_S_TUNER, &vt, "VIDIOC_S_TUNER") == 0)
-			printf("GPIO output set to 0x%04X\n", gpio_out);
+		struct ivtv_ioctl_register reg;
+
+		reg.reg = IVTV_REG_GPIO_DIR_OFFSET;
+		reg.val = gpio_dir;
+		if (gpio_set_dir && doioctl(fd, IVTV_IOC_S_ITVC_REG, &reg,
+			"IVTV_IOC_S_ITVC_REG") == 0)
+			printf("GPIO dir set to 0x%04x\n", reg.val);
+		reg.reg = IVTV_REG_GPIO_OUT_OFFSET;
+		reg.val = gpio_out;
+		if (doioctl(fd, IVTV_IOC_S_ITVC_REG, &reg,
+			"IVTV_IOC_S_ITVC_REG") == 0)
+			printf("GPIO out set to 0x%04x\n", reg.val);
 	}
 
-	if (options[OptInputDetect] && !(info.hw_flags & (IVTV_HW_SAA7114|IVTV_HW_SAA7115))) {
-		puts("Option --input-detect is only valid for SAA7114/SAA7115 based cards\n");
-	}
-	else if (options[OptInputDetect]) {
-		int reg1e, reg1f;
-		int err = 0;
+	if (options[OptListGPIO]) {
+		struct ivtv_ioctl_register reg;
 
-		puts("check SAA7115 input signal");
-		// Reading SAA7115 status regs and decode them
-		saa7115_reg.reg = (unsigned char)0x1e;
-		if (ioctl(fd, IVTV_IOC_G_DECODER_REG, &saa7115_reg) < 0) {
-			fprintf(stderr,
-				"ioctl: Input detect failed (cannot read SAA regs)\n");
-			err = 1;
-		}
-		reg1e = saa7115_reg.val;
-		saa7115_reg.reg = (unsigned char)0x1f;
-		if (ioctl(fd, IVTV_IOC_G_DECODER_REG, &saa7115_reg) < 0) {
-			fprintf(stderr,
-				"ioctl: Input detect failed (cannot read SAA regs)\n");
-			err = 1;
-		}
-		reg1f = saa7115_reg.val;
-
-		if (err == 0) {
-			int horizFreq = !(reg1e & 0x40),
-			    fieldLength = (reg1e & 0x80),
-			    readyForCapture = (reg1f & 0x01),
-			    hvLoops = !(reg1f & 0x40),
-			    interlace = (reg1f & 0x80),
-			    signalOk = horizFreq &&
-				    fieldLength &&
-				    readyForCapture &&
-				    hvLoops && interlace, vcr = !(reg1f & 0x10);
-
-			printf("ioctl: VIDEO_STATUS = ");
-			if (signalOk) {
-				printf("%s\n", vcr ? "Vcr" : "Sat/Dvd");
-			} else {
-				printf("Bad\n");
-			}
-
-			printf("\nSAA7115 reg[STATUS#1 - 0x1E] = %02x\n", reg1e);
-
-			switch (reg1e & 0x03) {
-				case 1:
-					printf(" Color Standard (D0-D1): NTSC\n");
-					break;
-				case 2:
-					printf(" Color Standard (D0-D1): PAL\n");
-					break;
-				case 3:
-					printf(" Color Standard (D0-D1): SECAM\n");
-					break;
-				default:
-					printf(" Color Standard (D0-D1): BW/No color\n");
-			}
-
-			printf(" White peak loop (D2): %s\n",
-					(reg1e & 0x04) ? "Activated" : "Not activated");
-			printf(" Gain value for act.lum. (min) (D3): %s\n",
-					(reg1e & 0x08) ? "Limited" : "Not limited");
-			printf(" Gain value for act.lum. (max) (D4): %s\n",
-					(reg1e & 0x10) ? "Limited" : "Not limited");
-			printf(" WIPA-mode slow-time constant (D5): %s\n",
-					(reg1e & 0x20) ? "Activated" : "Not activated");
-			printf(" Horizontal frequency (D6): %s\n",
-					(reg1e & 0x40) ? "Unlocked" : "Locked");
-			printf(" Field length (D7): %s\n",
-					(reg1e & 0x80) ? "Standard" : "Non-standard");
-
-			printf("\nSAA7115 reg[STATUS#2 - 0x1F] = %02x\n", reg1f);
-
-			printf(" Ready for capture (D0): %s\n",
-					(reg1f & 0x01) ? "Yes" : "No");
-			printf(" Macrovision 7.01 (D1): %s\n",
-					(reg1f & 0x02) ? "Yes" : "No");
-			printf(" MV Colorstripe burst (D2): %s\n",
-					(reg1f & 0x04) ? "Yes" : "No");
-			printf(" MV Colorstripe burst type 3 (D3): %s\n",
-					(reg1f & 0x08) ? "Yes" : "No");
-			printf(" Input signal timebase (D4): %s\n",
-					(reg1f & 0x10) ? "Stable (broadcast/DVD)" :
-					"Non-stable (VCR)");
-			printf(" Field frequency (D5): %s\n",
-					(reg1f & 0x20) ? "60Hz" : "50Hz");
-			printf(" H & V loops (D6): %s\n",
-					(reg1f & 0x40) ? "Unlocked" : "Locked");
-			printf(" Interlace (D7): %s\n",
-					(reg1f & 0x80) ? "Detected" : "Not detected");
-		}
+		reg.reg = IVTV_REG_GPIO_IN_OFFSET;
+		if (ioctl(fd, IVTV_IOC_G_ITVC_REG, &reg) == 0) 
+			printf("GPIO in:  0x%04x\n", reg.val);
+		reg.reg = IVTV_REG_GPIO_DIR_OFFSET;
+		if (ioctl(fd, IVTV_IOC_G_ITVC_REG, &reg) == 0) 
+			printf("GPIO dir: 0x%04x\n", reg.val);
+		reg.reg = IVTV_REG_GPIO_OUT_OFFSET;
+		if (ioctl(fd, IVTV_IOC_G_ITVC_REG, &reg) == 0) 
+			printf("GPIO out: 0x%04x\n", reg.val);
 	}
 
 	if (options[OptSetDebugLevel]) {
@@ -1763,11 +1750,20 @@ int main(int argc, char **argv)
 		struct v4l2_tuner vt;
 		memset(&vt, 0, sizeof(struct v4l2_tuner));
 		if (doioctl(fd, VIDIOC_G_TUNER, &vt, "VIDIOC_G_TUNER") == 0) {
+			if (vt.capability & V4L2_TUNER_CAP_LOW)
+				printf(" frequency range: %.1f MHz - %.1f MHz\n",
+				     vt.rangelow / 16000.0, vt.rangehigh / 16000.0);
+			else
+				printf(" frequency range: %.1f MHz - %.1f MHz\n",
+				     vt.rangelow / 16.0, vt.rangehigh / 16.0);
+			printf(" signal strength: %d%%\n", (int)(vt.signal / 655.35));
 			printf(" current audio mode   : %d\n", vt.audmode);
-			printf("   (stereo %d, lang1 %d, lang2 %d, mono %d)\n",
+			printf("   (stereo %d, lang1 %d, lang2 %d, both %d, mono %d)\n",
 		       		V4L2_TUNER_MODE_STEREO,
 		       		V4L2_TUNER_MODE_LANG1,
-		       		V4L2_TUNER_MODE_LANG2, V4L2_TUNER_MODE_MONO);
+		       		V4L2_TUNER_MODE_LANG2,
+		       		V4L2_TUNER_MODE_LANG1_LANG2,
+			       	V4L2_TUNER_MODE_MONO);
 			printf(" available subchannels: 0x%x\n", vt.rxsubchans);
 			printf("   (stereo 0x%x, lang1 0x%x, lang2 0x%x, mono 0x%x)\n",
 		       		V4L2_TUNER_SUB_STEREO,
